@@ -23,28 +23,23 @@ OIIO_PLUGIN_NAMESPACE_BEGIN
 class DPXInput final : public ImageInput {
 public:
     DPXInput() { init(); }
-    virtual ~DPXInput() { close(); }
-    virtual const char* format_name(void) const override { return "dpx"; }
-    virtual int supports(string_view feature) const override
+    ~DPXInput() override { close(); }
+    const char* format_name(void) const override { return "dpx"; }
+    int supports(string_view feature) const override
     {
         return (feature == "ioproxy");
     }
-    virtual bool valid_file(const std::string& filename) const override;
-    virtual bool open(const std::string& name, ImageSpec& newspec) override;
-    virtual bool open(const std::string& name, ImageSpec& newspec,
-                      const ImageSpec& config) override;
-    virtual bool close() override;
-    virtual int current_subimage(void) const override { return m_subimage; }
-    virtual bool seek_subimage(int subimage, int miplevel) override;
-    virtual bool read_native_scanline(int subimage, int miplevel, int y, int z,
-                                      void* data) override;
-    virtual bool read_native_scanlines(int subimage, int miplevel, int ybegin,
-                                       int yend, int z, void* data) override;
-    virtual bool set_ioproxy(Filesystem::IOProxy* ioproxy) override
-    {
-        m_io = ioproxy;
-        return true;
-    }
+    bool valid_file(const std::string& filename) const override;
+    bool open(const std::string& name, ImageSpec& newspec) override;
+    bool open(const std::string& name, ImageSpec& newspec,
+              const ImageSpec& config) override;
+    bool close() override;
+    int current_subimage(void) const override { return m_subimage; }
+    bool seek_subimage(int subimage, int miplevel) override;
+    bool read_native_scanline(int subimage, int miplevel, int y, int z,
+                              void* data) override;
+    bool read_native_scanlines(int subimage, int miplevel, int ybegin, int yend,
+                               int z, void* data) override;
 
 private:
     int m_subimage;
@@ -53,9 +48,6 @@ private:
     std::vector<unsigned char> m_userBuf;
     bool m_rawcolor;
     std::vector<unsigned char> m_decodebuf;  // temporary decode buffer
-    std::unique_ptr<Filesystem::IOProxy> m_io_local;
-    Filesystem::IOProxy* m_io = nullptr;
-    int64_t m_io_offset       = 0;
 
     /// Reset everything to initial state
     ///
@@ -69,7 +61,7 @@ private:
         }
         m_userBuf.clear();
         m_rawcolor = false;
-        m_io       = nullptr;
+        ioproxy_clear();
     }
 
     /// Helper function - retrieve string for libdpx characteristic
@@ -91,7 +83,7 @@ private:
 
 
 
-// Obligatory material to make this a recognizeable imageio plugin:
+// Obligatory material to make this a recognizable imageio plugin:
 OIIO_PLUGIN_EXPORTS_BEGIN
 
 OIIO_EXPORT ImageInput*
@@ -137,18 +129,10 @@ DPXInput::valid_file(const std::string& filename) const
 bool
 DPXInput::open(const std::string& name, ImageSpec& newspec)
 {
-    if (!m_io) {
-        // If no proxy was supplied, create a file reader
-        m_io = new Filesystem::IOFile(name, Filesystem::IOProxy::Mode::Read);
-        m_io_local.reset(m_io);
-    }
-    if (!m_io || m_io->mode() != Filesystem::IOProxy::Mode::Read) {
-        errorf("Could not open file \"%s\"", name);
+    if (!ioproxy_use_or_open(name))
         return false;
-    }
-    m_io_offset = m_io->tell();
 
-    m_stream = new InStream(m_io);
+    m_stream = new InStream(ioproxy());
     if (!m_stream) {
         errorf("Could not open file \"%s\"", name);
         return false;
@@ -179,9 +163,7 @@ DPXInput::open(const std::string& name, ImageSpec& newspec,
     m_rawcolor = config.get_int_attribute("dpx:RawColor")
                  || config.get_int_attribute("dpx:RawData")  // deprecated
                  || config.get_int_attribute("oiio:RawColor");
-    auto ioparam = config.find_attribute("oiio:ioproxy", TypeDesc::PTR);
-    if (ioparam)
-        m_io = ioparam->get<Filesystem::IOProxy*>();
+    ioproxy_retrieve_from_config(config);
     return open(name, newspec);
 }
 
@@ -332,7 +314,7 @@ DPXInput::seek_subimage(int subimage, int miplevel)
         if (!std::isnan(m_dpx.header.Gamma()) && m_dpx.header.Gamma() != 0) {
             float g = float(m_dpx.header.Gamma());
             m_spec.attribute("oiio:ColorSpace",
-                             Strutil::sprintf("GammaCorrected%.2g", g));
+                             Strutil::sprintf("Gamma%.2g", g));
             m_spec.attribute("oiio:Gamma", g);
             break;
         }
@@ -378,10 +360,12 @@ DPXInput::seek_subimage(int subimage, int miplevel)
     }
     if (m_dpx.header.ImageEncoding(subimage) == dpx::kRLE)
         m_spec.attribute("compression", "rle");
-    char buf[32 + 1];
-    m_dpx.header.Description(subimage, buf);
-    if (buf[0] && buf[0] != char(-1))
-        m_spec.attribute("ImageDescription", buf);
+    {
+        char desc[32 + 1];
+        m_dpx.header.Description(subimage, desc);
+        if (desc[0] && desc[0] != char(-1))
+            m_spec.attribute("ImageDescription", desc);
+    }
     m_spec.attribute("PixelAspectRatio",
                      m_dpx.header.AspectRatio(1)
                          ? (m_dpx.header.AspectRatio(0)
@@ -482,13 +466,13 @@ DPXInput::seek_subimage(int subimage, int miplevel)
                                      m_dpx.header.userBits };
         m_spec.attribute("smpte:TimeCode", TypeTimeCode, timecode);
 
-        // This attribute is dpx specific and is left in for backwards compatability.
+        // This attribute is dpx specific and is left in for backwards compatibility.
         // Users should utilise the new smpte:TimeCode attribute instead
         Imf::TimeCode tc(m_dpx.header.timeCode, m_dpx.header.userBits);
         m_spec.attribute("dpx:TimeCode", get_timecode_string(tc));
     }
 
-    // This attribute is dpx specific and is left in for backwards compatability.
+    // This attribute is dpx specific and is left in for backwards compatibility.
     // Users should utilise the new smpte:TimeCode attribute instead
     if (m_dpx.header.userBits != 0xFFFFFFFF)
         m_spec.attribute("dpx:UserBits", m_dpx.header.userBits);
@@ -502,9 +486,12 @@ DPXInput::seek_subimage(int subimage, int miplevel)
         date[19] = 0;
         m_spec.attribute("dpx:SourceDateTime", date);
     }
-    m_dpx.header.FilmEdgeCode(buf);
-    if (buf[0])
-        m_spec.attribute("dpx:FilmEdgeCode", buf);
+    {
+        char filmedge[17];
+        m_dpx.header.FilmEdgeCode(filmedge);
+        if (filmedge[0])
+            m_spec.attribute("dpx:FilmEdgeCode", filmedge);
+    }
 
     tmpstr.clear();
     switch (m_dpx.header.Signal()) {
@@ -576,18 +563,6 @@ DPXInput::seek_subimage(int subimage, int miplevel)
 bool
 DPXInput::close()
 {
-    if (m_io_local) {
-        // If we allocated our own ioproxy, close it.
-        m_io_local.reset();
-        m_io = nullptr;
-    }
-    // N.B. If we were passed an ioproxy from the user (m_io != nullptr, but
-    // m_io_local was not set), don't actually close it, it belongs to the
-    // caller. And in the case of an ImageCache file, it's possible that the
-    // IC won't close this ImageInput until after the owner of the IOProxy
-    // destroyed it. So don't mess with it here in close() at all, because
-    // we just can't be sure if it's still alive or not.
-
     init();  // Reset to initial state
     return true;
 }
