@@ -24,17 +24,17 @@ using namespace ICO_pvt;
 class ICOOutput final : public ImageOutput {
 public:
     ICOOutput();
-    virtual ~ICOOutput();
-    virtual const char* format_name(void) const override { return "ico"; }
-    virtual int supports(string_view feature) const override;
-    virtual bool open(const std::string& name, const ImageSpec& spec,
-                      OpenMode mode = Create) override;
-    virtual bool close() override;
-    virtual bool write_scanline(int y, int z, TypeDesc format, const void* data,
-                                stride_t xstride) override;
-    virtual bool write_tile(int x, int y, int z, TypeDesc format,
-                            const void* data, stride_t xstride,
-                            stride_t ystride, stride_t zstride) override;
+    ~ICOOutput() override;
+    const char* format_name(void) const override { return "ico"; }
+    int supports(string_view feature) const override;
+    bool open(const std::string& name, const ImageSpec& spec,
+              OpenMode mode = Create) override;
+    bool close() override;
+    bool write_scanline(int y, int z, TypeDesc format, const void* data,
+                        stride_t xstride) override;
+    bool write_tile(int x, int y, int z, TypeDesc format, const void* data,
+                    stride_t xstride, stride_t ystride,
+                    stride_t zstride) override;
 
 private:
     std::string m_filename;                ///< Stash the filename
@@ -94,7 +94,7 @@ private:
 
 
 
-// Obligatory material to make this a recognizeable imageio plugin:
+// Obligatory material to make this a recognizable imageio plugin:
 OIIO_PLUGIN_EXPORTS_BEGIN
 
 OIIO_EXPORT ImageOutput*
@@ -419,7 +419,11 @@ ICOOutput::close()
     }
 
     if (m_png) {
-        PNG_pvt::finish_image(m_png, m_info);
+        PNG_pvt::write_end(m_png, m_info);
+        if (m_png || m_info)
+            PNG_pvt::destroy_write_struct(m_png, m_info);
+        m_png  = nullptr;
+        m_info = nullptr;
     }
     fclose(m_file);
     m_file = NULL;
