@@ -48,6 +48,9 @@
 #    ifndef NOMINMAX
 #        define NOMINMAX
 #    endif
+#    ifndef NOGDI
+#        define NOGDI
+#    endif
 #    include <windows.h>
 #endif
 
@@ -55,6 +58,11 @@
 #    include <intrin.h>
 #endif
 
+//Avoid min and max being defined
+#if defined(_WIN32)
+#define NOMINMAX
+#endif
+
 #include <OpenImageIO/oiioversion.h>
 #include <OpenImageIO/export.h>
 
@@ -68,7 +76,7 @@
 //                    nothing (this is useful for things that can only be
 //                    constexpr for particular versions or greater).
 // OIIO_INLINE_CONSTEXPR : inline constexpr variables, added in C++17. For
-//                         older C++, just constexpr.
+//                         older C++, static constexpr.
 //
 // Note: oiioversion.h defines OIIO_BUILD_CPP (set to 14, 17, etc.)
 // reflecting what OIIO itself was *built* with.  In contrast,
@@ -92,7 +100,7 @@
 #    define OIIO_CPLUSPLUS_VERSION 14
 #    define OIIO_CONSTEXPR17 /* not constexpr before C++17 */
 #    define OIIO_CONSTEXPR20 /* not constexpr before C++20 */
-#    define OIIO_INLINE_CONSTEXPR constexpr
+#    define OIIO_INLINE_CONSTEXPR static constexpr
 #else
 #    error "This version of OIIO is meant to work only with C++14 and above"
 #endif
@@ -137,7 +145,11 @@
 //   __INTEL_LLVM_COMPILER is defined only for icx
 //   _MSC_VER is defined for MSVS compiler (not gcc/clang/icc even on Windows)
 //   _WIN32 is defined on Windows regardless of compiler
-//   __CUDACC__ is defined for nvcc and clang during Cuda compilation.
+//   __CUDACC__   is defined any time we are compiling a module for Cuda
+//                (both for the host pass and the device pass). "Do this
+//                when using nvcc or clang with ptx target."
+//   __CUDA_ARCH__  is only defined when doing the device pass. "Do this only
+//                for code that will actually run on the GPU."
 
 
 // Define OIIO_GNUC_VERSION to hold an encoded gcc version (e.g. 40802 for
@@ -399,7 +411,7 @@
 
 // OIIO_PURE_FUNC is a function attribute that assures the compiler that the
 // function does not write to any non-local memory other than its return
-// value and has no side effects. This can ebable additional compiler
+// value and has no side effects. This can enable additional compiler
 // optimizations by knowing that calling the function cannot possibly alter
 // any other memory. This declaration goes after the function declaration:
 //   int blah (int arg) OIIO_PURE_FUNC;
@@ -485,7 +497,7 @@
 // false positives that you can't easily get rid of.
 // This should work for any clang >= 3.3 and gcc >= 4.8, which are
 // guaranteed by our minimum requirements.
-#if defined(__clang__) || (defined(__GNUC__) && !defined(__INTEL_COMPILER)) \
+#if defined(__clang__) || (OIIO_GNUC_VERSION > 90000 && !defined(__INTEL_COMPILER)) \
                        || __has_attribute(no_sanitize_address)
 #    define OIIO_NO_SANITIZE_ADDRESS __attribute__((no_sanitize_address))
 #else
@@ -493,6 +505,17 @@
 #endif
 
 
+// OIIO_NO_SANITIZE_UNDEFINED can be used to mark a function that you don't
+// want undefined behavior sanitizer to catch. Only use this if you know there
+// are false positives that you can't easily get rid of.
+#if defined(__clang__) || (OIIO_GNUC_VERSION > 90000 && !defined(__INTEL_COMPILER)) \
+                       || __has_attribute(no_sanitize)
+#    define OIIO_NO_SANITIZE_UNDEFINED __attribute__((no_sanitize("undefined")))
+#else
+#    define OIIO_NO_SANITIZE_UNDEFINED
+#endif
+
+
 // OIIO_RETURNS_NONNULL following a function declaration of a function
 // indicates that the pointer returned by the function is guaranteed to
 // never be nullptr.
@@ -566,7 +589,7 @@ inline void cpuid (int info[4], int infoType, int extra)
 {
     // Implementation cribbed from Halide (http://halide-lang.org), which
     // cribbed it from ISPC (https://github.com/ispc/ispc).
-#if (defined(_WIN32) || defined(__i386__) || defined(__x86_64__))
+#if (defined(_M_X64) || defined(_M_IX86) || defined(_M_AMD64) || defined(__i386__) || defined(__x86_64__))
 # ifdef _MSC_VER
     __cpuidex(info, infoType, extra);
 # elif defined(__x86_64__)
