@@ -14,14 +14,27 @@
 #include <OpenImageIO/simd.h>
 #include <OpenImageIO/ustring.h>
 #include <OpenImageIO/varyingref.h>
+#include <OpenImageIO/vecparam.h>
 
 
 // Define symbols that let client applications determine if newly added
 // features are supported.
 #define OIIO_TEXTURESYSTEM_SUPPORTS_CLOSE 1
 
+// Is the getattributetype() method present? (Added in 2.5)
+#define OIIO_TEXTURESYSTEM_SUPPORTS_GETATTRIBUTETYPE 1
+
 #define OIIO_TEXTURESYSTEM_SUPPORTS_STOCHASTIC 1
 
+#ifndef INCLUDED_IMATHVEC_H
+// Placeholder declaration for Imath::V3f if no Imath headers have been
+// included.
+namespace Imath {
+template <class T> class Vec3;
+using V3f = Vec3<float>;
+}
+#endif
+
 
 OIIO_NAMESPACE_BEGIN
 
@@ -95,8 +108,8 @@ enum class MipMode {
     OneLevel,   ///< Use just one mipmap level
     Trilinear,  ///< Use two MIPmap levels (trilinear)
     Aniso,      ///< Use two MIPmap levels w/ anisotropic
-    StochasticTrilinear, ///< Stochastic trilinear
-    StochasticAniso, ///< Stochastic anisotropic
+    StochasticTrilinear, ///< DEPRECATED Stochastic trilinear
+    StochasticAniso, ///< DEPRECATED Stochastic anisotropic
 };
 
 /// Interp mode determines how we sample within a mipmap level
@@ -118,8 +131,8 @@ enum class InterpMode {
 /// The SIMD width for batched texturing operations. This is fixed within
 /// any release of OpenImageIO, but may change from release to release and
 /// also may be overridden at build time. A typical batch size is 16.
-static constexpr int BatchWidth = OIIO_TEXTURE_SIMD_BATCH_WIDTH;
-static constexpr int BatchAlign = BatchWidth * sizeof(float);
+OIIO_INLINE_CONSTEXPR int BatchWidth = OIIO_TEXTURE_SIMD_BATCH_WIDTH;
+OIIO_INLINE_CONSTEXPR int BatchAlign = BatchWidth * sizeof(float);
 
 /// A type alias for a SIMD vector of floats with the batch width.
 typedef simd::VecType<float, OIIO_TEXTURE_SIMD_BATCH_WIDTH>::type FloatWide;
@@ -138,15 +151,15 @@ typedef uint64_t RunMask;
 // The defined constant `RunMaskOn` contains the value with all bits
 // `0..BatchWidth-1` set to 1.
 #if OIIO_TEXTURE_SIMD_BATCH_WIDTH == 4
-static constexpr RunMask RunMaskOn = 0xf;
+OIIO_INLINE_CONSTEXPR RunMask RunMaskOn = 0xf;
 #elif OIIO_TEXTURE_SIMD_BATCH_WIDTH == 8
-static constexpr RunMask RunMaskOn = 0xff;
+OIIO_INLINE_CONSTEXPR RunMask RunMaskOn = 0xff;
 #elif OIIO_TEXTURE_SIMD_BATCH_WIDTH == 16
-static constexpr RunMask RunMaskOn = 0xffff;
+OIIO_INLINE_CONSTEXPR RunMask RunMaskOn = 0xffff;
 #elif OIIO_TEXTURE_SIMD_BATCH_WIDTH == 32
-static constexpr RunMask RunMaskOn = 0xffffffff;
+OIIO_INLINE_CONSTEXPR RunMask RunMaskOn = 0xffffffff;
 #elif OIIO_TEXTURE_SIMD_BATCH_WIDTH == 64
-static constexpr RunMask RunMaskOn = 0xffffffffffffffffULL;
+OIIO_INLINE_CONSTEXPR RunMask RunMaskOn = 0xffffffffffffffffULL;
 #else
 #    error "Not a valid OIIO_TEXTURE_SIMD_BATCH_WIDTH choice"
 #endif
@@ -194,8 +207,8 @@ public:
         MipModeOneLevel,   ///< Use just one mipmap level
         MipModeTrilinear,  ///< Use two MIPmap levels (trilinear)
         MipModeAniso,      ///< Use two MIPmap levels w/ anisotropic
-        MipModeStochasticTrilinear, ///< Stochastic trilinear
-        MipModeStochasticAniso, ///< Stochastic anisotropic
+        MipModeStochasticTrilinear, ///< DEPRECATED Stochastic trilinear
+        MipModeStochasticAniso, ///< DEPRECATED Stochastic anisotropic
     };
 
     /// Interp mode determines how we sample within a mipmap level
@@ -217,7 +230,7 @@ public:
         anisotropic(32), conservative_filter(true),
         sblur(0.0f), tblur(0.0f), swidth(1.0f), twidth(1.0f),
         fill(0.0f), missingcolor(nullptr),
-        time(0.0f), rnd(0.0f), samples(1),
+        time(0.0f), rnd(-1.0f), samples(1),
         rwrap(WrapDefault), rblur(0.0f), rwidth(1.0f),
         envlayout(0)
     { }
@@ -241,7 +254,7 @@ public:
     const float* missingcolor;  ///< Color for missing texture
     float time;                 ///< Time (for time-dependent texture lookups)
     union {
-        float bias;                 ///< Bias for shadows (DEPRECATED?)
+        float bias;                 ///< Bias for shadows (DEPRECATED)
         float rnd;                  ///< Stratified sample value
     };
     int samples;                ///< Number of samples for shadows
@@ -287,7 +300,10 @@ class OIIO_API TextureOptBatch {
 public:
     /// Create a TextureOptBatch with all fields initialized to reasonable
     /// defaults.
-    TextureOptBatch () {}   // use inline initializers
+    TextureOptBatch () {
+        for (int i = 0; i < Tex::BatchWidth; ++i)
+            rnd[i] = -1.0f;
+    }
 
     // Options that may be different for each point we're texturing
     alignas(Tex::BatchAlign) float sblur[Tex::BatchWidth];    ///< Blur amount
@@ -582,6 +598,11 @@ public:
     /// - `int flip_t` :
     ///             If nonzero, `t` coordinates will be flipped `1-t` for
     ///             all texture lookups. The default is 0.
+    /// - `int stochastic` :
+    ///             Bit field determining how to use stochastic sampling for
+    ///             MipModeStochasticAniso and/or MipModeStochasticTrilinear.
+    ///             Bit 1 = sample MIP level, bit 2 = sample anisotropy
+    ///             (default=0).
     ///
     /// - `string options`
     ///             This catch-all is simply a comma-separated list of
@@ -697,6 +718,12 @@ public:
     /// as a `std::string`.
     virtual bool getattribute(string_view name, std::string& val) const = 0;
 
+    /// If the named attribute is known, return its data type. If no such
+    /// attribute exists, return `TypeUnknown`.
+    ///
+    /// This was added in version 2.5.
+    virtual TypeDesc getattributetype (string_view name) const = 0;
+
     /// @}
 
     /// @{
@@ -962,8 +989,8 @@ public:
     ///             plugin.
     ///
     virtual bool texture3d (ustring filename, TextureOpt &options,
-                            const Imath::V3f &P, const Imath::V3f &dPdx,
-                            const Imath::V3f &dPdy, const Imath::V3f &dPdz,
+                            V3fParam P, V3fParam dPdx,
+                            V3fParam dPdy, V3fParam dPdz,
                             int nchannels, float *result,
                             float *dresultds=nullptr, float *dresultdt=nullptr,
                             float *dresultdr=nullptr) = 0;
@@ -972,8 +999,8 @@ public:
     /// a texture handle and per-thread info.
     virtual bool texture3d (TextureHandle *texture_handle,
                             Perthread *thread_info, TextureOpt &options,
-                            const Imath::V3f &P, const Imath::V3f &dPdx,
-                            const Imath::V3f &dPdy, const Imath::V3f &dPdz,
+                            V3fParam P, V3fParam dPdx,
+                            V3fParam dPdy, V3fParam dPdz,
                             int nchannels, float *result,
                             float *dresultds=nullptr, float *dresultdt=nullptr,
                             float *dresultdr=nullptr) = 0;
@@ -984,16 +1011,16 @@ public:
     // Return true if the file is found and could be opened by an
     // available ImageIO plugin, otherwise return false.
     virtual bool shadow (ustring filename, TextureOpt &options,
-                         const Imath::V3f &P, const Imath::V3f &dPdx,
-                         const Imath::V3f &dPdy, float *result,
+                         V3fParam P, V3fParam dPdx,
+                         V3fParam dPdy, float *result,
                          float *dresultds=nullptr, float *dresultdt=nullptr) = 0;
 
     // Slightly faster version of texture3d() lookup if the app already
     // has a texture handle and per-thread info.
     virtual bool shadow (TextureHandle *texture_handle, Perthread *thread_info,
                          TextureOpt &options,
-                         const Imath::V3f &P, const Imath::V3f &dPdx,
-                         const Imath::V3f &dPdy, float *result,
+                         V3fParam P, V3fParam dPdx,
+                         V3fParam dPdy, float *result,
                          float *dresultds=nullptr, float *dresultdt=nullptr) = 0;
 
     /// Perform a filtered directional environment map lookup in the
@@ -1053,16 +1080,16 @@ public:
     ///             found or could not be opened by any available ImageIO
     ///             plugin.
     virtual bool environment (ustring filename, TextureOpt &options,
-                              const Imath::V3f &R, const Imath::V3f &dRdx,
-                              const Imath::V3f &dRdy, int nchannels, float *result,
+                              V3fParam R, V3fParam dRdx,
+                              V3fParam dRdy, int nchannels, float *result,
                               float *dresultds=nullptr, float *dresultdt=nullptr) = 0;
 
     /// Slightly faster version of environment() if the app already has a
     /// texture handle and per-thread info.
     virtual bool environment (TextureHandle *texture_handle,
                               Perthread *thread_info, TextureOpt &options,
-                              const Imath::V3f &R, const Imath::V3f &dRdx,
-                              const Imath::V3f &dRdy, int nchannels, float *result,
+                              V3fParam R, V3fParam dRdx,
+                              V3fParam dRdy, int nchannels, float *result,
                               float *dresultds=nullptr, float *dresultdt=nullptr) = 0;
 
     /// @}
