@@ -10,6 +10,8 @@
 
 #include <boost/container/flat_map.hpp>
 
+#include <OpenImageIO/Imath.h>
+
 #include <OpenImageIO/color.h>
 #include <OpenImageIO/filesystem.h>
 #include <OpenImageIO/imagebufalgo.h>
@@ -26,26 +28,29 @@
 #    include <OpenColorIO/OpenColorIO.h>
 #    if OCIO_VERSION_HEX >= MAKE_OCIO_VERSION_HEX(2, 0, 0)
 #        define OCIO_v2 1
-// NOTE: these 3 header files were part of the pre-release OCIO 2.0 but
-// eventually were removed. After OCIO v2 is released and we are fairly
-// confident nobody will encounter them, we can remove these includes
-// entirely.
-#        if __has_include(<OpenColorIO/apphelpers/ColorSpaceHelpers.h>)
-#            include <OpenColorIO/apphelpers/ColorSpaceHelpers.h>
-#        endif
-#        if __has_include(<OpenColorIO/apphelpers/DisplayViewHelpers.h>)
-#            include <OpenColorIO/apphelpers/DisplayViewHelpers.h>
-#        endif
-#        if __has_include(<OpenColorIO/apphelpers/ViewingPipeline.h>)
-#            include <OpenColorIO/apphelpers/ViewingPipeline.h>
-#        endif
+#    endif
+#    if OCIO_VERSION_HEX >= MAKE_OCIO_VERSION_HEX(2, 2, 0)
+#        define OCIO_HAS_BUILTIN_CONFIGS 1
 #    endif
 namespace OCIO = OCIO_NAMESPACE;
+#else
+#    define OCIO_VERSION_HEX 0
 #endif
 
+
 OIIO_NAMESPACE_BEGIN
 
 
+#if 0 /* allow color configuration debugging */
+static bool colordebug = Strutil::stoi(Sysutil::getenv("OIIO_COLOR_DEBUG"));
+#    define DBG(...)    \
+        if (colordebug) \
+        Strutil::print(__VA_ARGS__)
+#else
+#    define DBG(...)
+#endif
+
+
 static std::shared_ptr<ColorConfig> default_colorconfig;  // default color config
 static spin_mutex colorconfig_mutex;
 #ifdef USE_OCIO
@@ -82,42 +87,12 @@ public:
     friend bool operator<(const ColorProcCacheKey& a,
                           const ColorProcCacheKey& b)
     {
-        if (a.hash < b.hash)
-            return true;
-        if (b.hash < a.hash)
-            return false;
-        // They hash the same, so now compare for real. Note that we just to
-        // impose an order, any order -- does not need to be alphabetical --
-        // so we just compare the pointers.
-        if (a.inputColorSpace.c_str() < b.inputColorSpace.c_str())
-            return true;
-        if (b.inputColorSpace.c_str() < a.inputColorSpace.c_str())
-            return false;
-        if (a.outputColorSpace.c_str() < b.outputColorSpace.c_str())
-            return true;
-        if (b.outputColorSpace.c_str() < a.outputColorSpace.c_str())
-            return false;
-        if (a.context_key.c_str() < b.context_key.c_str())
-            return true;
-        if (b.context_key.c_str() < a.context_key.c_str())
-            return false;
-        if (a.looks.c_str() < b.looks.c_str())
-            return true;
-        if (b.looks.c_str() < a.looks.c_str())
-            return false;
-        if (a.display.c_str() < b.display.c_str())
-            return true;
-        if (b.display.c_str() < a.display.c_str())
-            return false;
-        if (a.view.c_str() < b.view.c_str())
-            return true;
-        if (b.view.c_str() < a.view.c_str())
-            return false;
-        if (a.file.c_str() < b.view.c_str())
-            return true;
-        if (b.view.c_str() < a.view.c_str())
-            return false;
-        return int(a.inverse) < int(b.inverse);
+        return std::tie(a.hash, a.inputColorSpace, a.outputColorSpace,
+                        a.context_key, a.context_value, a.looks, a.display,
+                        a.view, a.file, a.inverse)
+               < std::tie(b.hash, b.inputColorSpace, b.outputColorSpace,
+                          b.context_key, b.context_value, b.looks, b.display,
+                          b.view, b.file, b.inverse);
     }
 
     ustring inputColorSpace;
@@ -154,11 +129,7 @@ ColorConfig::supportsOpenColorIO()
 int
 ColorConfig::OpenColorIO_version_hex()
 {
-#ifdef USE_OCIO
     return OCIO_VERSION_HEX;
-#else
-    return 0;
-#endif
 }
 
 
@@ -188,12 +159,13 @@ public:
         // Debugging the cache -- make sure we're creating a small number
         // compared to repeated requests.
         if (colorprocs_requested)
-            Strutil::printf ("ColorConfig::Impl : color procs requested: %d, created: %d\n",
-                             colorprocs_requested, colorprocs_created);
+            DBG("ColorConfig::Impl : color procs requested: {}, created: {}\n",
+                           colorprocs_requested, colorprocs_created);
 #endif
     }
 
     void inventory();
+
     void add(const std::string& name, int index)
     {
         colorspaces.emplace_back(name, index);
@@ -242,11 +214,10 @@ public:
     std::string geterror(bool clear = true)
     {
         std::string err;
+        spin_rw_write_lock lock(m_mutex);
         if (clear) {
-            spin_rw_write_lock lock(m_mutex);
             std::swap(err, m_error);
         } else {
-            spin_rw_read_lock lock(m_mutex);
             err = m_error;
         }
         return err;
@@ -335,13 +306,17 @@ ColorConfig::reset(string_view filename)
     // If no filename was specified, use env $OCIO
     if (filename.empty())
         filename = Sysutil::getenv("OCIO");
-
-    if (filename.size() && !Filesystem::exists(filename)) {
-        getImpl()->error("Requested non-existant OCIO config \"{}\"", filename);
+    if (filename.size() && !Filesystem::exists(filename)
+#    ifdef OCIO_HAS_BUILTIN_CONFIGS
+        && !Strutil::istarts_with(filename, "ocio://")
+#    endif
+    ) {
+        getImpl()->error("Requested non-existent OCIO config \"{}\"", filename);
     } else {
         // Either filename passed, or taken from $OCIO, and it seems to exist
         try {
-            getImpl()->config_ = OCIO::Config::CreateFromFile(filename.c_str());
+            getImpl()->config_ = OCIO::Config::CreateFromFile(
+                std::string(filename).c_str());
             getImpl()->configname(filename);
         } catch (OCIO::Exception& e) {
             getImpl()->error("Error reading OCIO config \"{}\": {}", filename,
@@ -407,7 +382,7 @@ ColorConfig::getColorSpaceFamilyByName(string_view name) const
 #ifdef USE_OCIO
     if (getImpl()->config_) {
         OCIO::ConstColorSpaceRcPtr c = getImpl()->config_->getColorSpace(
-            name.c_str());
+            std::string(name).c_str());
         if (c)
             return c->getFamily();
     }
@@ -494,13 +469,49 @@ ColorConfig::getLookNames() const
 
 
 
+bool
+ColorConfig::isColorSpaceLinear(string_view name) const
+{
+#if OCIO_VERSION_HEX >= MAKE_OCIO_VERSION_HEX(2, 2, 0)
+    if (getImpl()->config_) {
+        return getImpl()->config_->isColorSpaceLinear(
+                   c_str(name), OCIO::REFERENCE_SPACE_SCENE)
+               || getImpl()->config_->isColorSpaceLinear(
+                   c_str(name), OCIO::REFERENCE_SPACE_DISPLAY);
+    }
+#endif
+    return Strutil::iequals(name, "linear")
+           || Strutil::iequals(name, "scene_linear");
+}
+
+
+
+std::vector<std::string>
+ColorConfig::getAliases(string_view color_space) const
+{
+    std::vector<std::string> result;
+#if OCIO_VERSION_HEX >= MAKE_OCIO_VERSION_HEX(2, 0, 0)
+    auto config = getImpl()->config_;
+    if (config) {
+        auto cs = config->getColorSpace(c_str(color_space));
+        if (cs) {
+            for (int i = 0, e = cs->getNumAliases(); i < e; ++i)
+                result.emplace_back(cs->getAlias(i));
+        }
+    }
+#endif
+    return result;
+}
+
+
+
 const char*
 ColorConfig::getColorSpaceNameByRole(string_view role) const
 {
 #ifdef USE_OCIO
     if (getImpl()->config_) {
         OCIO::ConstColorSpaceRcPtr c = getImpl()->config_->getColorSpace(
-            role.c_str());
+            std::string(role).c_str());
         // Catch special case of obvious name synonyms
         if (!c
             && (Strutil::iequals(role, "RGB")
@@ -529,20 +540,22 @@ TypeDesc
 ColorConfig::getColorSpaceDataType(string_view name, int* bits) const
 {
 #ifdef USE_OCIO
-    OCIO::ConstColorSpaceRcPtr c = getImpl()->config_->getColorSpace(
-        name.c_str());
-    if (c) {
-        OCIO::BitDepth b = c->getBitDepth();
-        switch (b) {
-        case OCIO::BIT_DEPTH_UNKNOWN: return TypeDesc::UNKNOWN;
-        case OCIO::BIT_DEPTH_UINT8: *bits = 8; return TypeDesc::UINT8;
-        case OCIO::BIT_DEPTH_UINT10: *bits = 10; return TypeDesc::UINT16;
-        case OCIO::BIT_DEPTH_UINT12: *bits = 12; return TypeDesc::UINT16;
-        case OCIO::BIT_DEPTH_UINT14: *bits = 14; return TypeDesc::UINT16;
-        case OCIO::BIT_DEPTH_UINT16: *bits = 16; return TypeDesc::UINT16;
-        case OCIO::BIT_DEPTH_UINT32: *bits = 32; return TypeDesc::UINT32;
-        case OCIO::BIT_DEPTH_F16: *bits = 16; return TypeDesc::HALF;
-        case OCIO::BIT_DEPTH_F32: *bits = 32; return TypeDesc::FLOAT;
+    if (getImpl()->config_) {
+        OCIO::ConstColorSpaceRcPtr c = getImpl()->config_->getColorSpace(
+            std::string(name).c_str());
+        if (c) {
+            OCIO::BitDepth b = c->getBitDepth();
+            switch (b) {
+            case OCIO::BIT_DEPTH_UNKNOWN: return TypeDesc::UNKNOWN;
+            case OCIO::BIT_DEPTH_UINT8: *bits = 8; return TypeDesc::UINT8;
+            case OCIO::BIT_DEPTH_UINT10: *bits = 10; return TypeDesc::UINT16;
+            case OCIO::BIT_DEPTH_UINT12: *bits = 12; return TypeDesc::UINT16;
+            case OCIO::BIT_DEPTH_UINT14: *bits = 14; return TypeDesc::UINT16;
+            case OCIO::BIT_DEPTH_UINT16: *bits = 16; return TypeDesc::UINT16;
+            case OCIO::BIT_DEPTH_UINT32: *bits = 32; return TypeDesc::UINT32;
+            case OCIO::BIT_DEPTH_F16: *bits = 16; return TypeDesc::HALF;
+            case OCIO::BIT_DEPTH_F32: *bits = 32; return TypeDesc::FLOAT;
+            }
         }
     }
 #endif
@@ -593,7 +606,7 @@ ColorConfig::getNumViews(string_view display) const
     if (display.empty())
         display = getDefaultDisplayName();
     if (getImpl()->config_)
-        return getImpl()->config_->getNumViews(display.c_str());
+        return getImpl()->config_->getNumViews(std::string(display).c_str());
 #endif
     return 0;
 }
@@ -607,7 +620,7 @@ ColorConfig::getViewNameByIndex(string_view display, int index) const
     if (display.empty())
         display = getDefaultDisplayName();
     if (getImpl()->config_)
-        return getImpl()->config_->getView(display.c_str(), index);
+        return getImpl()->config_->getView(std::string(display).c_str(), index);
 #endif
     return NULL;
 }
@@ -644,13 +657,51 @@ ColorConfig::getDefaultViewName(string_view display) const
 {
 #ifdef USE_OCIO
     if (getImpl()->config_)
-        return getImpl()->config_->getDefaultView(display.c_str());
+        return getImpl()->config_->getDefaultView(std::string(display).c_str());
 #endif
     return NULL;
 }
 
 
 
+const char*
+ColorConfig::getDisplayViewColorSpaceName(const std::string& display,
+                                          const std::string& view) const
+{
+#ifdef USE_OCIO
+    if (getImpl()->config_)
+#    if OCIO_VERSION_HEX >= MAKE_OCIO_VERSION_HEX(2, 0, 0)
+        return getImpl()->config_->getDisplayViewColorSpaceName(display.c_str(),
+                                                                view.c_str());
+#    else
+        return getImpl()->config_->getDisplayColorSpaceName(display.c_str(),
+                                                            view.c_str());
+#    endif
+#endif
+    return nullptr;
+}
+
+
+
+const char*
+ColorConfig::getDisplayViewLooks(const std::string& display,
+                                 const std::string& view) const
+{
+#ifdef USE_OCIO
+    if (getImpl()->config_)
+#    if OCIO_VERSION_HEX >= MAKE_OCIO_VERSION_HEX(2, 0, 0)
+        return getImpl()->config_->getDisplayViewLooks(display.c_str(),
+                                                       view.c_str());
+#    else
+        return getImpl()->config_->getDisplayLooks(display.c_str(),
+                                                   view.c_str());
+#    endif
+#endif
+    return nullptr;
+}
+
+
+
 std::string
 ColorConfig::configname() const
 {
@@ -696,16 +747,16 @@ public:
 #    endif
     {
     }
-    virtual ~ColorProcessor_OCIO(void) {}
+    ~ColorProcessor_OCIO() override {}
 
-    virtual bool isNoOp() const { return m_p->isNoOp(); }
-    virtual bool hasChannelCrosstalk() const
+    bool isNoOp() const override { return m_p->isNoOp(); }
+    bool hasChannelCrosstalk() const override
     {
         return m_p->hasChannelCrosstalk();
     }
-    virtual void apply(float* data, int width, int height, int channels,
-                       stride_t chanstride, stride_t xstride,
-                       stride_t ystride) const
+    void apply(float* data, int width, int height, int channels,
+               stride_t chanstride, stride_t xstride,
+               stride_t ystride) const override
     {
 #    if OCIO_VERSION_HEX >= 0x02000000
         OCIO::PackedImageDesc pid(data, width, height, channels,
@@ -734,11 +785,11 @@ class ColorProcessor_sRGB_to_linear final : public ColorProcessor {
 public:
     ColorProcessor_sRGB_to_linear()
         : ColorProcessor() {};
-    ~ColorProcessor_sRGB_to_linear() {};
+    ~ColorProcessor_sRGB_to_linear() override {}
 
-    virtual void apply(float* data, int width, int height, int channels,
-                       stride_t chanstride, stride_t xstride,
-                       stride_t ystride) const
+    void apply(float* data, int width, int height, int channels,
+               stride_t chanstride, stride_t xstride,
+               stride_t ystride) const override
     {
         if (channels > 3)
             channels = 3;
@@ -771,11 +822,11 @@ class ColorProcessor_linear_to_sRGB final : public ColorProcessor {
 public:
     ColorProcessor_linear_to_sRGB()
         : ColorProcessor() {};
-    ~ColorProcessor_linear_to_sRGB() {};
+    ~ColorProcessor_linear_to_sRGB() override {}
 
-    virtual void apply(float* data, int width, int height, int channels,
-                       stride_t chanstride, stride_t xstride,
-                       stride_t ystride) const
+    void apply(float* data, int width, int height, int channels,
+               stride_t chanstride, stride_t xstride,
+               stride_t ystride) const override
     {
         if (channels > 3)
             channels = 3;
@@ -809,11 +860,11 @@ class ColorProcessor_Rec709_to_linear final : public ColorProcessor {
 public:
     ColorProcessor_Rec709_to_linear()
         : ColorProcessor() {};
-    ~ColorProcessor_Rec709_to_linear() {};
+    ~ColorProcessor_Rec709_to_linear() override {}
 
-    virtual void apply(float* data, int width, int height, int channels,
-                       stride_t chanstride, stride_t xstride,
-                       stride_t ystride) const
+    void apply(float* data, int width, int height, int channels,
+               stride_t chanstride, stride_t xstride,
+               stride_t ystride) const override
     {
         if (channels > 3)
             channels = 3;
@@ -834,11 +885,11 @@ class ColorProcessor_linear_to_Rec709 final : public ColorProcessor {
 public:
     ColorProcessor_linear_to_Rec709()
         : ColorProcessor() {};
-    ~ColorProcessor_linear_to_Rec709() {};
+    ~ColorProcessor_linear_to_Rec709() override {}
 
-    virtual void apply(float* data, int width, int height, int channels,
-                       stride_t chanstride, stride_t xstride,
-                       stride_t ystride) const
+    void apply(float* data, int width, int height, int channels,
+               stride_t chanstride, stride_t xstride,
+               stride_t ystride) const override
     {
         if (channels > 3)
             channels = 3;
@@ -861,11 +912,11 @@ public:
     ColorProcessor_gamma(float gamma)
         : ColorProcessor()
         , m_gamma(gamma) {};
-    ~ColorProcessor_gamma() {};
+    ~ColorProcessor_gamma() override {}
 
-    virtual void apply(float* data, int width, int height, int channels,
-                       stride_t chanstride, stride_t xstride,
-                       stride_t ystride) const
+    void apply(float* data, int width, int height, int channels,
+               stride_t chanstride, stride_t xstride,
+               stride_t ystride) const override
     {
         if (channels > 3)
             channels = 3;
@@ -904,10 +955,10 @@ public:
         : ColorProcessor()
     {
     }
-    ~ColorProcessor_Ident() {}
-    virtual void apply(float* /*data*/, int /*width*/, int /*height*/,
-                       int /*channels*/, stride_t /*chanstride*/,
-                       stride_t /*xstride*/, stride_t /*ystride*/) const
+    ~ColorProcessor_Ident() override {}
+    void apply(float* /*data*/, int /*width*/, int /*height*/, int /*channels*/,
+               stride_t /*chanstride*/, stride_t /*xstride*/,
+               stride_t /*ystride*/) const override
     {
     }
 };
@@ -924,11 +975,11 @@ public:
         if (inverse)
             m_M = m_M.inverse();
     }
-    ~ColorProcessor_Matrix() {}
+    ~ColorProcessor_Matrix() override {}
 
-    virtual void apply(float* data, int width, int height, int channels,
-                       stride_t chanstride, stride_t xstride,
-                       stride_t ystride) const
+    void apply(float* data, int width, int height, int channels,
+               stride_t chanstride, stride_t xstride,
+               stride_t ystride) const override
     {
         using namespace simd;
         if (channels == 3 && chanstride == sizeof(float)) {
@@ -1093,18 +1144,18 @@ ColorConfig::createColorProcessor(ustring inputColorSpace,
                     || iequals(inputrole, "linear")
                     || iequals(inputColorSpace, "lnf")
                     || iequals(inputColorSpace, "lnh"))
-                   && istarts_with(outputColorSpace, "GammaCorrected")) {
+                   && istarts_with(outputColorSpace, "Gamma")) {
             string_view gamstr = outputColorSpace;
-            Strutil::parse_prefix(gamstr, "GammaCorrected");
+            Strutil::parse_word(gamstr);
             float g = from_string<float>(gamstr);
             handle  = ColorProcessorHandle(new ColorProcessor_gamma(1.0f / g));
-        } else if (istarts_with(inputColorSpace, "GammaCorrected")
+        } else if (istarts_with(inputColorSpace, "Gamma")
                    && (iequals(outputColorSpace, "linear")
                        || iequals(outputrole, "linear")
                        || iequals(outputColorSpace, "lnf")
                        || iequals(outputColorSpace, "lnh"))) {
             string_view gamstr = inputColorSpace;
-            Strutil::parse_prefix(gamstr, "GammaCorrected");
+            Strutil::parse_word(gamstr);
             float g = from_string<float>(gamstr);
             handle  = ColorProcessorHandle(new ColorProcessor_gamma(g));
         }
@@ -1334,7 +1385,7 @@ ColorConfig::createFileTransform(ustring name, bool inverse) const
         OCIO::ConstProcessorRcPtr p;
         try {
             // Get the processor corresponding to this transform.
-            p = getImpl()->config_->getProcessor(context, transform, dir);
+            p = config->getProcessor(context, transform, dir);
             getImpl()->clear_error();
             handle = ColorProcessorHandle(new ColorProcessor_OCIO(p));
         } catch (OCIO::Exception& e) {
@@ -1352,9 +1403,10 @@ ColorConfig::createFileTransform(ustring name, bool inverse) const
 
 
 ColorProcessorHandle
-ColorConfig::createMatrixTransform(const Imath::M44f& M, bool inverse) const
+ColorConfig::createMatrixTransform(M44fParam M, bool inverse) const
 {
-    return ColorProcessorHandle(new ColorProcessor_Matrix(M, inverse));
+    return ColorProcessorHandle(
+        new ColorProcessor_Matrix(*(const Imath::M44f*)M.data(), inverse));
 }
 
 
@@ -1364,9 +1416,10 @@ ColorConfig::getColorSpaceFromFilepath(string_view str) const
 {
 #if defined(USE_OCIO) && (OCIO_VERSION_HEX >= MAKE_OCIO_VERSION_HEX(2, 1, 0))
     if (getImpl() && getImpl()->config_) {
+        std::string s(str);
         string_view r = getImpl()->config_->getColorSpaceFromFilepath(
-            str.c_str());
-        if (!getImpl()->config_->filepathOnlyMatchesDefaultRule(str.c_str()))
+            s.c_str());
+        if (!getImpl()->config_->filepathOnlyMatchesDefaultRule(s.c_str()))
             return r;
     }
 #endif
@@ -1384,7 +1437,7 @@ ColorConfig::parseColorSpaceFromString(string_view str) const
         return getImpl()->config_->parseColorSpaceFromString(str.c_str());
 #endif
 
-    // Reproduce the logic in OCIO v1 parseColorSpaceFromSring
+    // Reproduce the logic in OCIO v1 parseColorSpaceFromString
 
     if (str.empty())
         return "";
@@ -1402,7 +1455,7 @@ ColorConfig::parseColorSpaceFromString(string_view str) const
     size_t rightMostColorPos = std::string::npos;
     std::string rightMostColorspace;
 
-    // Find the right-most occcurance within the string for each colorspace.
+    // Find the right-most occurrence within the string for each colorspace.
     for (auto&& csname : names) {
         // find right-most extension matched in filename
         size_t pos = Strutil::irfind(str, csname);
@@ -1490,8 +1543,8 @@ ImageBufAlgo::colorconvert(const ImageBuf& src, string_view from,
 
 bool
 ImageBufAlgo::colormatrixtransform(ImageBuf& dst, const ImageBuf& src,
-                                   const Imath::M44f& M, bool unpremult,
-                                   ROI roi, int nthreads)
+                                   M44fParam M, bool unpremult, ROI roi,
+                                   int nthreads)
 {
     pvt::LoggedTimer logtime("IBA::colormatrixtransform");
     ColorProcessorHandle processor;
@@ -1511,7 +1564,7 @@ ImageBufAlgo::colormatrixtransform(ImageBuf& dst, const ImageBuf& src,
 
 
 ImageBuf
-ImageBufAlgo::colormatrixtransform(const ImageBuf& src, const Imath::M44f& M,
+ImageBufAlgo::colormatrixtransform(const ImageBuf& src, M44fParam M,
                                    bool unpremult, ROI roi, int nthreads)
 {
     ImageBuf result;
@@ -1668,7 +1721,7 @@ colorconvert_impl_float_rgba(ImageBuf& R, const ImageBuf& A,
                     }
                 }
                 memcpy(R.pixeladdr(roi.xbegin, j, k), scanline,
-                       width * 4 * sizeof(float));
+                       width * 4 * sizeof(float));  //NOSONAR
             }
         }
     });
