@@ -7,6 +7,8 @@
 #include <cstdlib>
 #include <numeric>
 
+#include <OpenImageIO/half.h>
+
 #include <OpenImageIO/dassert.h>
 #include <OpenImageIO/deepdata.h>
 #include <OpenImageIO/fmath.h>
@@ -1025,7 +1027,7 @@ DeepData::split(int64_t pixel, float depth)
 
 namespace {
 
-// Comparitor functor for depth sorting sample indices of a deep pixel.
+// Comparator functor for depth sorting sample indices of a deep pixel.
 class SampleComparator {
 public:
     SampleComparator(const DeepData& dd, int pixel, int zchan, int zbackchan)
