@@ -10,6 +10,8 @@
 #include <iostream>
 #include <limits>
 
+#include <OpenImageIO/half.h>
+
 #include <OpenImageIO/dassert.h>
 #include <OpenImageIO/hash.h>
 #include <OpenImageIO/imagebuf.h>
@@ -135,7 +137,7 @@ computePixelStats_(const ImageBuf& src, ImageBufAlgo::PixelStats& stats,
     parallel_options opt(nthreads);
     if (src.deep()) {
         parallel_for_chunked(roi.ybegin, roi.yend, 64,
-                             [&](int /*id*/, int64_t ybegin, int64_t yend) {
+                             [&](int64_t ybegin, int64_t yend) {
             ROI subroi(roi.xbegin, roi.xend, ybegin, yend, roi.zbegin,
                        roi.zend, roi.chbegin, roi.chend);
             ImageBufAlgo::PixelStats tmp(nchannels);
@@ -157,7 +159,7 @@ computePixelStats_(const ImageBuf& src, ImageBufAlgo::PixelStats& stats,
 
     } else {  // Non-deep case
         parallel_for_chunked(roi.ybegin, roi.yend, 64,
-                             [&](int /*id*/, int64_t ybegin, int64_t yend) {
+                             [&](int64_t ybegin, int64_t yend) {
             ROI subroi(roi.xbegin, roi.xend, ybegin, yend, roi.zbegin,
                        roi.zend, roi.chbegin, roi.chend);
             ImageBufAlgo::PixelStats tmp(nchannels);
@@ -218,12 +220,13 @@ ImageBufAlgo::computePixelStats(PixelStats& stats, const ImageBuf& src, ROI roi,
 
 
 
-template<class BUFT>
+template<class BUFT, class VALT>
 inline void
-compare_value(ImageBuf::ConstIterator<BUFT, float>& a, int chan, float aval,
-              float bval, ImageBufAlgo::CompareResults& result, float& maxval,
+compare_value(ImageBuf::ConstIterator<BUFT, float>& a, int chan, VALT aval,
+              VALT bval, ImageBufAlgo::CompareResults& result, float& maxval,
               double& batcherror, double& batch_sqrerror, bool& failed,
-              bool& warned, float failthresh, float warnthresh)
+              bool& warned, float failthresh, float warnthresh,
+              float failrelative, float warnrelative)
 {
     if (!isfinite(aval) || !isfinite(bval)) {
         if (isnan(aval) == isnan(bval) && isinf(aval) == isinf(bval))
@@ -238,8 +241,13 @@ compare_value(ImageBuf::ConstIterator<BUFT, float>& a, int chan, float aval,
             return;
         }
     }
-    maxval   = std::max(maxval, std::max(aval, bval));
-    double f = fabs(aval - bval);
+    auto aabs    = std::abs(aval);
+    auto babs    = std::abs(bval);
+    auto meanabs = lerp(aabs, babs, 0.5);
+    auto maxabs  = std::max(aabs, babs);
+    maxval       = std::max(maxval, maxabs);
+    double f     = std::abs(aval - bval);
+    double rel   = meanabs > 0.0 ? f / meanabs : 0.0;
     batcherror += f;
     batch_sqrerror += f * f;
     // We use the awkward '!(a<=threshold)' construct so that we have
@@ -252,11 +260,11 @@ compare_value(ImageBuf::ConstIterator<BUFT, float>& a, int chan, float aval,
         result.maxz     = a.z();
         result.maxc     = chan;
     }
-    if (!warned && !(f <= warnthresh)) {
+    if (!warned && !(f <= warnthresh) && !(rel <= warnrelative)) {
         ++result.nwarn;
         warned = true;
     }
-    if (!failed && !(f <= failthresh)) {
+    if (!failed && !(f <= failthresh) && !(rel <= failrelative)) {
         ++result.nfail;
         failed = true;
     }
@@ -267,8 +275,8 @@ compare_value(ImageBuf::ConstIterator<BUFT, float>& a, int chan, float aval,
 template<class Atype, class Btype>
 static bool
 compare_(const ImageBuf& A, const ImageBuf& B, float failthresh,
-         float warnthresh, ImageBufAlgo::CompareResults& result, ROI roi,
-         int /*nthreads*/)
+         float warnthresh, float failrelative, float warnrelative,
+         ImageBufAlgo::CompareResults& result, ROI roi, int /*nthreads*/)
 {
     imagesize_t npels = roi.npixels();
     imagesize_t nvals = npels * roi.nchannels();
@@ -286,14 +294,14 @@ compare_(const ImageBuf& A, const ImageBuf& B, float failthresh,
     // N.B. [PSNR](https://en.wikipedia.org/wiki/Peak_signal-to-noise_ratio)
     // formula requires the max possible value. We assume a normalized 1.0,
     // but for an HDR image with potentially values > 1.0, there is no true
-    // max value, so we punt and use the highest value found in either
-    // image. The compare_value() function we call on every pixel value will
-    // check and adjust our max as needed.
+    // max value, so we punt and use the highest absolute value found in
+    // either image. The compare_value() function we call on every pixel value
+    // will check and adjust our max as needed.
 
     ImageBuf::ConstIterator<Atype> a(A, roi, ImageBuf::WrapBlack);
     ImageBuf::ConstIterator<Btype> b(B, roi, ImageBuf::WrapBlack);
     bool deep = A.deep();
-    // Break up into batches to reduce cancelation errors as the error
+    // Break up into batches to reduce cancellation errors as the error
     // sums become too much larger than the error for individual pixels.
     const int batchsize = 4096;  // As good a guess as any
     for (; !a.done();) {
@@ -308,7 +316,8 @@ compare_(const ImageBuf& A, const ImageBuf& B, float failthresh,
                         compare_value(a, c, a.deep_value(c, s),
                                       b.deep_value(c, s), result, maxval,
                                       batcherror, batch_sqrerror, failed,
-                                      warned, failthresh, warnthresh);
+                                      warned, failthresh, warnthresh,
+                                      failrelative, warnrelative);
                     }
             }
         } else {  // non-deep
@@ -318,7 +327,8 @@ compare_(const ImageBuf& A, const ImageBuf& B, float failthresh,
                     compare_value(a, c, c < Achannels ? a[c] : 0.0f,
                                   c < Bchannels ? b[c] : 0.0f, result, maxval,
                                   batcherror, batch_sqrerror, failed, warned,
-                                  failthresh, warnthresh);
+                                  failthresh, warnthresh, failrelative,
+                                  warnrelative);
             }
         }
         totalerror += batcherror;
@@ -335,6 +345,17 @@ compare_(const ImageBuf& A, const ImageBuf& B, float failthresh,
 ImageBufAlgo::CompareResults
 ImageBufAlgo::compare(const ImageBuf& A, const ImageBuf& B, float failthresh,
                       float warnthresh, ROI roi, int nthreads)
+{
+    // equivalent to compare with the relative thresholds equal to 0
+    return compare(A, B, failthresh, warnthresh, 0.0f, 0.0f, roi, nthreads);
+}
+
+
+
+ImageBufAlgo::CompareResults
+ImageBufAlgo::compare(const ImageBuf& A, const ImageBuf& B, float failthresh,
+                      float warnthresh, float failrelative, float warnrelative,
+                      ROI roi, int nthreads)
 {
     pvt::LoggedTimer logtimer("IBA::compare");
     ImageBufAlgo::CompareResults result;
@@ -355,7 +376,8 @@ ImageBufAlgo::compare(const ImageBuf& A, const ImageBuf& B, float failthresh,
     bool ok;
     OIIO_DISPATCH_COMMON_TYPES2_CONST(ok, "compare", compare_, A.spec().format,
                                       B.spec().format, A, B, failthresh,
-                                      warnthresh, result, roi, nthreads);
+                                      warnthresh, failrelative, warnrelative,
+                                      result, roi, nthreads);
     // FIXME - The nthreads argument is for symmetry with the rest of
     // ImageBufAlgo and for future expansion. But for right now, we
     // don't actually split by threads.  Maybe later.
@@ -381,18 +403,45 @@ static bool
 isConstantColor_(const ImageBuf& src, float threshold, span<float> color,
                  ROI roi, int nthreads)
 {
-    atomic_int result(true);
-    if (threshold == 0.0f) {
+    // Single flag that will be set to false by any of the threads if they
+    // discover that the image is non-constant, so the others can abort early. It
+    // does NOT need to be atomic, because the timing is not critical and we
+    // don't want to slow things down by locking.
+    bool result = true;
+
+    imagesize_t npixels = roi.npixels();
+    if (npixels == 0) {
+        // Empty ROI? Just fail.
+        return false;
+    }
+
+    // Record the value of the first pixel. That's what we'll compare against.
+    std::vector<T> constval(roi.nchannels());
+    ImageBuf::ConstIterator<T, T> s(src, roi);
+    for (int c = roi.chbegin; c < roi.chend; ++c)
+        constval[c] = s[c];
+
+    if (npixels > 2) {
+        // Just check the second pixel. If it doesn't match the first (which
+        // is a pretty common case for non-constant images), then we're
+        // already done and don't need to even launch the threads that will
+        // traverse the rest of the image to check it.
+        ++s;
+        for (int c = roi.chbegin; c < roi.chend; ++c)
+            if (s[c] != constval[c])
+                return false;
+    }
+
+    if (npixels == 1) {
+        // One pixel? Yes, it's a constant color! Skip the image scan.
+    } else if (threshold == 0.0f) {
         // For 0.0 threshold, use shortcut of avoiding the conversion
         // to float, just compare original type values.
-        std::vector<T> constval(roi.nchannels());
-        ImageBuf::ConstIterator<T, T> s(src, roi);
-        for (int c = roi.chbegin; c < roi.chend; ++c)
-            constval[c] = s[c];
         ImageBufAlgo::parallel_image(roi, nthreads, [&](ROI roi) {
             if (!result)
                 return;  // another parallel bucket already failed, don't bother
-            for (ImageBuf::ConstIterator<T, T> s(src, roi); !s.done(); ++s) {
+            for (ImageBuf::ConstIterator<T, T> s(src, roi); result && !s.done();
+                 ++s) {
                 for (int c = roi.chbegin; c < roi.chend; ++c)
                     if (s[c] != constval[c]) {
                         result = false;
@@ -402,14 +451,11 @@ isConstantColor_(const ImageBuf& src, float threshold, span<float> color,
         });
     } else {
         // Nonzero threshold case
-        std::vector<T> constval(roi.nchannels());
-        ImageBuf::ConstIterator<T> s(src, roi);
-        for (int c = roi.chbegin; c < roi.chend; ++c)
-            constval[c] = s[c];
         ImageBufAlgo::parallel_image(roi, nthreads, [&](ROI roi) {
             if (!result)
                 return;  // another parallel bucket already failed, don't bother
-            for (ImageBuf::ConstIterator<T> s(src, roi); !s.done(); ++s) {
+            for (ImageBuf::ConstIterator<T> s(src, roi); result && !s.done();
+                 ++s) {
                 for (int c = roi.chbegin; c < roi.chend; ++c)
                     if (std::abs(s[c] - constval[c]) > threshold) {
                         result = false;
