@@ -35,6 +35,12 @@ public:
     // Thread-specific error message for this ImageInput.
     thread_specific_ptr<std::string> m_errormessage;
     int m_threads = 0;
+
+    // The IOProxy object we will use for all I/O operations.
+    Filesystem::IOProxy* m_io = nullptr;
+    // The "local" proxy that we will create to use if the user didn't
+    // supply a proxy for us to use.
+    std::unique_ptr<Filesystem::IOProxy> m_io_local;
 };
 
 
@@ -91,6 +97,7 @@ ImageInput::valid_file(const std::string& filename) const
     bool ok = const_cast<ImageInput*>(this)->open(filename, tmpspec);
     if (ok)
         const_cast<ImageInput*>(this)->close();
+    (void)geterror();  // Clear any errors
     return ok;
 }
 
@@ -854,8 +861,9 @@ ImageInput::read_image(TypeDesc format, void* data, stride_t xstride,
                        ProgressCallback progress_callback,
                        void* progress_callback_data)
 {
-    return read_image(0, -1, format, data, xstride, ystride, zstride,
-                      progress_callback, progress_callback_data);
+    return read_image(current_subimage(), current_miplevel(), 0, -1, format,
+                      data, xstride, ystride, zstride, progress_callback,
+                      progress_callback_data);
 }
 
 
@@ -1061,9 +1069,11 @@ ImageInput::append_error(string_view message) const
     OIIO_DASSERT(
         errptr->size() < 1024 * 1024 * 16
         && "Accumulated error messages > 16MB. Try checking return codes!");
-    if (errptr->size() && errptr->back() != '\n')
-        *errptr += '\n';
-    *errptr += message;
+    if (errptr->size() < 1024 * 1024 * 16) {
+        if (errptr->size() && errptr->back() != '\n')
+            *errptr += '\n';
+        *errptr += message;
+    }
 }
 
 
@@ -1131,4 +1141,119 @@ ImageInput::unlock() const
 }
 
 
+Filesystem::IOProxy*
+ImageInput::ioproxy()
+{
+    return m_impl->m_io;
+}
+
+
+
+const Filesystem::IOProxy*
+ImageInput::ioproxy() const
+{
+    return m_impl->m_io;
+}
+
+
+
+bool
+ImageInput::set_ioproxy(Filesystem::IOProxy* ioproxy)
+{
+    m_impl->m_io = ioproxy;
+    return (ioproxy == nullptr || supports("ioproxy"));
+}
+
+
+
+bool
+ImageInput::ioproxy_opened() const
+{
+    Filesystem::IOProxy*& m_io(m_impl->m_io);
+    return m_io != nullptr && m_io->mode() == Filesystem::IOProxy::Read;
+}
+
+
+
+void
+ImageInput::ioproxy_clear()
+{
+    m_impl->m_io = nullptr;
+    m_impl->m_io_local.reset();
+}
+
+
+
+void
+ImageInput::ioproxy_retrieve_from_config(const ImageSpec& config)
+{
+    if (auto p = config.find_attribute("oiio:ioproxy", TypeDesc::PTR))
+        set_ioproxy(p->get<Filesystem::IOProxy*>());
+}
+
+
+
+bool
+ImageInput::ioproxy_use_or_open(string_view name)
+{
+    Filesystem::IOProxy*& m_io(m_impl->m_io);
+    if (!m_io) {
+        // If no proxy was supplied, create an IOFile
+        m_io = new Filesystem::IOFile(name, Filesystem::IOProxy::Mode::Read);
+        m_impl->m_io_local.reset(m_io);
+    }
+    if (!m_io || m_io->mode() != Filesystem::IOProxy::Mode::Read) {
+        errorfmt("Could not open file \"{}\"", name);
+        ioproxy_clear();
+        return false;
+    }
+    return true;
+}
+
+
+
+bool
+ImageInput::ioread(void* buf, size_t itemsize, size_t nitems)
+{
+    Filesystem::IOProxy*& m_io(m_impl->m_io);
+    size_t size = itemsize * nitems;
+    size_t n    = m_io->read(buf, size);
+    if (n != size) {
+        if (size_t(m_io->tell()) >= m_io->size())
+            ImageInput::errorfmt("Read error: hit end of file");
+        else
+            ImageInput::errorfmt(
+                "Read error at position {}, could only read {}/{} bytes {}",
+                m_io->tell() - n, n, size, m_io->error());
+    }
+    return n == size;
+}
+
+
+
+bool
+ImageInput::ioseek(int64_t pos, int origin)
+{
+    Filesystem::IOProxy*& m_io(m_impl->m_io);
+    if (m_io && !m_io->seek(pos, origin)) {
+        errorfmt("Seek error, could not seek from {} to {} (total size {}) {}",
+                 m_io->tell(),
+                 origin == SEEK_SET ? pos
+                                    : (origin == SEEK_CUR ? pos + m_io->tell()
+                                                          : pos + m_io->size()),
+                 m_io->size(), m_io->error());
+        return false;
+    }
+    return true;
+}
+
+
+
+int64_t
+ImageInput::iotell() const
+{
+    return m_impl->m_io->tell();
+}
+
+
 OIIO_NAMESPACE_END
