@@ -5,6 +5,9 @@
 #include <cstdio>
 #include <cstdlib>
 
+#include <OpenImageIO/half.h>
+
+#include <OpenImageIO/color.h>
 #include <OpenImageIO/dassert.h>
 #include <OpenImageIO/fmath.h>
 #include <OpenImageIO/hash.h>
@@ -33,7 +36,7 @@ threads_default()
 
 // Global private data
 namespace pvt {
-recursive_mutex imageio_mutex;
+mutex imageio_mutex;
 atomic_int oiio_threads(threads_default());
 atomic_int oiio_exr_threads(threads_default());
 atomic_int oiio_read_chunk(256);
@@ -41,6 +44,7 @@ atomic_int oiio_try_all_readers(1);
 int openexr_core(0);  // Should we use "Exr core C library"?
 int tiff_half(0);
 int tiff_multithread(1);
+int dds_bc5normal(0);
 int limit_channels(1024);
 int limit_imagesize_MB(32 * 1024);
 ustring font_searchpath;
@@ -50,14 +54,7 @@ std::string input_format_list;   // comma-separated list of readable formats
 std::string output_format_list;  // comma-separated list of writable formats
 std::string extension_list;      // list of all extensions for all formats
 std::string library_list;        // list of all libraries for all formats
-static const char* oiio_debug_env = getenv("OPENIMAGEIO_DEBUG");
-#ifdef NDEBUG
-int oiio_print_debug(oiio_debug_env ? Strutil::stoi(oiio_debug_env) : 0);
-#else
-int oiio_print_debug(oiio_debug_env ? Strutil::stoi(oiio_debug_env) : 1);
-#endif
-int oiio_log_times = Strutil::from_string<int>(
-    Sysutil::getenv("OPENIMAGEIO_LOG_TIMES"));
+int oiio_log_times = Strutil::stoi(Sysutil::getenv("OPENIMAGEIO_LOG_TIMES"));
 std::vector<float> oiio_missingcolor;
 }  // namespace pvt
 
@@ -67,8 +64,7 @@ using namespace pvt;
 namespace {
 // Hidden global OIIO data.
 static spin_mutex attrib_mutex;
-static const int maxthreads  = 256;  // reasonable maximum for sanity check
-static FILE* oiio_debug_file = NULL;
+static const int maxthreads = 512;  // reasonable maximum for sanity check
 
 class TimingLog {
 public:
@@ -199,6 +195,7 @@ oiio_simd_caps()
     if (OIIO_AVX512CD_ENABLED)   caps.emplace_back ("avx512cd");
     if (OIIO_AVX512BW_ENABLED)   caps.emplace_back ("avx512bw");
     if (OIIO_AVX512VL_ENABLED)   caps.emplace_back ("avx512vl");
+    if (OIIO_SIMD_NEON)          caps.emplace_back ("neon");
     if (OIIO_FMA_ENABLED)        caps.emplace_back ("fma");
     if (OIIO_F16C_ENABLED)       caps.emplace_back ("f16c");
     // if (OIIO_POPCOUNT_ENABLED)   caps.emplace_back ("popcnt");
@@ -266,18 +263,7 @@ geterror(bool clear)
 void
 debug(string_view message)
 {
-    recursive_lock_guard lock(pvt::imageio_mutex);
-    if (oiio_print_debug) {
-        if (!oiio_debug_file) {
-            const char* filename = getenv("OPENIMAGEIO_DEBUG_FILE");
-            oiio_debug_file = filename && filename[0] ? fopen(filename, "a")
-                                                      : stderr;
-            OIIO_ASSERT(oiio_debug_file);
-            if (!oiio_debug_file)
-                return;
-        }
-        Strutil::print(oiio_debug_file, "OIIO DEBUG: {}", message);
-    }
+    Strutil::pvt::debug(message);
 }
 
 
@@ -334,6 +320,10 @@ attribute(string_view name, TypeDesc type, const void* val)
         tiff_multithread = *(const int*)val;
         return true;
     }
+    if (name == "dds:bc5normal" && type == TypeInt) {
+        dds_bc5normal = *(const int*)val;
+        return true;
+    }
     if (name == "limits:channels" && type == TypeInt) {
         limit_channels = *(const int*)val;
         return true;
@@ -342,6 +332,10 @@ attribute(string_view name, TypeDesc type, const void* val)
         limit_imagesize_MB = *(const int*)val;
         return true;
     }
+    if (name == "use_tbb" && type == TypeInt) {
+        oiio_use_tbb = *(const int*)val;
+        return true;
+    }
     if (name == "debug" && type == TypeInt) {
         oiio_print_debug = *(const int*)val;
         return true;
@@ -379,6 +373,10 @@ getattribute(string_view name, TypeDesc type, void* val)
         *(int*)val = oiio_threads;
         return true;
     }
+    if (name == "version" && type == TypeString) {
+        *(ustring*)val = ustring(OIIO_VERSION_STRING);
+        return true;
+    }
     spin_lock lock(attrib_mutex);
     if (name == "read_chunk" && type == TypeInt) {
         *(int*)val = oiio_read_chunk;
@@ -434,10 +432,26 @@ getattribute(string_view name, TypeDesc type, void* val)
         *(int*)val = tiff_half;
         return true;
     }
+    if (name == "limits:channels" && type == TypeInt) {
+        *(int*)val = limit_channels;
+        return true;
+    }
+    if (name == "limits:imagesize_MB" && type == TypeInt) {
+        *(int*)val = limit_imagesize_MB;
+        return true;
+    }
     if (name == "tiff:multithread" && type == TypeInt) {
         *(int*)val = tiff_multithread;
         return true;
     }
+    if (name == "dds:bc5normal" && type == TypeInt) {
+        *(int*)val = dds_bc5normal;
+        return true;
+    }
+    if (name == "use_tbb" && type == TypeInt) {
+        *(int*)val = oiio_use_tbb;
+        return true;
+    }
     if (name == "debug" && type == TypeInt) {
         *(int*)val = oiio_print_debug;
         return true;
@@ -482,6 +496,12 @@ getattribute(string_view name, TypeDesc type, void* val)
         *(int*)val = oiio_try_all_readers;
         return true;
     }
+    if (name == "opencolorio_version" && type == TypeString) {
+        int v          = ColorConfig::OpenColorIO_version_hex();
+        *(ustring*)val = ustring::fmtformat("{}.{}.{}", v >> 24,
+                                            (v >> 16) & 0xff, (v >> 8) & 0xff);
+        return true;
+    }
     return false;
 }
 
@@ -780,14 +800,13 @@ parallel_convert_image(int nchannels, int width, int height, int depth,
                            nchannels, width, height);
 
     int blocksize = std::max(1, height / nthreads);
-    parallel_for_chunked(
-        0, height, blocksize, [=](int /*id*/, int64_t ybegin, int64_t yend) {
-            convert_image(nchannels, width, yend - ybegin, depth,
-                          (const char*)src + src_ystride * ybegin, src_type,
-                          src_xstride, src_ystride, src_zstride,
-                          (char*)dst + dst_ystride * ybegin, dst_type,
-                          dst_xstride, dst_ystride, dst_zstride);
-        });
+    parallel_for_chunked(0, height, blocksize, [=](int64_t ybegin, int64_t yend) {
+        convert_image(nchannels, width, yend - ybegin, depth,
+                      (const char*)src + src_ystride * ybegin, src_type,
+                      src_xstride, src_ystride, src_zstride,
+                      (char*)dst + dst_ystride * ybegin, dst_type, dst_xstride,
+                      dst_ystride, dst_zstride);
+    });
     return true;
 }
 
