@@ -11,6 +11,7 @@
 #include <memory>
 #include <sstream>
 
+#include <OpenImageIO/Imath.h>
 #include <OpenImageIO/argparse.h>
 #include <OpenImageIO/color.h>
 #include <OpenImageIO/dassert.h>
@@ -1151,7 +1152,7 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
         }
     }
     stat_readtime += alltime.lap();
-    STATUS(Strutil::fmt::format("read \"{}\"", src->name()), stat_readtime);
+    STATUS(Strutil::sprintf("read \"%s\"", src->name()), stat_readtime);
 
     if (mode == ImageBufAlgo::MakeTxEnvLatlFromLightProbe) {
         ImageSpec newspec = src->spec();
@@ -1192,8 +1193,8 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
 
     if (configspec.get_int_attribute("maketx:cdf")) {
         // Writes Gaussian CDF and Inverse Gaussian CDF as per-channel
-        // metadata. We provide both the inverse transfrom and forward
-        // transfrom, so in theory we're free to change the distribution.
+        // metadata. We provide both the inverse transform and forward
+        // transform, so in theory we're free to change the distribution.
         //
         // References:
         //
@@ -1235,12 +1236,12 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
             }
 
             // Store the inverse CDF as a lookup-table which we'll use to
-            // transform the image data to a Guassian distribution. As
+            // transform the image data to a Gaussian distribution. As
             // mentioned in Burley [2019] we're combining two steps here when
             // using the invCDF lookup table: we first "look up" the image
             // value through its CDF (the normalized histogram) which gives us
             // a uniformly distributed value, which we're then feeding in to
-            // the Gaussian inverse CDF to transfrom the unifrom distribution
+            // the Gaussian inverse CDF to transform the uniform distribution
             // to Gaussian.
             for (uint64_t j = 0; j < bins; j++) {
                 float u = float(hist[j]) / hist[bins - 1];
@@ -1471,7 +1472,7 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
     dstspec.tile_height = configspec.tile_height ? configspec.tile_height : 64;
     dstspec.tile_depth  = configspec.tile_depth ? configspec.tile_depth : 1;
 
-    // Try to force zip (still can be overriden by configspec
+    // Try to force zip (still can be overridden by configspec
     dstspec.attribute("compression", "zip");
     // Always prefer contiguous channels, unless overridden by configspec
     dstspec.attribute("planarconfig", "contig");
@@ -1573,7 +1574,7 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
 
     // Color convert the pixels, if needed, in place.  If a color
     // conversion is required we will promote the src to floating point
-    // (or there wont be enough precision potentially).  Also,
+    // (or there won't be enough precision potentially).  Also,
     // independently color convert the constant color metadata
     std::string colorconfigname = configspec.get_string_attribute(
         "maketx:colorconfig");
@@ -1746,7 +1747,15 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
     std::string desc = dstspec.get_string_attribute("ImageDescription");
     bool updatedDesc = false;
 
-    // Eliminate any SHA-1 or ConstantColor hints in the ImageDescription.
+    // Clear a bunch of special attributes that we don't want to propagate
+    // from an input file to the output file, unless we explicitly set it
+    // farther below.
+    dstspec.erase_attribute("oiio:ConstantColor=");
+    dstspec.erase_attribute("ConstantColor=");
+    dstspec.erase_attribute("oiio:AverageColor=");
+    dstspec.erase_attribute("AverageColor=");
+    dstspec.erase_attribute("oiio:SHA-1=");
+    dstspec.erase_attribute("SHA-1=");
     if (desc.size()) {
         Strutil::excise_string_after_head(desc, "oiio:ConstantColor=");
         Strutil::excise_string_after_head(desc, "ConstantColor=");
@@ -1826,6 +1835,19 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
             outstream << "  AverageColor: " << avgstr << std::endl;
     }
 
+    string_view handed = configspec.get_string_attribute("handed");
+    if (handed == "right" || handed == "left") {
+        if (out->supports("arbitrary_metadata")) {
+            dstspec.attribute("handed", handed);
+        } else {
+            desc += Strutil::fmt::format("{}oiio:handed={}",
+                                         desc.length() ? " " : "", handed);
+            updatedDesc = true;
+        }
+        if (verbose)
+            outstream << "  Handed: " << handed << std::endl;
+    }
+
     if (updatedDesc) {
         dstspec.attribute("ImageDescription", desc);
     }
