@@ -8,6 +8,8 @@
 #include <sstream>
 #include <string>
 
+#include <OpenImageIO/Imath.h>
+
 #include <OpenImageIO/dassert.h>
 #include <OpenImageIO/filter.h>
 #include <OpenImageIO/fmath.h>
@@ -264,11 +266,11 @@ inline void
 vector_to_latlong(const Imath::V3f& R, bool y_is_up, float& s, float& t)
 {
     if (y_is_up) {
-        s = atan2f(-R[0], R[2]) / (2.0f * (float)M_PI) + 0.5f;
-        t = 0.5f - atan2f(R[1], hypotf(R[2], -R[0])) / (float)M_PI;
+        s = atan2f(-R.x, R.z) / (2.0f * (float)M_PI) + 0.5f;
+        t = 0.5f - atan2f(R.y, hypotf(R.z, -R.x)) / (float)M_PI;
     } else {
-        s = atan2f(R[1], R[0]) / (2.0f * (float)M_PI) + 0.5f;
-        t = 0.5f - atan2f(R[2], hypotf(R[0], R[1])) / (float)M_PI;
+        s = atan2f(R.y, R.x) / (2.0f * (float)M_PI) + 0.5f;
+        t = 0.5f - atan2f(R.z, hypotf(R.x, R.y)) / (float)M_PI;
     }
     // learned from experience, beware NaNs
     if (isnan(s))
@@ -281,9 +283,8 @@ vector_to_latlong(const Imath::V3f& R, bool y_is_up, float& s, float& t)
 
 bool
 TextureSystemImpl::environment(ustring filename, TextureOpt& options,
-                               const Imath::V3f& R, const Imath::V3f& dRdx,
-                               const Imath::V3f& dRdy, int nchannels,
-                               float* result, float* dresultds,
+                               V3fParam R, V3fParam dRdx, V3fParam dRdy,
+                               int nchannels, float* result, float* dresultds,
                                float* dresultdt)
 {
     PerThreadInfo* thread_info = m_imagecache->get_perthread_info();
@@ -298,9 +299,8 @@ TextureSystemImpl::environment(ustring filename, TextureOpt& options,
 bool
 TextureSystemImpl::environment(TextureHandle* texture_handle_,
                                Perthread* thread_info_, TextureOpt& options,
-                               const Imath::V3f& _R, const Imath::V3f& _dRdx,
-                               const Imath::V3f& _dRdy, int nchannels,
-                               float* result, float* dresultds,
+                               V3fParam _R, V3fParam _dRdx, V3fParam _dRdy,
+                               int nchannels, float* result, float* dresultds,
                                float* dresultdt)
 {
     // Handle >4 channel lookups by recursion.
@@ -385,11 +385,11 @@ TextureSystemImpl::environment(TextureHandle* texture_handle_,
 
     // Calculate unit-length vectors in the direction of R, R+dRdx, R+dRdy.
     // These define the ellipse we're filtering over.
-    Imath::V3f R = _R;
+    Imath::V3f R = _R.cast<Imath::V3f>();
     R.normalize();  // center
-    Imath::V3f Rx = _R + _dRdx;
+    Imath::V3f Rx = _R.cast<Imath::V3f>() + _dRdx.cast<Imath::V3f>();
     Rx.normalize();  // x axis of the ellipse
-    Imath::V3f Ry = _R + _dRdy;
+    Imath::V3f Ry = _R.cast<Imath::V3f>() + _dRdy.cast<Imath::V3f>();
     Ry.normalize();  // y axis of the ellipse
     // angles formed by the ellipse axes.
     float xfilt_noblur = std::max(safe_acos(R.dot(Rx)), 1e-8f);
@@ -523,11 +523,9 @@ TextureSystemImpl::environment(TextureHandle* texture_handle_,
 
         float levelweight[2] = { 1.0f - levelblend, levelblend };
 
-        int npointson = 0;
         for (int level = 0; level < 2; ++level) {
             if (!levelweight[level])
                 continue;
-            ++npointson;
             int lev = miplevel[level];
             if (options.interpmode == TextureOpt::InterpSmartBicubic) {
                 if (lev == 0
@@ -606,9 +604,7 @@ TextureSystemImpl::environment(TextureHandle* texture_handle,
             opt.tblur  = options.tblur[i];
             opt.swidth = options.swidth[i];
             opt.twidth = options.twidth[i];
-#if OIIO_VERSION_GREATER_EQUAL(2, 4, 0)
-            opt.rnd = options.rnd[i];
-#endif
+            opt.rnd    = options.rnd[i];
             Imath::V3f R_(R[i], R[i + Tex::BatchWidth],
                           R[i + 2 * Tex::BatchWidth]);
             Imath::V3f dRdx_(dRdx[i], dRdx[i + Tex::BatchWidth],
