@@ -8,6 +8,8 @@
 #include <sstream>
 #include <string>
 
+#include <OpenImageIO/Imath.h>
+
 #include <OpenImageIO/dassert.h>
 #include <OpenImageIO/fmath.h>
 #include <OpenImageIO/imagecache.h>
@@ -18,18 +20,14 @@
 #include <OpenImageIO/ustring.h>
 #include <OpenImageIO/varyingref.h>
 
-#include "../field3d.imageio/field3d_backdoor.h"
 #include "imagecache_pvt.h"
 #include "texture_pvt.h"
 
 OIIO_NAMESPACE_BEGIN
 using namespace pvt;
-using namespace f3dpvt;
 
 namespace {  // anonymous
 
-static ustring s_field3d("field3d");
-
 OIIO_FORCEINLINE float
 ushort2float(unsigned short val)
 {
@@ -43,9 +41,8 @@ namespace pvt {  // namespace pvt
 
 
 bool
-TextureSystemImpl::texture3d(ustring filename, TextureOpt& options,
-                             const Imath::V3f& P, const Imath::V3f& dPdx,
-                             const Imath::V3f& dPdy, const Imath::V3f& dPdz,
+TextureSystemImpl::texture3d(ustring filename, TextureOpt& options, V3fParam P,
+                             V3fParam dPdx, V3fParam dPdy, V3fParam dPdz,
                              int nchannels, float* result, float* dresultds,
                              float* dresultdt, float* dresultdr)
 {
@@ -61,10 +58,10 @@ TextureSystemImpl::texture3d(ustring filename, TextureOpt& options,
 bool
 TextureSystemImpl::texture3d(TextureHandle* texture_handle_,
                              Perthread* thread_info_, TextureOpt& options,
-                             const Imath::V3f& P, const Imath::V3f& dPdx,
-                             const Imath::V3f& dPdy, const Imath::V3f& dPdz,
-                             int nchannels, float* result, float* dresultds,
-                             float* dresultdt, float* dresultdr)
+                             V3fParam P, V3fParam dPdx, V3fParam dPdy,
+                             V3fParam dPdz, int nchannels, float* result,
+                             float* dresultds, float* dresultdt,
+                             float* dresultdr)
 {
     // Handle >4 channel lookups by recursion.
     if (nchannels > 4) {
@@ -91,9 +88,7 @@ TextureSystemImpl::texture3d(TextureHandle* texture_handle_,
     }
 
 #if 0
-    // FIXME: currently, no support of actual MIPmapping.  No rush,
-    // since the only volume format we currently support, Field3D,
-    // doesn't support MIPmapping.
+    // FIXME: currently, no support of actual MIPmapping.
     static const texture3d_lookup_prototype lookup_functions[] = {
         // Must be in the same order as Mipmode enum
         &TextureSystemImpl::texture3d_lookup,
@@ -167,21 +162,11 @@ TextureSystemImpl::texture3d(TextureHandle* texture_handle_,
     if (si.Mlocal) {
         // See if there is a world-to-local transform stored in the cache
         // entry. If so, use it to transform the input point.
-        si.Mlocal->multVecMatrix(P, Plocal);
-    } else if (texturefile->fileformat() == s_field3d) {
-        // Field3d is special -- it allows nonlinear or time-varying
-        // transforms procedurally, but we have to use a back door.
-        auto input                   = texturefile->open(thread_info);
-        Field3DInput_Interface* f3di = (Field3DInput_Interface*)input.get();
-        if (!f3di) {
-            error("Unable to open texture \"{}\"", texturefile->filename());
-            return false;
-        }
-        f3di->worldToLocal(P, Plocal, options.time);
+        si.Mlocal->multVecMatrix(P.cast<Imath::V3f>(), Plocal);
     } else {
         // If no world-to-local matrix could be discerned, just use the
         // input point directly.
-        Plocal = P;
+        Plocal = P.cast<Imath::V3f>();
     }
 
     // FIXME: we don't bother with this for dPdx, dPdy, and dPdz only
@@ -319,9 +304,9 @@ TextureSystemImpl::accum3d_sample_closest(
         texturefile.levelinfo(options.subimage, miplevel));
     TypeDesc::BASETYPE pixeltype = texturefile.pixeltype(options.subimage);
     // As passed in, (s,t) map the texture to (0,1).  Remap to texel coords.
-    float s = P[0] * spec.full_width + spec.full_x;
-    float t = P[1] * spec.full_height + spec.full_y;
-    float r = P[2] * spec.full_depth + spec.full_z;
+    float s = P.x * spec.full_width + spec.full_x;
+    float t = P.y * spec.full_height + spec.full_y;
+    float r = P.z * spec.full_depth + spec.full_z;
     int stex, ttex, rtex;       // Texel coordinates
     (void)floorfrac(s, &stex);  // don't need fractional result
     (void)floorfrac(t, &ttex);
@@ -420,9 +405,9 @@ TextureSystemImpl::accum3d_sample_bilinear(
     TypeDesc::BASETYPE pixeltype = texturefile.pixeltype(options.subimage);
     // As passed in, (s,t) map the texture to (0,1).  Remap to texel coords
     // and subtract 0.5 because samples are at texel centers.
-    float s = P[0] * spec.full_width + spec.full_x - 0.5f;
-    float t = P[1] * spec.full_height + spec.full_y - 0.5f;
-    float r = P[2] * spec.full_depth + spec.full_z - 0.5f;
+    float s = P.x * spec.full_width + spec.full_x - 0.5f;
+    float t = P.y * spec.full_height + spec.full_y - 0.5f;
+    float r = P.z * spec.full_depth + spec.full_z - 0.5f;
     int sint, tint, rint;
     float sfrac = floorfrac(s, &sint);
     float tfrac = floorfrac(t, &tint);
