@@ -38,38 +38,42 @@ public:
     typedef ImageCacheFile TextureFile;
 
     TextureSystemImpl(ImageCache* imagecache);
-    virtual ~TextureSystemImpl();
+    ~TextureSystemImpl() override;
 
-    virtual bool attribute(string_view name, TypeDesc type, const void* val);
-    virtual bool attribute(string_view name, int val)
+    bool attribute(string_view name, TypeDesc type, const void* val) override;
+    bool attribute(string_view name, int val) override
     {
         return attribute(name, TypeDesc::INT, &val);
     }
-    virtual bool attribute(string_view name, float val)
+    bool attribute(string_view name, float val) override
     {
         return attribute(name, TypeDesc::FLOAT, &val);
     }
-    virtual bool attribute(string_view name, double val)
+    bool attribute(string_view name, double val) override
     {
         float f = (float)val;
         return attribute(name, TypeDesc::FLOAT, &f);
     }
-    virtual bool attribute(string_view name, string_view val)
+    bool attribute(string_view name, string_view val) override
     {
-        const char* s = val.c_str();
+        std::string valstr(val);
+        const char* s = valstr.c_str();
         return attribute(name, TypeDesc::STRING, &s);
     }
 
-    virtual bool getattribute(string_view name, TypeDesc type, void* val) const;
-    virtual bool getattribute(string_view name, int& val) const
+    TypeDesc getattributetype(string_view name) const override;
+
+    bool getattribute(string_view name, TypeDesc type,
+                      void* val) const override;
+    bool getattribute(string_view name, int& val) const override
     {
         return getattribute(name, TypeDesc::INT, &val);
     }
-    virtual bool getattribute(string_view name, float& val) const
+    bool getattribute(string_view name, float& val) const override
     {
         return getattribute(name, TypeDesc::FLOAT, &val);
     }
-    virtual bool getattribute(string_view name, double& val) const
+    bool getattribute(string_view name, double& val) const override
     {
         float f;
         bool ok = getattribute(name, TypeDesc::FLOAT, &f);
@@ -77,11 +81,11 @@ public:
             val = f;
         return ok;
     }
-    virtual bool getattribute(string_view name, char** val) const
+    bool getattribute(string_view name, char** val) const override
     {
         return getattribute(name, TypeDesc::STRING, val);
     }
-    virtual bool getattribute(string_view name, std::string& val) const
+    bool getattribute(string_view name, std::string& val) const override
     {
         const char* s;
         bool ok = getattribute(name, TypeDesc::STRING, &s);
@@ -94,24 +98,24 @@ public:
     // Retrieve options
     void get_commontoworld(Imath::M44f& result) const { result = m_Mc2w; }
 
-    virtual Perthread* get_perthread_info(Perthread* thread_info = NULL)
+    Perthread* get_perthread_info(Perthread* thread_info = NULL) override
     {
         return (Perthread*)m_imagecache->get_perthread_info(
             (ImageCachePerThreadInfo*)thread_info);
     }
-    virtual Perthread* create_thread_info()
+    Perthread* create_thread_info() override
     {
         OIIO_ASSERT(m_imagecache);
         return (Perthread*)m_imagecache->create_thread_info();
     }
-    virtual void destroy_thread_info(Perthread* threadinfo)
+    void destroy_thread_info(Perthread* threadinfo) override
     {
         OIIO_ASSERT(m_imagecache);
         m_imagecache->destroy_thread_info((ImageCachePerThreadInfo*)threadinfo);
     }
 
-    virtual TextureHandle* get_texture_handle(ustring filename,
-                                              Perthread* thread)
+    TextureHandle* get_texture_handle(ustring filename,
+                                      Perthread* thread) override
     {
         PerThreadInfo* thread_info = thread
                                          ? ((PerThreadInfo*)thread)
@@ -119,243 +123,208 @@ public:
         return (TextureHandle*)find_texturefile(filename, thread_info);
     }
 
-    virtual bool good(TextureHandle* texture_handle)
+    bool good(TextureHandle* texture_handle) override
     {
         return texture_handle && !((TextureFile*)texture_handle)->broken();
     }
 
-    virtual ustring filename_from_handle(TextureHandle* handle)
+    ustring filename_from_handle(TextureHandle* handle) override
     {
         return handle ? ((ImageCache::ImageHandle*)handle)->filename()
                       : ustring();
     }
 
-    virtual bool texture(ustring filename, TextureOpt& options, float s,
-                         float t, float dsdx, float dtdx, float dsdy,
-                         float dtdy, int nchannels, float* result,
-                         float* dresultds = NULL, float* dresultdt = NULL);
-    virtual bool texture(TextureHandle* texture_handle, Perthread* thread_info,
-                         TextureOpt& options, float s, float t, float dsdx,
-                         float dtdx, float dsdy, float dtdy, int nchannels,
-                         float* result, float* dresultds = NULL,
-                         float* dresultdt = NULL);
-    virtual bool texture(ustring filename, TextureOptBatch& options,
-                         Tex::RunMask mask, const float* s, const float* t,
-                         const float* dsdx, const float* dtdx,
-                         const float* dsdy, const float* dtdy, int nchannels,
-                         float* result, float* dresultds = nullptr,
-                         float* dresultdt = nullptr);
-    virtual bool texture(TextureHandle* texture_handle, Perthread* thread_info,
-                         TextureOptBatch& options, Tex::RunMask mask,
-                         const float* s, const float* t, const float* dsdx,
-                         const float* dtdx, const float* dsdy,
-                         const float* dtdy, int nchannels, float* result,
-                         float* dresultds = nullptr,
-                         float* dresultdt = nullptr);
-    virtual bool texture(ustring filename, TextureOptions& options,
-                         Runflag* runflags, int beginactive, int endactive,
-                         VaryingRef<float> s, VaryingRef<float> t,
-                         VaryingRef<float> dsdx, VaryingRef<float> dtdx,
-                         VaryingRef<float> dsdy, VaryingRef<float> dtdy,
-                         int nchannels, float* result, float* dresultds = NULL,
-                         float* dresultdt = NULL);
-    virtual bool texture(TextureHandle* texture_handle, Perthread* thread_info,
-                         TextureOptions& options, Runflag* runflags,
-                         int beginactive, int endactive, VaryingRef<float> s,
-                         VaryingRef<float> t, VaryingRef<float> dsdx,
-                         VaryingRef<float> dtdx, VaryingRef<float> dsdy,
-                         VaryingRef<float> dtdy, int nchannels, float* result,
-                         float* dresultds = NULL, float* dresultdt = NULL);
+    bool texture(ustring filename, TextureOpt& options, float s, float t,
+                 float dsdx, float dtdx, float dsdy, float dtdy, int nchannels,
+                 float* result, float* dresultds = NULL,
+                 float* dresultdt = NULL) override;
+    bool texture(TextureHandle* texture_handle, Perthread* thread_info,
+                 TextureOpt& options, float s, float t, float dsdx, float dtdx,
+                 float dsdy, float dtdy, int nchannels, float* result,
+                 float* dresultds = NULL, float* dresultdt = NULL) override;
+    bool texture(ustring filename, TextureOptBatch& options, Tex::RunMask mask,
+                 const float* s, const float* t, const float* dsdx,
+                 const float* dtdx, const float* dsdy, const float* dtdy,
+                 int nchannels, float* result, float* dresultds = nullptr,
+                 float* dresultdt = nullptr) override;
+    bool texture(TextureHandle* texture_handle, Perthread* thread_info,
+                 TextureOptBatch& options, Tex::RunMask mask, const float* s,
+                 const float* t, const float* dsdx, const float* dtdx,
+                 const float* dsdy, const float* dtdy, int nchannels,
+                 float* result, float* dresultds = nullptr,
+                 float* dresultdt = nullptr) override;
+    bool texture(ustring filename, TextureOptions& options, Runflag* runflags,
+                 int beginactive, int endactive, VaryingRef<float> s,
+                 VaryingRef<float> t, VaryingRef<float> dsdx,
+                 VaryingRef<float> dtdx, VaryingRef<float> dsdy,
+                 VaryingRef<float> dtdy, int nchannels, float* result,
+                 float* dresultds = NULL, float* dresultdt = NULL) override;
+    bool texture(TextureHandle* texture_handle, Perthread* thread_info,
+                 TextureOptions& options, Runflag* runflags, int beginactive,
+                 int endactive, VaryingRef<float> s, VaryingRef<float> t,
+                 VaryingRef<float> dsdx, VaryingRef<float> dtdx,
+                 VaryingRef<float> dsdy, VaryingRef<float> dtdy, int nchannels,
+                 float* result, float* dresultds = NULL,
+                 float* dresultdt = NULL) override;
 
-    virtual bool texture3d(ustring filename, TextureOpt& options,
-                           const Imath::V3f& P, const Imath::V3f& dPdx,
-                           const Imath::V3f& dPdy, const Imath::V3f& dPdz,
-                           int nchannels, float* result,
-                           float* dresultds = NULL, float* dresultdt = NULL,
-                           float* dresultdr = NULL);
-    virtual bool texture3d(TextureHandle* texture_handle,
-                           Perthread* thread_info, TextureOpt& options,
-                           const Imath::V3f& P, const Imath::V3f& dPdx,
-                           const Imath::V3f& dPdy, const Imath::V3f& dPdz,
-                           int nchannels, float* result,
-                           float* dresultds = NULL, float* dresultdt = NULL,
-                           float* dresultdr = NULL);
-    virtual bool texture3d(ustring filename, TextureOptBatch& options,
-                           Tex::RunMask mask, const float* P, const float* dPdx,
-                           const float* dPdy, const float* dPdz, int nchannels,
-                           float* result, float* dresultds = nullptr,
-                           float* dresultdt = nullptr,
-                           float* dresultdr = nullptr);
-    virtual bool texture3d(TextureHandle* texture_handle,
-                           Perthread* thread_info, TextureOptBatch& options,
-                           Tex::RunMask mask, const float* P, const float* dPdx,
-                           const float* dPdy, const float* dPdz, int nchannels,
-                           float* result, float* dresultds = nullptr,
-                           float* dresultdt = nullptr,
-                           float* dresultdr = nullptr);
-    virtual bool texture3d(ustring filename, TextureOptions& options,
-                           Runflag* runflags, int beginactive, int endactive,
-                           VaryingRef<Imath::V3f> P,
-                           VaryingRef<Imath::V3f> dPdx,
-                           VaryingRef<Imath::V3f> dPdy,
-                           VaryingRef<Imath::V3f> dPdz, int nchannels,
-                           float* result, float* dresultds = NULL,
-                           float* dresultdt = NULL, float* dresultdr = NULL);
-    virtual bool texture3d(TextureHandle* texture_handle,
-                           Perthread* thread_info, TextureOptions& options,
-                           Runflag* runflags, int beginactive, int endactive,
-                           VaryingRef<Imath::V3f> P,
-                           VaryingRef<Imath::V3f> dPdx,
-                           VaryingRef<Imath::V3f> dPdy,
-                           VaryingRef<Imath::V3f> dPdz, int nchannels,
-                           float* result, float* dresultds = NULL,
-                           float* dresultdt = NULL, float* dresultdr = NULL);
+    bool texture3d(ustring filename, TextureOpt& options, V3fParam P,
+                   V3fParam dPdx, V3fParam dPdy, V3fParam dPdz, int nchannels,
+                   float* result, float* dresultds = NULL,
+                   float* dresultdt = NULL, float* dresultdr = NULL) override;
+    bool texture3d(TextureHandle* texture_handle, Perthread* thread_info,
+                   TextureOpt& options, V3fParam P, V3fParam dPdx,
+                   V3fParam dPdy, V3fParam dPdz, int nchannels, float* result,
+                   float* dresultds = NULL, float* dresultdt = NULL,
+                   float* dresultdr = NULL) override;
+    bool texture3d(ustring filename, TextureOptBatch& options,
+                   Tex::RunMask mask, const float* P, const float* dPdx,
+                   const float* dPdy, const float* dPdz, int nchannels,
+                   float* result, float* dresultds = nullptr,
+                   float* dresultdt = nullptr,
+                   float* dresultdr = nullptr) override;
+    bool texture3d(TextureHandle* texture_handle, Perthread* thread_info,
+                   TextureOptBatch& options, Tex::RunMask mask, const float* P,
+                   const float* dPdx, const float* dPdy, const float* dPdz,
+                   int nchannels, float* result, float* dresultds = nullptr,
+                   float* dresultdt = nullptr,
+                   float* dresultdr = nullptr) override;
+    bool texture3d(ustring filename, TextureOptions& options, Runflag* runflags,
+                   int beginactive, int endactive, VaryingRef<Imath::V3f> P,
+                   VaryingRef<Imath::V3f> dPdx, VaryingRef<Imath::V3f> dPdy,
+                   VaryingRef<Imath::V3f> dPdz, int nchannels, float* result,
+                   float* dresultds = NULL, float* dresultdt = NULL,
+                   float* dresultdr = NULL) override;
+    bool texture3d(TextureHandle* texture_handle, Perthread* thread_info,
+                   TextureOptions& options, Runflag* runflags, int beginactive,
+                   int endactive, VaryingRef<Imath::V3f> P,
+                   VaryingRef<Imath::V3f> dPdx, VaryingRef<Imath::V3f> dPdy,
+                   VaryingRef<Imath::V3f> dPdz, int nchannels, float* result,
+                   float* dresultds = NULL, float* dresultdt = NULL,
+                   float* dresultdr = NULL) override;
 
-    virtual bool shadow(ustring /*filename*/, TextureOpt& /*options*/,
-                        const Imath::V3f& /*P*/, const Imath::V3f& /*dPdx*/,
-                        const Imath::V3f& /*dPdy*/, float* /*result*/,
-                        float* /*dresultds*/, float* /*dresultdt*/)
+    bool shadow(ustring /*filename*/, TextureOpt& /*options*/, V3fParam /*P*/,
+                V3fParam /*dPdx*/, V3fParam /*dPdy*/, float* /*result*/,
+                float* /*dresultds*/, float* /*dresultdt*/) override
     {
         return false;
     }
-    virtual bool shadow(TextureHandle* /*texture_handle*/,
-                        Perthread* /*thread_info*/, TextureOpt& /*options*/,
-                        const Imath::V3f& /*P*/, const Imath::V3f& /*dPdx*/,
-                        const Imath::V3f& /*dPdy*/, float* /*result*/,
-                        float* /*dresultds*/, float* /*dresultdt*/)
+    bool shadow(TextureHandle* /*texture_handle*/, Perthread* /*thread_info*/,
+                TextureOpt& /*options*/, V3fParam /*P*/, V3fParam /*dPdx*/,
+                V3fParam /*dPdy*/, float* /*result*/, float* /*dresultds*/,
+                float* /*dresultdt*/) override
     {
         return false;
     }
-    virtual bool shadow(ustring /*filename*/, TextureOptBatch& /*options*/,
-                        Tex::RunMask /*mask*/, const float* /*P*/,
-                        const float* /*dPdx*/, const float* /*dPdy*/,
-                        float* /*result*/, float* /*dresultds*/,
-                        float* /*dresultdt*/)
+    bool shadow(ustring /*filename*/, TextureOptBatch& /*options*/,
+                Tex::RunMask /*mask*/, const float* /*P*/,
+                const float* /*dPdx*/, const float* /*dPdy*/, float* /*result*/,
+                float* /*dresultds*/, float* /*dresultdt*/) override
     {
         return false;
     }
-    virtual bool shadow(TextureHandle* /*texture_handle*/,
-                        Perthread* /*thread_info*/,
-                        TextureOptBatch& /*options*/, Tex::RunMask /*mask*/,
-                        const float* /*P*/, const float* /*dPdx*/,
-                        const float* /*dPdy*/, float* /*result*/,
-                        float* /*dresultds*/, float* /*dresultdt*/)
+    bool shadow(TextureHandle* /*texture_handle*/, Perthread* /*thread_info*/,
+                TextureOptBatch& /*options*/, Tex::RunMask /*mask*/,
+                const float* /*P*/, const float* /*dPdx*/,
+                const float* /*dPdy*/, float* /*result*/, float* /*dresultds*/,
+                float* /*dresultdt*/) override
     {
         return false;
     }
-    virtual bool shadow(ustring /*filename*/, TextureOptions& /*options*/,
-                        Runflag* /*runflags*/, int /*beginactive*/,
-                        int /*endactive*/, VaryingRef<Imath::V3f> /*P*/,
-                        VaryingRef<Imath::V3f> /*dPdx*/,
-                        VaryingRef<Imath::V3f> /*dPdy*/, float* /*result*/,
-                        float* /*dresultds*/, float* /*dresultdt*/)
+    bool shadow(ustring /*filename*/, TextureOptions& /*options*/,
+                Runflag* /*runflags*/, int /*beginactive*/, int /*endactive*/,
+                VaryingRef<Imath::V3f> /*P*/, VaryingRef<Imath::V3f> /*dPdx*/,
+                VaryingRef<Imath::V3f> /*dPdy*/, float* /*result*/,
+                float* /*dresultds*/, float* /*dresultdt*/) override
     {
         return false;
     }
-    virtual bool shadow(TextureHandle* /*texture_handle*/,
-                        Perthread* /*thread_info*/, TextureOptions& /*options*/,
-                        Runflag* /*runflags*/, int /*beginactive*/,
-                        int /*endactive*/, VaryingRef<Imath::V3f> /*P*/,
-                        VaryingRef<Imath::V3f> /*dPdx*/,
-                        VaryingRef<Imath::V3f> /*dPdy*/, float* /*result*/,
-                        float* /*dresultds*/, float* /*dresultdt*/)
+    bool shadow(TextureHandle* /*texture_handle*/, Perthread* /*thread_info*/,
+                TextureOptions& /*options*/, Runflag* /*runflags*/,
+                int /*beginactive*/, int /*endactive*/,
+                VaryingRef<Imath::V3f> /*P*/, VaryingRef<Imath::V3f> /*dPdx*/,
+                VaryingRef<Imath::V3f> /*dPdy*/, float* /*result*/,
+                float* /*dresultds*/, float* /*dresultdt*/) override
     {
         return false;
     }
 
 
-    virtual bool environment(ustring filename, TextureOpt& options,
-                             const Imath::V3f& R, const Imath::V3f& dRdx,
-                             const Imath::V3f& dRdy, int nchannels,
-                             float* result, float* dresultds = NULL,
-                             float* dresultdt = NULL);
-    virtual bool environment(TextureHandle* texture_handle,
-                             Perthread* thread_info, TextureOpt& options,
-                             const Imath::V3f& R, const Imath::V3f& dRdx,
-                             const Imath::V3f& dRdy, int nchannels,
-                             float* result, float* dresultds = NULL,
-                             float* dresultdt = NULL);
-    virtual bool environment(ustring filename, TextureOptBatch& options,
-                             Tex::RunMask mask, const float* R,
-                             const float* dRdx, const float* dRdy,
-                             int nchannels, float* result,
-                             float* dresultds = nullptr,
-                             float* dresultdt = nullptr);
-    virtual bool environment(TextureHandle* texture_handle,
-                             Perthread* thread_info, TextureOptBatch& options,
-                             Tex::RunMask mask, const float* R,
-                             const float* dRdx, const float* dRdy,
-                             int nchannels, float* result,
-                             float* dresultds = nullptr,
-                             float* dresultdt = nullptr);
-    virtual bool environment(ustring filename, TextureOptions& options,
-                             Runflag* runflags, int beginactive, int endactive,
-                             VaryingRef<Imath::V3f> R,
-                             VaryingRef<Imath::V3f> dRdx,
-                             VaryingRef<Imath::V3f> dRdy, int nchannels,
-                             float* result, float* dresultds = NULL,
-                             float* dresultdt = NULL);
-    virtual bool environment(TextureHandle* texture_handle,
-                             Perthread* thread_info, TextureOptions& options,
-                             Runflag* runflags, int beginactive, int endactive,
-                             VaryingRef<Imath::V3f> R,
-                             VaryingRef<Imath::V3f> dRdx,
-                             VaryingRef<Imath::V3f> dRdy, int nchannels,
-                             float* result, float* dresultds = NULL,
-                             float* dresultdt = NULL);
+    bool environment(ustring filename, TextureOpt& options, V3fParam R,
+                     V3fParam dRdx, V3fParam dRdy, int nchannels, float* result,
+                     float* dresultds = NULL, float* dresultdt = NULL) override;
+    bool environment(TextureHandle* texture_handle, Perthread* thread_info,
+                     TextureOpt& options, V3fParam R, V3fParam dRdx,
+                     V3fParam dRdy, int nchannels, float* result,
+                     float* dresultds = NULL, float* dresultdt = NULL) override;
+    bool environment(ustring filename, TextureOptBatch& options,
+                     Tex::RunMask mask, const float* R, const float* dRdx,
+                     const float* dRdy, int nchannels, float* result,
+                     float* dresultds = nullptr,
+                     float* dresultdt = nullptr) override;
+    bool environment(TextureHandle* texture_handle, Perthread* thread_info,
+                     TextureOptBatch& options, Tex::RunMask mask,
+                     const float* R, const float* dRdx, const float* dRdy,
+                     int nchannels, float* result, float* dresultds = nullptr,
+                     float* dresultdt = nullptr) override;
+    bool environment(ustring filename, TextureOptions& options,
+                     Runflag* runflags, int beginactive, int endactive,
+                     VaryingRef<Imath::V3f> R, VaryingRef<Imath::V3f> dRdx,
+                     VaryingRef<Imath::V3f> dRdy, int nchannels, float* result,
+                     float* dresultds = NULL, float* dresultdt = NULL) override;
+    bool environment(TextureHandle* texture_handle, Perthread* thread_info,
+                     TextureOptions& options, Runflag* runflags,
+                     int beginactive, int endactive, VaryingRef<Imath::V3f> R,
+                     VaryingRef<Imath::V3f> dRdx, VaryingRef<Imath::V3f> dRdy,
+                     int nchannels, float* result, float* dresultds = NULL,
+                     float* dresultdt = NULL) override;
 
-    virtual std::string resolve_filename(const std::string& filename) const;
+    std::string resolve_filename(const std::string& filename) const override;
 
-    virtual bool get_texture_info(ustring filename, int subimage,
-                                  ustring dataname, TypeDesc datatype,
-                                  void* data);
-    virtual bool get_texture_info(TextureHandle* texture_handle,
-                                  Perthread* thread_info, int subimage,
-                                  ustring dataname, TypeDesc datatype,
-                                  void* data);
+    bool get_texture_info(ustring filename, int subimage, ustring dataname,
+                          TypeDesc datatype, void* data) override;
+    bool get_texture_info(TextureHandle* texture_handle, Perthread* thread_info,
+                          int subimage, ustring dataname, TypeDesc datatype,
+                          void* data) override;
 
-    virtual bool get_imagespec(ustring filename, int subimage, ImageSpec& spec);
-    virtual bool get_imagespec(TextureHandle* texture_handle,
-                               Perthread* thread_info, int subimage,
-                               ImageSpec& spec);
+    bool get_imagespec(ustring filename, int subimage,
+                       ImageSpec& spec) override;
+    bool get_imagespec(TextureHandle* texture_handle, Perthread* thread_info,
+                       int subimage, ImageSpec& spec) override;
 
-    virtual const ImageSpec* imagespec(ustring filename, int subimage = 0);
-    virtual const ImageSpec* imagespec(TextureHandle* texture_handle,
-                                       Perthread* thread_info = NULL,
-                                       int subimage           = 0);
+    const ImageSpec* imagespec(ustring filename, int subimage = 0) override;
+    const ImageSpec* imagespec(TextureHandle* texture_handle,
+                               Perthread* thread_info = NULL,
+                               int subimage           = 0) override;
 
-    virtual bool get_texels(ustring filename, TextureOpt& options, int miplevel,
-                            int xbegin, int xend, int ybegin, int yend,
-                            int zbegin, int zend, int chbegin, int chend,
-                            TypeDesc format, void* result);
-    virtual bool get_texels(TextureHandle* texture_handle,
-                            Perthread* thread_info, TextureOpt& options,
-                            int miplevel, int xbegin, int xend, int ybegin,
-                            int yend, int zbegin, int zend, int chbegin,
-                            int chend, TypeDesc format, void* result);
+    bool get_texels(ustring filename, TextureOpt& options, int miplevel,
+                    int xbegin, int xend, int ybegin, int yend, int zbegin,
+                    int zend, int chbegin, int chend, TypeDesc format,
+                    void* result) override;
+    bool get_texels(TextureHandle* texture_handle, Perthread* thread_info,
+                    TextureOpt& options, int miplevel, int xbegin, int xend,
+                    int ybegin, int yend, int zbegin, int zend, int chbegin,
+                    int chend, TypeDesc format, void* result) override;
 
-    virtual bool is_udim(ustring filename);
-    virtual bool is_udim(TextureHandle* udimfile);
-    virtual TextureHandle* resolve_udim(ustring filename, float s, float t);
-    virtual TextureHandle* resolve_udim(TextureHandle* udimfile,
-                                        Perthread* thread_info, float s,
-                                        float t);
-    virtual void inventory_udim(ustring udimpattern,
-                                std::vector<ustring>& filenames, int& nutiles,
-                                int& nvtiles);
-    virtual void inventory_udim(TextureHandle* udimfile, Perthread* thread_info,
-                                std::vector<ustring>& filenames, int& nutiles,
-                                int& nvtiles);
+    bool is_udim(ustring filename) override;
+    bool is_udim(TextureHandle* udimfile) override;
+    TextureHandle* resolve_udim(ustring filename, float s, float t) override;
+    TextureHandle* resolve_udim(TextureHandle* udimfile, Perthread* thread_info,
+                                float s, float t) override;
+    void inventory_udim(ustring udimpattern, std::vector<ustring>& filenames,
+                        int& nutiles, int& nvtiles) override;
+    void inventory_udim(TextureHandle* udimfile, Perthread* thread_info,
+                        std::vector<ustring>& filenames, int& nutiles,
+                        int& nvtiles) override;
 
-    virtual bool has_error() const;
-    virtual std::string geterror(bool clear = true) const;
-    virtual std::string getstats(int level = 1, bool icstats = true) const;
-    virtual void reset_stats();
+    bool has_error() const override;
+    std::string geterror(bool clear = true) const override;
+    std::string getstats(int level = 1, bool icstats = true) const override;
+    void reset_stats() override;
 
-    virtual void invalidate(ustring filename, bool force);
-    virtual void invalidate_all(bool force = false);
-    virtual void close(ustring filename);
-    virtual void close_all();
+    void invalidate(ustring filename, bool force) override;
+    void invalidate_all(bool force = false) override;
+    void close(ustring filename) override;
+    void close_all() override;
 
     void operator delete(void* todel) { ::delete ((char*)todel); }
 
@@ -363,7 +332,7 @@ public:
 
     /// Return an opaque, non-owning pointer to the underlying ImageCache
     /// (if there is one).
-    virtual ImageCache* imagecache() const { return m_imagecache; }
+    ImageCache* imagecache() const override { return m_imagecache; }
 
 private:
     typedef ImageCacheTileRef TileRef;
@@ -580,8 +549,16 @@ private:
     bool m_flip_t;            ///< Flip direction of t coord?
     int m_max_tile_channels;  ///< narrow tile ID channel range when
                               ///<   the file has more channels
+    int m_stochastic;
     static EightBitConverter<float> uchar2float;
 
+    enum StochasticStrategyBits {
+        StochasticStrategy_None  = 0,
+        StochasticStrategy_MIP   = 1,  // select MIP level
+        StochasticStrategy_Aniso = 2,  // single anisotropic probe
+        StochasticStrategy_Texel = 4   // single FIS texel probe
+    };
+
     /// Saved error string, per-thread
     ///
     mutable thread_specific_ptr<std::string> m_errormessage;
