@@ -8,14 +8,16 @@
 #include <cstdlib>
 #include <fcntl.h>
 #include <iostream>
+#include <random>
 #include <regex>
 #include <string>
 
 #include <OpenImageIO/dassert.h>
 #include <OpenImageIO/filesystem.h>
 #include <OpenImageIO/platform.h>
-#include <OpenImageIO/refcnt.h>
+// #include <OpenImageIO/refcnt.h>
 #include <OpenImageIO/strutil.h>
+#include <OpenImageIO/sysutil.h>
 #include <OpenImageIO/ustring.h>
 
 #ifdef _WIN32
@@ -23,14 +25,23 @@
 #    include <direct.h>
 #    include <io.h>
 #    include <shellapi.h>
+#    include <sys/types.h>
+#    include <sys/utime.h>
 #else
+#    include <sys/stat.h>
+#    include <sys/types.h>
 #    include <unistd.h>
 #endif
 
-#include <boost/filesystem.hpp>
+#if defined(USE_STD_FILESYSTEM)
+#    include <filesystem>
+namespace filesystem = std::filesystem;
+using std::error_code;
+#else
+#    include <boost/filesystem.hpp>
 namespace filesystem = boost::filesystem;
-using error_code     = boost::system::error_code;
-// FIXME: use std::filesystem when available
+using boost::system::error_code;
+#endif
 
 
 
@@ -46,7 +57,7 @@ inline filesystem::path
 u8path(string_view name)
 {
 #ifdef _WIN32
-    return filesystem::path(Strutil::utf8_to_utf16(name));
+    return filesystem::path(Strutil::utf8_to_utf16wstring(name));
 #else
     return filesystem::path(name.begin(), name.end());
 #endif
@@ -56,7 +67,7 @@ inline filesystem::path
 u8path(const std::string& name)
 {
 #ifdef _WIN32
-    return filesystem::path(Strutil::utf8_to_utf16(name));
+    return filesystem::path(Strutil::utf8_to_utf16wstring(name));
 #else
     return filesystem::path(name);
 #endif
@@ -99,6 +110,15 @@ std::string
 Filesystem::extension(string_view filepath, bool include_dot) noexcept
 {
     std::string s;
+#if !defined(USE_STD_FILESYSTEM)
+    if (filepath.find('.') == 0 && filepath.rfind('.') == 0) {
+        // Work around bug in boost::filesystem::path::extension(), where
+        // ".foo" thinks the extension is foo. But we know, and
+        // std::filesystem::path::extension() knows, that a file called ".foo"
+        // has no extension, that's just the base filename.
+        return s;
+    }
+#endif
     try {
         s = pathstr(u8path(filepath).extension());
     } catch (...) {
@@ -147,18 +167,6 @@ Filesystem::generic_filepath(string_view filepath) noexcept
 
 
 
-#if OIIO_VERSION_LESS(2, 4, 0)
-// For older versions, preserve this link-compatible entry point
-void
-Filesystem::searchpath_split(const std::string& searchpath,
-                             std::vector<std::string>& dirs, bool validonly)
-{
-    dirs = searchpath_split(searchpath, validonly);
-}
-#endif
-
-
-
 std::vector<std::string>
 Filesystem::searchpath_split(string_view searchpath, bool validonly)
 {
@@ -223,7 +231,7 @@ Filesystem::searchpath_find(const std::string& filename_utf8,
         const filesystem::path d(u8path(d_utf8));
         filesystem::path f = d / filename;
         error_code ec;
-        if (filesystem::is_regular(f, ec)) {
+        if (filesystem::is_regular_file(f, ec)) {
             return pathstr(f);
         }
 
@@ -249,6 +257,41 @@ Filesystem::searchpath_find(const std::string& filename_utf8,
 
 
 
+std::string
+Filesystem::find_program(string_view program)
+{
+    const filesystem::path filename(u8path(program));
+    bool abs = filename.is_absolute();
+
+    // If it's an absolute path, we are only checking if it's executable
+    if (abs)
+        return Filesystem::is_executable(program) ? std::string(program)
+                                                  : std::string();
+
+    // If it's found without searching, and it's executable, return its
+    // absolute path.
+    if (Filesystem::is_executable(program))
+        return pathstr(filesystem::absolute(filename));
+
+    // Relative filename, not yet found -- try each $PATH directory in turn
+    for (auto&& d_utf8 : searchpath_split(OIIO::Sysutil::getenv("PATH"))) {
+        const filesystem::path d(u8path(d_utf8));
+        filesystem::path f = d / filename;
+        // Strutil::print("\tPath = {}\n", f);
+        auto p = pathstr(filesystem::absolute(f));
+        if (is_executable(p))
+            return p;
+#ifdef _WIN32
+        if (!Strutil::iends_with(p, ".exe")
+            && is_executable(Strutil::concat(p, ".exe")))
+            return Strutil::concat(p, ".exe");
+#endif
+    }
+    return std::string();
+}
+
+
+
 bool
 Filesystem::get_directory_entries(const std::string& dirname,
                                   std::vector<std::string>& filenames,
@@ -339,6 +382,30 @@ Filesystem::is_regular(string_view path) noexcept
 
 
 
+bool
+Filesystem::is_executable(string_view path) noexcept
+{
+    if (!is_regular(path))
+        return false;
+    error_code ec;
+    auto stat = filesystem::status(u8path(path), ec);
+    auto perm = stat.permissions();
+#ifdef USE_STD_FILESYSTEM
+    return (perm & filesystem::perms::owner_exec) != filesystem::perms::none
+           || (perm & filesystem::perms::group_exec) != filesystem::perms::none
+           || (perm & filesystem::perms::others_exec)
+                  != filesystem::perms::none;
+#else
+    return (perm & filesystem::perms::owner_exe) != filesystem::perms::no_perms
+           || (perm & filesystem::perms::group_exe)
+                  != filesystem::perms::no_perms
+           || (perm & filesystem::perms::others_exe)
+                  != filesystem::perms::no_perms;
+#endif
+}
+
+
+
 bool
 Filesystem::create_directory(string_view path, std::string& err)
 {
@@ -425,9 +492,38 @@ Filesystem::temp_directory_path()
 std::string
 Filesystem::unique_path(string_view model)
 {
+#ifdef USE_BOOST_FILESYSTEM
+    // Boost filesystem has unique_path().
     error_code ec;
     filesystem::path p = filesystem::unique_path(u8path(model), ec);
     return ec ? std::string() : pathstr(p);
+#else
+    // std::filesystem does not have unique_path(). Punt!
+#    if defined(_WIN32)
+    // boost internally doesn't use MultiByteToWideChar (CP_UTF8,...
+    // to convert char* to wchar_t* because they do not know the encoding
+    // See boost/filesystem/path.hpp
+    // The only correct way to do this is to do the conversion ourselves
+    std::wstring modelStr    = Strutil::utf8_to_utf16wstring(model);
+#    else
+    std::string modelStr = model.str();
+#    endif
+    static const char chrs[] = "0123456789abcdef";
+    static std::mt19937 rg { std::random_device {}() };
+    static std::uniform_int_distribution<size_t> pick(0, 15);
+    static std::mutex mutex;
+    std::lock_guard<std::mutex> lock(mutex);
+    std::string name;
+    while (true) {
+        name = model.str();
+        for (size_t i = 0, e = model.size(); i < e; ++i)
+            if (name[i] == '%')
+                name[i] = chrs[pick(rg)];
+        if (!exists(name))
+            break;
+    }
+    return name;
+#endif
 }
 
 
@@ -447,13 +543,12 @@ Filesystem::fopen(string_view path, string_view mode)
 {
 #ifdef _WIN32
     // on Windows fopen does not accept UTF-8 paths, so we convert to wide char
-    std::wstring wpath = Strutil::utf8_to_utf16(path);
-    std::wstring wmode = Strutil::utf8_to_utf16(mode);
-
+    std::wstring wpath = Strutil::utf8_to_utf16wstring(path);
+    std::wstring wmode = Strutil::utf8_to_utf16wstring(mode);
     return ::_wfopen(wpath.c_str(), wmode.c_str());
 #else
     // on Unix platforms passing in UTF-8 works
-    return ::fopen(path.c_str(), mode.c_str());
+    return ::fopen(std::string(path).c_str(), std::string(mode).c_str());
 #endif
 }
 
@@ -490,7 +585,7 @@ Filesystem::open(OIIO::ifstream& stream, string_view path,
 #ifdef _WIN32
     // Windows std::ifstream accepts non-standard wchar_t*
     // On MingW, we use our own OIIO::ifstream
-    std::wstring wpath = Strutil::utf8_to_utf16(path);
+    std::wstring wpath = Strutil::utf8_to_utf16wstring(path);
     stream.open(wpath.c_str(), mode);
     stream.seekg(0, std::ios_base::beg);  // force seek, otherwise broken
 #else
@@ -507,7 +602,7 @@ Filesystem::open(OIIO::ofstream& stream, string_view path,
 #ifdef _WIN32
     // Windows std::ofstream accepts non-standard wchar_t*
     // On MingW, we use our own OIIO::ofstream
-    std::wstring wpath = Strutil::utf8_to_utf16(path);
+    std::wstring wpath = Strutil::utf8_to_utf16wstring(path);
     stream.open(wpath.c_str(), mode);
 #else
     stream.open(path, mode);
@@ -522,11 +617,11 @@ Filesystem::open(string_view path, int flags)
 #ifdef _WIN32
     // on Windows _open does not accept UTF-8 paths, so we convert to wide
     // char and use _wopen.
-    std::wstring wpath = Strutil::utf8_to_utf16(path);
+    std::wstring wpath = Strutil::utf8_to_utf16wstring(path);
     return ::_wopen(wpath.c_str(), flags);
 #else
     // on Unix platforms passing in UTF-8 works
-    return ::open(path.c_str(), flags);
+    return ::open(std::string(path).c_str(), flags);
 #endif
 }
 
@@ -586,9 +681,20 @@ Filesystem::read_bytes(string_view path, void* buffer, size_t n, size_t pos)
 std::time_t
 Filesystem::last_write_time(string_view path) noexcept
 {
-    error_code ec;
-    std::time_t t = filesystem::last_write_time(u8path(path), ec);
-    return ec ? 0 : t;
+#ifdef _WIN32
+    struct __stat64 st;
+    auto r = _wstat64(u8path(path).c_str(), &st);
+#else
+    struct stat st;
+    auto r = stat(u8path(path).c_str(), &st);
+#endif
+    if (r == 0) {
+        // success
+        return st.st_mtime;
+    } else {
+        // failure
+        return 0;
+    }
 }
 
 
@@ -596,8 +702,19 @@ Filesystem::last_write_time(string_view path) noexcept
 void
 Filesystem::last_write_time(string_view path, std::time_t time) noexcept
 {
-    error_code ec;
-    filesystem::last_write_time(u8path(path), time, ec);
+#ifdef _WIN32
+    struct _utimbuf times;
+    times.actime  = time;
+    times.modtime = time;
+    _wutime(u8path(path).c_str(), &times);
+#else
+    struct timespec times[2];
+    times[0].tv_sec  = 0;
+    times[0].tv_nsec = UTIME_OMIT;
+    times[1].tv_sec  = time;
+    times[1].tv_nsec = 0;
+    utimensat((int)AT_FDCWD, u8path(path).c_str(), times, AT_SYMLINK_NOFOLLOW);
+#endif
 }
 
 
@@ -921,7 +1038,7 @@ Filesystem::scan_for_matching_filenames(const std::string& pattern_,
              !ec && it != end_it; ++it) {
             std::string itpath = Filesystem::generic_filepath(
                 it->path().string());
-            if (filesystem::is_regular(itpath, ec)) {
+            if (filesystem::is_regular_file(itpath, ec)) {
                 const std::string f = pathstr(itpath);
                 std::match_results<std::string::const_iterator> frame_match;
                 if (regex_match(f, frame_match, pattern_re)) {
@@ -1034,8 +1151,7 @@ Filesystem::IOFile::IOFile(string_view filename, Mode mode)
 {
     // Call Filesystem::fopen since it handles UTF-8 file paths on Windows,
     // which std fopen does not.
-    m_file = Filesystem::fopen(m_filename.c_str(),
-                               m_mode == Write ? "wb" : "rb");
+    m_file = Filesystem::fopen(m_filename, m_mode == Write ? "wb" : "rb");
     if (!m_file) {
         m_mode          = Closed;
         int e           = errno;
@@ -1091,6 +1207,15 @@ Filesystem::IOFile::read(void* buf, size_t size)
         return 0;
     size_t r = fread(buf, 1, size, m_file);
     m_pos += r;
+    if (r < size) {
+        if (feof(m_file))
+            error("end of file");
+        else if (ferror(m_file)) {
+            int e           = errno;
+            const char* msg = e ? std::strerror(e) : nullptr;
+            error(msg ? msg : "unknown error");
+        }
+    }
     return r;
 }
 
@@ -1108,7 +1233,9 @@ Filesystem::IOFile::pread(void* buf, size_t size, int64_t offset)
     return r;
 #else /* Non-Windows: assume POSIX pread is available */
     int fd = fileno(m_file);
-    return ::pread(fd, buf, size, offset);
+    auto r = ::pread(fd, buf, size, offset);
+    // FIXME: the system pread returns ssize_t and is -1 on error.
+    return r < 0 ? size_t(0) : size_t(r);
 #endif
 }
 
@@ -1137,8 +1264,10 @@ Filesystem::IOFile::pwrite(const void* buf, size_t size, int64_t offset)
     seek(origpos);
     return r;
 #else /* Non-Windows: assume POSIX pwrite is available */
-    int fd   = fileno(m_file);
-    size_t r = ::pwrite(fd, buf, size, offset);
+    int fd = fileno(m_file);
+    auto r = ::pwrite(fd, buf, size, offset);
+    // FIXME: the system pwrite returns ssize_t and is -1 on error.
+    return r < 0 ? size_t(0) : size_t(r);
 #endif
     offset += r;
     if (m_pos > int64_t(m_size))
@@ -1205,8 +1334,17 @@ size_t
 Filesystem::IOMemReader::pread(void* buf, size_t size, int64_t offset)
 {
     // N.B. No lock necessary
-    if (size + size_t(offset) > size_t(m_buf.size()))
+    if (!m_buf.size() || !size)
+        return 0;
+    if (size + size_t(offset) > size_t(m_buf.size())) {
+        if (offset < 0 || offset >= m_buf.size()) {
+            error(Strutil::fmt::format(
+                "Invalid pread offset {} for an IOMemReader buffer of size {}",
+                offset, m_buf.size()));
+            return 0;
+        }
         size = m_buf.size() - size_t(offset);
+    }
     memcpy(buf, m_buf.data() + offset, size);
     return size;
 }
