@@ -5,7 +5,6 @@
 #include <fstream>
 #include <sstream>
 
-#include <OpenImageIO/Imath.h>
 #include <OpenImageIO/filesystem.h>
 #include <OpenImageIO/imageio.h>
 #include <OpenImageIO/platform.h>
@@ -36,6 +35,13 @@ test_filename_decomposition()
                      ".ext");
     OIIO_CHECK_EQUAL(Filesystem::extension("/directory/filename"), "");
     OIIO_CHECK_EQUAL(Filesystem::extension("/directory/filename."), ".");
+    OIIO_CHECK_EQUAL(Filesystem::extension("a.foo"), ".foo");
+    OIIO_CHECK_EQUAL(Filesystem::extension("a.foo", false), "foo");
+    OIIO_CHECK_EQUAL(Filesystem::extension("foo"), "");
+    OIIO_CHECK_EQUAL(Filesystem::extension("foo", false), "");
+    OIIO_CHECK_EQUAL(Filesystem::extension(".foo"), "");
+    OIIO_CHECK_EQUAL(Filesystem::extension(".foo", false), "");
+
     OIIO_CHECK_EQUAL(Filesystem::parent_path(test), "/directoryA/directory");
 
     std::cout << "Testing path_is_absolute\n";
@@ -122,6 +128,10 @@ test_filename_searchpath_find()
                                                  true),
                      ".." DIRSEP ".." DIRSEP "include" DIRSEP
                      "OpenImageIO" DIRSEP "oiioversion.h");
+
+    // Test find_program
+    OIIO_CHECK_ASSERT(
+        Filesystem::is_executable(Filesystem::find_program("bash")));
 }
 
 
@@ -173,13 +183,17 @@ test_file_status()
     OIIO_CHECK_ASSERT(Filesystem::exists("testfile"));
     OIIO_CHECK_ASSERT(Filesystem::exists("testdir"));
     OIIO_CHECK_ASSERT(!Filesystem::exists("noexist"));
-    std::cout << "Testing is_directory, is_regular\n";
+    std::cout << "Testing is_directory, is_regular, is_executable\n";
     OIIO_CHECK_ASSERT(Filesystem::is_regular("testfile"));
     OIIO_CHECK_ASSERT(!Filesystem::is_directory("testfile"));
+    OIIO_CHECK_ASSERT(!Filesystem::is_executable("testfile"));
     OIIO_CHECK_ASSERT(!Filesystem::is_regular("testdir"));
     OIIO_CHECK_ASSERT(Filesystem::is_directory("testdir"));
+    OIIO_CHECK_ASSERT(!Filesystem::is_executable("testdir"));
     OIIO_CHECK_ASSERT(!Filesystem::is_regular("noexist"));
     OIIO_CHECK_ASSERT(!Filesystem::is_directory("noexist"));
+    OIIO_CHECK_ASSERT(!Filesystem::is_executable("noexist"));
+    OIIO_CHECK_ASSERT(Filesystem::is_executable(Sysutil::this_program_path()));
 
     std::cout << "Testing copy, rename, remove\n";
     OIIO_CHECK_ASSERT(!Filesystem::exists("testfile2"));
@@ -222,7 +236,7 @@ test_seq(const char* str, const char* expected)
 
 
 static void
-test_file_seq(const char* pattern, const char* override,
+test_file_seq(const char* pattern, string_view overrideval,
               const std::string& expected)
 {
     std::vector<int> numbers;
@@ -231,23 +245,21 @@ test_file_seq(const char* pattern, const char* override,
     std::string frame_range;
 
     Filesystem::parse_pattern(pattern, 0, normalized_pattern, frame_range);
-    if (override && strlen(override) > 0)
-        frame_range = override;
+    if (overrideval.size())
+        frame_range = overrideval;
     Filesystem::enumerate_sequence(frame_range, numbers);
     Filesystem::enumerate_file_sequence(normalized_pattern, numbers, names);
     std::string joined = Strutil::join(names, " ");
-    std::cout << "  " << pattern;
-    if (override)
-        std::cout << " + " << override;
-    std::cout << " -> " << joined << "\n";
+    Strutil::print(" {}{}{} -> {}\n", pattern, overrideval.size() ? " + " : "",
+                   overrideval, joined);
     OIIO_CHECK_EQUAL(joined, expected);
 }
 
 
 
 static void
-test_file_seq_with_view(const char* pattern, const char* override,
-                        const char* view, const std::string& expected)
+test_file_seq_with_view(const char* pattern, string_view overrideval,
+                        string_view view, const std::string& expected)
 {
     std::vector<int> numbers;
     std::vector<string_view> views;
@@ -256,11 +268,11 @@ test_file_seq_with_view(const char* pattern, const char* override,
     std::string frame_range;
 
     Filesystem::parse_pattern(pattern, 0, normalized_pattern, frame_range);
-    if (override && strlen(override) > 0)
-        frame_range = override;
-    Filesystem::enumerate_sequence(frame_range.c_str(), numbers);
+    if (overrideval.size())
+        frame_range = overrideval;
+    Filesystem::enumerate_sequence(frame_range, numbers);
 
-    if (view) {
+    if (view.size()) {
         for (size_t i = 0, e = numbers.size(); i < e; ++i)
             views.emplace_back(view);
     }
@@ -268,10 +280,8 @@ test_file_seq_with_view(const char* pattern, const char* override,
     Filesystem::enumerate_file_sequence(normalized_pattern, numbers, views,
                                         names);
     std::string joined = Strutil::join(names, " ");
-    std::cout << "  " << pattern;
-    if (override)
-        std::cout << " + " << override;
-    std::cout << " -> " << joined << "\n";
+    Strutil::print(" {}{}{} -> {}\n", pattern, overrideval.size() ? " + " : "",
+                   overrideval, joined);
     OIIO_CHECK_EQUAL(joined, expected);
 }
 
@@ -542,6 +552,19 @@ test_mem_proxies()
 
 
 
+void
+test_last_write_time()
+{
+    Filesystem::write_text_file("oiio-testtime.txt", "test");
+    time_t t = Filesystem::last_write_time("oiio-testtime.txt");
+    std::cout << "Last write time of oiio-testtime.txt is " << t << "\n";
+    Filesystem::last_write_time("oiio-testtime.txt", t - 42);
+    OIIO_CHECK_EQUAL(Filesystem::last_write_time("oiio-testtime.txt"), t - 42);
+    Filesystem::remove("oiio-testtime.txt");
+}
+
+
+
 int
 main(int /*argc*/, char* /*argv*/[])
 {
@@ -551,6 +574,7 @@ main(int /*argc*/, char* /*argv*/[])
     test_frame_sequences();
     test_scan_sequences();
     test_mem_proxies();
+    test_last_write_time();
 
     return unit_test_failures;
 }
