@@ -8,6 +8,7 @@
 #include <iostream>
 #include <vector>
 
+#include <OpenImageIO/Imath.h>
 #include <OpenImageIO/argparse.h>
 #include <OpenImageIO/benchmark.h>
 #include <OpenImageIO/fmath.h>
@@ -16,12 +17,8 @@
 #include <OpenImageIO/timer.h>
 #include <OpenImageIO/typedesc.h>
 #include <OpenImageIO/unittest.h>
+#include <OpenImageIO/vecparam.h>
 
-#if OIIO_USING_IMATH >= 3
-#    include <Imath/ImathFun.h>
-#else
-#    include <OpenEXR/ImathFun.h>
-#endif
 
 using namespace OIIO;
 
@@ -320,7 +317,7 @@ test_convert_type(double tolerance = 1e-6)
             }
         }
     } else {
-        for (float i = 0.0f; i <= 1.0f; i += 0.001) {  // NOLINT
+        for (float i = 0.0f; i <= 1.0f; i += 0.001) {  // NOLINT //NOSONAR
             T in  = (T)i;
             F f   = convert_type<T, F>(in);
             T out = convert_type<F, T>(f);
@@ -514,7 +511,7 @@ test_half_convert_accuracy()
     const int nhalfs = 1 << 16;
     std::vector<half> H(nhalfs, 0.0f);
     for (auto i = 0; i < nhalfs; ++i)
-        H[i] = bit_cast<unsigned short, half>((unsigned short)i);
+        H[i] = bitcast<half, uint16_t>((uint16_t)i);
 
     // Convert the whole array to float equivalents in one shot (which will
     // use SIMD ops if available).
@@ -532,9 +529,8 @@ test_half_convert_accuracy()
         float f = H[i];  // single assignment uses table from Imath
         half h  = (half)f;
         if ((f != F[i] || f != H2[i] || f != h || H[i] != H2[i]
-             || bit_cast<half, unsigned short>(h)
-                    != bit_cast<half, unsigned short>(H[i])
-             || bit_cast<half, unsigned short>(h) != i)
+             || bitcast<uint16_t, half>(h) != bitcast<uint16_t, half>(H[i])
+             || bitcast<uint16_t, half>(h) != i)
             && Imath::finitef(H[i])) {
             ++nwrong;
             Strutil::printf("wrong %d 0b%s  h=%g, f=%g %s\n", i, bin16(i), H[i],
@@ -552,6 +548,22 @@ test_half_convert_accuracy()
 
 
 
+static void
+test_bitcast()
+{
+    OIIO_CHECK_EQUAL((bitcast<uint16_t, half>(half(0.0f))), 0);
+    OIIO_CHECK_EQUAL((bitcast<uint32_t, float>(0.0f)), 0);
+    OIIO_CHECK_EQUAL((bitcast<int32_t, float>(0.0f)), 0);
+    OIIO_CHECK_EQUAL((bitcast<float, uint32_t>(0)), 0.0f);
+    OIIO_CHECK_EQUAL((bitcast<float, int32_t>(0)), 0.0f);
+    OIIO_CHECK_EQUAL((bitcast<uint64_t, double>(0.0)), 0);
+    OIIO_CHECK_EQUAL((bitcast<int64_t, double>(0.0)), 0);
+    OIIO_CHECK_EQUAL((bitcast<double, uint64_t>(0)), 0.0);
+    OIIO_CHECK_EQUAL((bitcast<double, int64_t>(0)), 0.0);
+}
+
+
+
 template<typename T>
 static void
 test_swap_endian(T val, T swapval)
@@ -588,6 +600,102 @@ test_swap_endian()
 
 
 
+// Minimal vector class having x, y, z struct members.
+struct XYZVector {
+    float x, y, z;
+    XYZVector() {}
+    XYZVector(float x, float y, float z)
+        : x(x)
+        , y(y)
+        , z(z)
+    {
+    }
+};
+
+// Minimal vector class enclosing an array[3].
+struct Arr3Vector {
+    float xyz[3];
+    Arr3Vector() {}
+    Arr3Vector(float x, float y, float z)
+    {
+        xyz[0] = x;
+        xyz[1] = y;
+        xyz[2] = z;
+    }
+    float operator[](int i) const { return xyz[i]; }
+};
+
+
+
+// Function that takes a V3fParam, and must implicitly convert it to an
+// Imath::V3f.
+Imath::V3f
+v3ffunc(V3fParam p)
+{
+    return p;
+}
+
+
+// Function that takes a M33Param, and must implicitly convert it to an
+// Imath::M33f.
+Imath::M33f
+M33func(M33fParam p)
+{
+    return p;
+}
+
+
+// Function that takes a M44Param, and must implicitly convert it to an
+// Imath::M44f.
+Imath::M44f
+m44func(M44fParam p)
+{
+    return p;
+}
+
+
+static void
+test_vecparam()
+{
+    Strutil::print("Testing vec proxy passing\n");
+
+    // Can we pass an Imath::V3f as a V3fParam?
+    Imath::V3f iv3f(1.0f, 2.0f, 3.0f);
+    OIIO_CHECK_EQUAL(v3ffunc(iv3f), iv3f);
+
+    // Can we pass a raw float[3] array as a V3fParam?
+    float arr[3] = { 1.0, 2.0, 3.0 };
+    OIIO_CHECK_EQUAL(v3ffunc(arr), iv3f);
+
+    // Can we pass a std::array<float,3> as a V3fParam?
+    std::array<float, 3> stdarr { 1.0, 2.0, 3.0 };
+    OIIO_CHECK_EQUAL(v3ffunc(stdarr), iv3f);
+
+    // Can we pass an initializer list as a V3fParam?
+    OIIO_CHECK_EQUAL(v3ffunc({ 1.0f, 2.0f, 3.0f }), iv3f);
+
+    // Can we pass a custom vector class with xyz components as a V3fParam?
+    XYZVector xyzv(1.0f, 2.0f, 3.0f);
+    OIIO_CHECK_EQUAL(v3ffunc(xyzv), iv3f);
+
+    // Can we pass a custom vector class with array components as a V3fParam?
+    Arr3Vector av(1.0f, 2.0f, 3.0f);
+    OIIO_CHECK_EQUAL(v3ffunc(av), iv3f);
+
+    // Can we pass our simd::vfloat3 as a V3fParam?
+    simd::vfloat3 vf3(1.0f, 2.0f, 3.0f);
+    OIIO_CHECK_EQUAL(v3ffunc(vf3), iv3f);
+
+    OIIO_CHECK_ASSERT((has_xyz<XYZVector, float>::value));
+    OIIO_CHECK_ASSERT((has_subscript_N<Arr3Vector, float, 3>::value));
+    OIIO_CHECK_ASSERT((has_subscript_N<simd::vfloat3, float, 3>::value));
+
+    Imath::M44f m44f(1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1);
+    OIIO_CHECK_EQUAL(m44func(m44f), m44f);
+}
+
+
+
 int
 main(int argc, char* argv[])
 {
@@ -643,7 +751,7 @@ main(int argc, char* argv[])
     benchmark_convert_type<half, float>();
     benchmark_convert_type<float, half>();
     benchmark_convert_type<float, float>();
-    // convertion to a type smaller in bytes causes error
+    // conversion to a type smaller in bytes causes error
     //    std::cout << "round trip convert float/short/float\n";
     //    test_convert_type<float,short> ();
     //    std::cout << "round trip convert unsigned float/char/float\n";
@@ -657,9 +765,12 @@ main(int argc, char* argv[])
 
     test_bit_range_convert();
     test_packbits();
+    test_bitcast();
     test_swap_endian();
 
     test_interpolate_linear();
 
+    test_vecparam();
+
     return unit_test_failures != 0;
 }
