@@ -6,8 +6,8 @@
 #include <cstdio>
 #include <cstdlib>
 
-#include <OpenImageIO/Imath.h>
 #include <OpenImageIO/dassert.h>
+#include <OpenImageIO/half.h>
 #include <OpenImageIO/paramlist.h>
 #include <OpenImageIO/ustring.h>
 
@@ -17,17 +17,19 @@ OIIO_NAMESPACE_BEGIN
 
 void
 ParamValue::init_noclear(ustring _name, TypeDesc _type, int _nvalues,
-                         const void* _value, bool _copy) noexcept
+                         const void* _value, Copy _copy,
+                         FromUstring _from_ustring) noexcept
 {
-    init_noclear(_name, _type, _nvalues, INTERP_CONSTANT, _value, _copy);
+    init_noclear(_name, _type, _nvalues, INTERP_CONSTANT, _value, _copy,
+                 _from_ustring);
 }
 
 
 
 void
 ParamValue::init_noclear(ustring _name, TypeDesc _type, int _nvalues,
-                         Interp _interp, const void* _value,
-                         bool _copy) noexcept
+                         Interp _interp, const void* _value, Copy _copy,
+                         FromUstring _from_ustring) noexcept
 {
     m_name      = _name;
     m_type      = _type;
@@ -46,18 +48,18 @@ ParamValue::init_noclear(ustring _name, TypeDesc _type, int _nvalues,
             m_copy     = false;
             m_nonlocal = false;
         } else {
-            m_data.ptr = malloc(size);
+            void* ptr = malloc(size);
             if (_value)
-                memcpy((char*)m_data.ptr, _value, size);
+                memcpy(ptr, _value, size);  //NOSONAR
             else
-                memset((char*)m_data.ptr, 0, size);
+                memset(ptr, 0, size);
+            m_data.ptr = ptr;
             m_copy     = true;
             m_nonlocal = true;
         }
-        if (m_type.basetype == TypeDesc::STRING) {
-            ustring* u = (ustring*)data();
-            for (size_t i = 0; i < n; ++i)
-                if (u)
+        if (m_type.basetype == TypeDesc::STRING && !_from_ustring) {
+            if (ustring* u = (ustring*)data())
+                for (size_t i = 0; i < n; ++i)
                     u[i] = ustring(u[i].c_str());
         }
     } else {
@@ -71,6 +73,35 @@ ParamValue::init_noclear(ustring _name, TypeDesc _type, int _nvalues,
 
 
 
+const ParamValue&
+ParamValue::operator=(const ParamValue& p) noexcept
+{
+    if (this != &p) {
+        clear_value();
+        init_noclear(p.name(), p.type(), p.nvalues(), p.interp(), p.data(),
+                     Copy(p.m_copy), FromUstring(true));
+    }
+    return *this;
+}
+
+
+
+const ParamValue&
+ParamValue::operator=(ParamValue&& p) noexcept
+{
+    if (this != &p) {
+        clear_value();
+        init_noclear(p.name(), p.type(), p.nvalues(), p.interp(), p.data(),
+                     Copy(false), FromUstring(true));
+        m_copy       = p.m_copy;
+        m_nonlocal   = p.m_nonlocal;
+        p.m_data.ptr = nullptr;  // make sure the old one won't free
+    }
+    return *this;
+}
+
+
+
 namespace Strutil {
 template<>
 inline short
@@ -284,19 +315,20 @@ formatType(const ParamValue& p, int beginindex, int endindex,
 std::string
 ParamValue::get_string(int maxsize) const
 {
-    int nfull  = int(type().numelements()) * nvalues();
-    int n      = std::min(nfull, maxsize);
+    int nfull  = int(type().basevalues()) * nvalues();
+    int n      = maxsize ? std::min(nfull, maxsize) : nfull;
     TypeDesc t = type();
-    if (nvalues() > 1 || n < nfull)
-        t.arraylen = n;
+    if (nvalues() > 1 || n < nfull) {
+        t.aggregate = TypeDesc::SCALAR;
+        t.arraylen  = n;
+    }
     static const tostring_formatting fmt = { "%d", "%g", "\"%s\"", "%p",
                                              "",   "",   ", ",     "",
                                              "",   ", ", true,     "%u" };
     std::string out                      = tostring(t, data(), fmt);
     if (n < nfull)
-        out += Strutil::sprintf(
-            ", ... [%d x %s]", nfull,
-            TypeDesc(TypeDesc::BASETYPE(type().basetype)).c_str());
+        out += Strutil::fmt::format(", ... [{} x {}]", nfull,
+                                    type().scalartype());
     return out;
 }
 
@@ -311,7 +343,7 @@ ParamValue::get_string_indexed(int index) const
     if (index < 0 || index >= n)
         return out;
     if (element.basetype == TypeDesc::STRING) {
-        return get<const char*>(index);
+        return get<ustring>(index).string();
     } else if (element.basetype == TypeDesc::FLOAT) {
         formatType<float>(*this, index, index + 1, "%g", out);
     } else if (element.basetype == TypeDesc::DOUBLE) {
