@@ -28,6 +28,7 @@
 #include <OpenImageIO/platform.h>
 #include <OpenImageIO/string_view.h>
 #include <OpenImageIO/strutil.h>
+#include <OpenImageIO/thread.h>
 #include <OpenImageIO/ustring.h>
 
 #ifdef _WIN32
@@ -45,6 +46,11 @@
 #    define STBI__ASAN OIIO_NO_SANITIZE_ADDRESS
 #endif
 #define stbsp__uintptr std::uintptr_t
+
+#ifdef OpenImageIO_SANITIZE
+#    define STB_SPRINTF_NOUNALIGNED
+#endif
+
 #include "stb_sprintf.h"
 
 
@@ -91,15 +97,14 @@ isdigit(char c)
 
 
 
-OIIO_NO_SANITIZE_ADDRESS
-const char*
-string_view::c_str() const
+OIIO_NO_SANITIZE_ADDRESS const char*
+c_str(string_view str)
 {
     // Usual case: either empty, or null-terminated
-    if (m_len == 0)  // empty string
+    if (str.size() == 0)  // empty string
         return "";
 
-    // This clause attempts to find out if there's a string-teriminating
+    // This clause attempts to find out if there's a string-terminating
     // '\0' character just beyond the boundary of the string_view, in which
     // case, simply returning m_chars (with no ustring creation) is a valid
     // C string.
@@ -117,38 +122,74 @@ string_view::c_str() const
     // in C++17 string_view. So maybe we'll find ourselves relying on it a
     // lot less, and therefore the performance hit of doing it foolproof
     // won't be as onerous.
-    if (m_chars[m_len] == 0)  // 0-terminated
-        return m_chars;
+    if (str[str.size()] == 0)  // 0-terminated
+        return str.data();
 
     // Rare case: may not be 0-terminated. Bite the bullet and construct a
     // 0-terminated string.  We use ustring as a way to avoid any issues of
     // who cleans up the allocation, though it means that it will stay in
     // the ustring table forever. Punt on this for now, it's an edge case
     // that we need to handle, but is not likely to ever be an issue.
-    return ustring(m_chars, 0, m_len).c_str();
+    return ustring(str).c_str();
 }
 
 
 
 void
-Strutil::sync_output(FILE* file, string_view str)
+Strutil::sync_output(FILE* file, string_view str, bool flush)
 {
     if (str.size() && file) {
         std::unique_lock<std::mutex> lock(output_mutex);
         fwrite(str.data(), 1, str.size(), file);
-        fflush(file);
+        if (flush)
+            fflush(file);
     }
 }
 
 
 
 void
-Strutil::sync_output(std::ostream& file, string_view str)
+Strutil::sync_output(std::ostream& file, string_view str, bool flush)
 {
     if (str.size()) {
         std::unique_lock<std::mutex> lock(output_mutex);
         file << str;
-        file.flush();
+        if (flush)
+            file.flush();
+    }
+}
+
+
+
+namespace pvt {
+static const char* oiio_debug_env = getenv("OPENIMAGEIO_DEBUG");
+#ifdef NDEBUG
+OIIO_UTIL_API int
+    oiio_print_debug(oiio_debug_env ? Strutil::stoi(oiio_debug_env) : 0);
+#else
+OIIO_UTIL_API int
+    oiio_print_debug(oiio_debug_env ? Strutil::stoi(oiio_debug_env) : 1);
+#endif
+}  // namespace pvt
+
+
+void
+Strutil::pvt::debug(string_view message)
+{
+    if (OIIO::pvt::oiio_print_debug) {
+        static mutex debug_mutex;
+        lock_guard lock(debug_mutex);
+        static FILE* oiio_debug_file = nullptr;
+        if (!oiio_debug_file) {
+            const char* filename = getenv("OPENIMAGEIO_DEBUG_FILE");
+            oiio_debug_file = filename && filename[0] ? fopen(filename, "a")
+                                                      : stderr;
+            OIIO_ASSERT(oiio_debug_file);
+            if (!oiio_debug_file)
+                oiio_debug_file = stderr;
+        }
+        print(oiio_debug_file, "OIIO DEBUG: {}", message);
+        fflush(oiio_debug_file);
     }
 }
 
@@ -311,8 +352,9 @@ Strutil::escape_chars(string_view unescaped)
             case '\r': c = 'r'; break;
             case '\f': c = 'f'; break;
             case '\a': c = 'a'; break;
+            default: break;
             }
-            s.insert(i, &c, 1);
+            s.insert(i, 1, c);
         }
     }
     return s;
@@ -338,10 +380,8 @@ Strutil::unescape_chars(string_view escaped)
                 case 'b': s[i] = '\b'; break;
                 case 'r': s[i] = '\r'; break;
                 case 'f': s[i] = '\f'; break;
-                case 'a':
-                    s[i] = '\a';
-                    break;
-                    // default case: the deletion is enough (backslash and quote)
+                case 'a': s[i] = '\a'; break;
+                default: break;  // the deletion is enough (backslash and quote)
                 }
             } else if (c >= '0' && c < '8') {
                 // up to 3 octal digits
@@ -369,7 +409,7 @@ Strutil::wordwrap(string_view src, int columns, int prefix, string_view sep,
         return src;  // give up, no way to make it wrap
     std::ostringstream out;
     columns -= prefix;  // now columns is the real width we have to work with
-    std::string allsep = Strutil::concat(sep, presep);
+    std::string allsep = Strutil::sprintf("%s%s", sep, presep);
     while ((int)src.length() > columns) {
         // Find the last `sep` break before the column limit.
         size_t breakpoint = src.find_last_of(allsep, columns);
@@ -740,16 +780,15 @@ Strutil::split(string_view str, std::vector<string_view>& result,
 std::string
 Strutil::concat(string_view s, string_view t)
 {
-    size_t sl  = s.size();
-    size_t tl  = t.size();
+    size_t sl = s.size();
+    size_t tl = t.size();
+    if (sl == 0)
+        return t;
+    if (tl == 0)
+        return s;
     size_t len = sl + tl;
-    std::unique_ptr<char[]> heap_buf;
-    char local_buf[256];
-    char* buf = local_buf;
-    if (len > sizeof(local_buf)) {
-        heap_buf.reset(new char[len]);
-        buf = heap_buf.get();
-    }
+    char* buf;
+    OIIO_ALLOCATE_STACK_OR_HEAP(buf, char, len);
     memcpy(buf, s.data(), sl);
     memcpy(buf + sl, t.data(), tl);
     return std::string(buf, len);
@@ -763,8 +802,8 @@ Strutil::repeat(string_view str, int n)
     size_t sl  = str.size();
     size_t len = sl * std::max(0, n);
     std::unique_ptr<char[]> heap_buf;
-    char local_buf[256];
-    char* buf = local_buf;
+    char local_buf[256] = "";
+    char* buf           = local_buf;
     if (len > sizeof(local_buf)) {
         heap_buf.reset(new char[len]);
         buf = heap_buf.get();
@@ -834,10 +873,13 @@ Strutil::replace(string_view str, string_view pattern, string_view replacement,
 // appropriate for error messages.
 
 std::wstring
-Strutil::utf8_to_utf16(string_view str) noexcept
+Strutil::utf8_to_utf16wstring(string_view str) noexcept
 {
     try {
+        OIIO_PRAGMA_WARNING_PUSH
+        OIIO_CLANG_PRAGMA(GCC diagnostic ignored "-Wdeprecated-declarations")
         std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>, wchar_t> conv;
+        OIIO_PRAGMA_WARNING_POP
         return conv.from_bytes(str.data(), str.data() + str.size());
     } catch (const std::exception&) {
         return std::wstring();
@@ -846,11 +888,41 @@ Strutil::utf8_to_utf16(string_view str) noexcept
 
 
 
+#if OPENIMAGEIO_VERSION < 30000
+// DEPRECATED(2.5) and slated for removal in 3.0.
+std::wstring
+Strutil::utf8_to_utf16(string_view str) noexcept
+{
+    return utf8_to_utf16wstring(str);
+}
+#endif
+
+
+
 std::string
 Strutil::utf16_to_utf8(const std::wstring& str) noexcept
 {
     try {
+        OIIO_PRAGMA_WARNING_PUSH
+        OIIO_CLANG_PRAGMA(GCC diagnostic ignored "-Wdeprecated-declarations")
         std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>, wchar_t> conv;
+        OIIO_PRAGMA_WARNING_POP
+        return conv.to_bytes(str);
+    } catch (const std::exception&) {
+        return std::string();
+    }
+}
+
+
+
+std::string
+Strutil::utf16_to_utf8(const std::u16string& str) noexcept
+{
+    try {
+        OIIO_PRAGMA_WARNING_PUSH
+        OIIO_CLANG_PRAGMA(GCC diagnostic ignored "-Wdeprecated-declarations")
+        std::wstring_convert<std::codecvt_utf8_utf16<char16_t>, char16_t> conv;
+        OIIO_PRAGMA_WARNING_POP
         return conv.to_bytes(str);
     } catch (const std::exception&) {
         return std::string();
@@ -1551,7 +1623,6 @@ Strutil::strtod(const char* nptr, char** endptr) noexcept
     // Can use strtod_l on platforms that support it
 #if defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__) \
     || defined(__FreeBSD_kernel__) || defined(__GLIBC__)
-    // static initialization inside function is thread-safe by C++11 rules!
     return strtod_l(nptr, endptr, c_loc);
 #elif defined(_WIN32)
     // Windows has _strtod_l
@@ -1631,7 +1702,7 @@ Strutil::stof(string_view s, size_t* pos)
     // will use the "short string optimization", meaning that this string
     // creation will NOT need an allocation/free for most strings we expect
     // to hold a text representation of a float.
-    return Strutil::stof(std::string(s).c_str(), pos);
+    return Strutil::stof(std::string(s), pos);
 }
 
 
@@ -1729,7 +1800,7 @@ Strutil::scan_datetime(string_view str, int& year, int& month, int& day,
                   || parse_char(str, '/', false))
               && parse_int(str, day) && parse_int(str, hour)
               && parse_char(str, ':', false) && parse_int(str, min)
-              && parse_char(str, ':', false) && parse_int(str, sec);
+              && parse_char(str, ':', false) && parse_int(str, sec);  //NOSONAR
     return ok && month >= 1 && month <= 12 && day >= 1 && day <= 31 && hour >= 0
            && hour <= 23 && min >= 0 && min <= 59 && sec >= 0 && sec <= 59;
 }
