@@ -77,18 +77,18 @@ test_format()
     bench.indent (2);
     bench.units (Benchmarker::Unit::ns);
     char buffer[256];
-    bench ("std::sprintf(\"%g\")", [&](){ DoNotOptimize (std::sprintf(buffer,"%g",123.45f)); });
+    bench ("std::snprintf(\"%g\")", [&](){ DoNotOptimize (std::snprintf(buffer,sizeof(buffer),"%g",123.45f)); });
     bench ("Strutil::sprintf(\"%g\")", [&](){ DoNotOptimize (Strutil::sprintf("%g",123.45f)); });
     bench ("Strutil::fmt::format(\"{:g}\")", [&](){ DoNotOptimize (Strutil::fmt::format("{:g}",123.45f)); });
     bench ("Strutil::to_string(float)", [&](){ DoNotOptimize (Strutil::to_string(123.45f)); });
 
-    bench ("std::sprintf(\"%d\")", [&](){ DoNotOptimize (std::sprintf(buffer,"%d",123)); });
+    bench ("std::snprintf(\"%d\")", [&](){ DoNotOptimize (std::snprintf(buffer,sizeof(buffer),"%d",123)); });
     bench ("Strutil::sprintf(\"%d\")", [&](){ DoNotOptimize (Strutil::sprintf("%g",123.0f)); });
     bench ("Strutil::fmt::format(\"{}\")", [&](){ DoNotOptimize (Strutil::fmt::format("{}",123)); });
     bench ("Strutil::to_string(int)", [&](){ DoNotOptimize (Strutil::to_string(123)); });
 
-    bench ("std::sprintf(\"%g %d %s %d %s %g\")", [&](){
-               DoNotOptimize (std::sprintf(buffer,"%g %d %s %d %s %g", 123.45f, 1234, "foobar", 42, "kablooey", 3.14159f));
+    bench ("std::snprintf(\"%g %d %s %d %s %g\")", [&](){
+               DoNotOptimize (std::snprintf(buffer,sizeof(buffer),"%g %d %s %d %s %g", 123.45f, 1234, "foobar", 42, "kablooey", 3.14159f));
            });
     bench ("Strutil::sprintf(\"%g %d %s %d %s %g\")", [&](){
                DoNotOptimize (Strutil::sprintf("%g %d %s %d %s %g", 123.45f, 1234, "foobar", 42, "kablooey", 3.14159f));
@@ -185,6 +185,18 @@ test_format_custom()
     OIIO_CHECK_EQUAL(Strutil::fmt::format("X|({:,.3f})|Y", ivf3iota),
                      "X|(1.500, 2.500, 3.500)|Y");
     Strutil::print("\n");
+
+    // Test custom formatting of spans
+    float farray[] = { 1.5f, 2.5f, 3.5f, 4.5f };
+    Strutil::print("cspan<float> {{}}  '{}'\n", cspan<float>(farray));
+    Strutil::print("cspan<float> {{:.3f}}  '{:.3f}'\n", cspan<float>(farray));
+    Strutil::print("cspan<float> {{:,.3f}}  '{:,.3f}'\n", cspan<float>(farray));
+    OIIO_CHECK_EQUAL(Strutil::fmt::format("X|{}|Y", cspan<float>(farray)),
+                     "X|1.5 2.5 3.5 4.5|Y");
+    OIIO_CHECK_EQUAL(Strutil::fmt::format("X|{:.3f}|Y", cspan<float>(farray)),
+                     "X|1.500 2.500 3.500 4.500|Y");
+    OIIO_CHECK_EQUAL(Strutil::fmt::format("X|({:,.3f})|Y", cspan<float>(farray)),
+                     "X|(1.500, 2.500, 3.500, 4.500)|Y");
 }
 
 
@@ -295,7 +307,7 @@ test_get_rest_arguments()
 void
 test_escape(string_view raw, string_view escaped)
 {
-    Strutil::print("escape '{}' <-> '{}'\n", raw, escaped);
+    Strutil::printf ("escape '%s' <-> '%s'\n", raw, escaped);
     OIIO_CHECK_EQUAL(Strutil::escape_chars(raw), escaped);
     OIIO_CHECK_EQUAL(Strutil::unescape_chars(escaped), raw);
 }
@@ -305,8 +317,9 @@ test_escape(string_view raw, string_view escaped)
 void
 test_escape_sequences()
 {
-    test_escape ("\\ \n \r \t", "\\\\ \\n \\r \\t");
+    test_escape ("\\ \n \r \t \v \b \f \a", "\\\\ \\n \\r \\t \\v \\b \\f \\a");
     test_escape (" \"quoted\" ",  " \\\"quoted\\\" ");
+    OIIO_CHECK_EQUAL(Strutil::unescape_chars("A\\023B"), "A\023B");
 }
 
 
@@ -482,7 +495,7 @@ test_comparisons()
     std::string abcnope = "1abcdefghijklmnopqrstuvwxyz";
     std::string haystack = std::string("begin") + abc + "oiio"
                          + Strutil::repeat(abc, 10) + "123" + abc + "end";
-    bench ("string== success", [&](){ DoNotOptimize(abc == abc); });
+    bench ("string== success", [&](){ DoNotOptimize(abc == abc); }); // NOSONAR
     bench ("string== failure", [&](){ DoNotOptimize(abc == abcmore); });
     bench ("iequals success", [&](){ DoNotOptimize(Strutil::iequals(abc, abc)); });
     bench ("iless easy", [&](){ DoNotOptimize(Strutil::iless(abc, abcnope)); });
@@ -852,6 +865,7 @@ test_numeric_conversion()
                      std::numeric_limits<int>::max());
     OIIO_CHECK_EQUAL(Strutil::stoi("-12345678901234567890"),
                      std::numeric_limits<int>::min());
+    OIIO_CHECK_EQUAL(Strutil::stoi("0x100", nullptr, 16), 256);  // hex
 
     OIIO_CHECK_EQUAL(Strutil::stoui("hi"), 0);
     OIIO_CHECK_EQUAL(Strutil::stoui("  "), 0);
@@ -921,6 +935,22 @@ test_numeric_conversion()
     OIIO_CHECK_EQUAL (Strutil::stof("100000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000001E-200"), 1.0f);
     OIIO_CHECK_EQUAL (Strutil::stof("0.00000000000000000001"), 1.0e-20f);
 
+    OIIO_CHECK_EQUAL(Strutil::strtod("314.25"), 314.25);
+    OIIO_CHECK_EQUAL(Strutil::strtod("hi"), 0.0);
+
+    pos = 100;
+    OIIO_CHECK_EQUAL(Strutil::stod(string_view("314.25"), &pos), 314.25);
+    OIIO_CHECK_EQUAL(pos, 6);
+    pos = 100;
+    OIIO_CHECK_EQUAL(Strutil::stod("hi", &pos), 0.0);
+    OIIO_CHECK_EQUAL(pos, 0);
+    pos = 100;
+    OIIO_CHECK_EQUAL(Strutil::stod("", &pos), 0.0);
+    OIIO_CHECK_EQUAL(pos, 0);
+    pos = 100;
+    OIIO_CHECK_EQUAL(Strutil::stod(nullptr, &pos), 0.0);
+    OIIO_CHECK_EQUAL(pos, 0);
+
     // Note: we don't test from_strings<> separately because it's just
     // implemented directly as calls to stoi, stoui, stof.
 
@@ -1112,6 +1142,36 @@ test_safe_strcat()
 
 
 
+// test safe_strlen and closely related safe_string_view, safe_string
+void
+test_safe_strlen()
+{
+    char a[] = "012";  // expected
+    char b[] = "012" "\0" "456789";  // nul embedded in the string
+    char c[] = "0123456789001234567890";  // long string
+    char d[] = "";  // empty string
+
+    Strutil::print("Testing safe_strlen\n");
+    OIIO_CHECK_EQUAL(Strutil::safe_strlen(a, 10), 3);
+    OIIO_CHECK_EQUAL(Strutil::safe_strlen(b, 10), 3);
+    OIIO_CHECK_EQUAL(Strutil::safe_strlen(c, 10), 10);
+    OIIO_CHECK_EQUAL(Strutil::safe_strlen(d, 10), 0);
+
+    std::cout << "Testing safe_string_view\n";
+    OIIO_CHECK_EQUAL(Strutil::safe_string_view(a, 10), string_view("012"));
+    OIIO_CHECK_EQUAL(Strutil::safe_string_view(b, 10), string_view("012"));
+    OIIO_CHECK_EQUAL(Strutil::safe_string_view(c, 10), string_view("0123456789"));
+    OIIO_CHECK_EQUAL(Strutil::safe_string_view(d, 10), string_view(""));
+
+    std::cout << "Testing safe_string\n";
+    OIIO_CHECK_EQUAL(Strutil::safe_string(a, 10), std::string("012"));
+    OIIO_CHECK_EQUAL(Strutil::safe_string(b, 10), std::string("012"));
+    OIIO_CHECK_EQUAL(Strutil::safe_string(c, 10), std::string("0123456789"));
+    OIIO_CHECK_EQUAL(Strutil::safe_string(d, 10), std::string(""));
+}
+
+
+
 // test some of the trickier methods in string_view.
 void
 test_string_view()
@@ -1126,6 +1186,24 @@ test_string_view()
     OIIO_CHECK_EQUAL(string_view(s), cstr);  // test ctr from std::string
     OIIO_CHECK_EQUAL(sr, cstr);  // These better be the same
 
+#ifdef OIIO_STD_STRING_VIEW_AVAILABLE
+    {
+        std::cout << "  Testing OIIO::string_view <-> std::string_view\n";
+        std::string_view ssv = sr;
+        OIIO::string_view osv = ssv;
+        OIIO_CHECK_EQUAL(osv, sr);
+    }
+#endif
+
+#ifdef OIIO_EXPERIMENTAL_STRING_VIEW_AVAILABLE
+    {
+        std::cout << "  Testing OIIO::string_view <-> std::experimental::string_view\n";
+        std::experimental::string_view ssv = sr;
+        OIIO::string_view osv = ssv;
+        OIIO_CHECK_EQUAL(osv, sr);
+    }
+#endif
+
     OIIO_CHECK_EQUAL(sr.substr(0), sr); // whole string
     OIIO_CHECK_EQUAL(sr.substr(2), "23401234"); // nonzero pos, default n
     OIIO_CHECK_EQUAL(sr.substr(2, 3), "234"); // true substrng
@@ -1196,6 +1274,22 @@ test_string_view()
     OIIO_CHECK_EQUAL(sr.find_last_not_of("234", 5), 1);
     OIIO_CHECK_EQUAL(sr.find_last_not_of("xyz"), 9);
     OIIO_CHECK_EQUAL(sr.find_last_not_of("01234"), string_view::npos);
+
+    // Just force template expansion of wchar_t variety to make sure it's
+    // not horribly broken.
+    wstring_view wsv;
+    OIIO_CHECK_ASSERT(wsv == wsv);
+
+    // Test the freestanding OIIO::c_str() function
+    OIIO_CHECK_EQUAL(OIIO::c_str(""), std::string());
+    OIIO_CHECK_EQUAL(OIIO::c_str(cstr), cstr);
+    OIIO_CHECK_EQUAL(OIIO::c_str(s), s);
+    OIIO_CHECK_EQUAL(OIIO::c_str(ustring(cstr)), ustring(cstr));
+    OIIO_CHECK_EQUAL(OIIO::c_str(sr), sr);
+    OIIO_CHECK_EQUAL(OIIO::c_str(string_view(sr.data(), 2)), std::string("01"));
+    Strutil::print("addr cstr={:p}, s={:p}, ustring={:p}, sr={:p}, c_str(sr)={:p}\n",
+                     (void*)cstr, (void*)s.c_str(), (void*)ustring(cstr).c_str(), (void*)sr.data(),
+                     (void*)c_str(sr));
 }
 
 
@@ -1217,6 +1311,18 @@ void test_parse ()
     s = "\tfoo\t"; remove_trailing_whitespace(s);  OIIO_CHECK_EQUAL (s, "\tfoo");
     s = "  foo  "; remove_trailing_whitespace(s);  OIIO_CHECK_EQUAL (s, "  foo");
 
+    s = "";        trim_whitespace(s);  OIIO_CHECK_EQUAL (s, "");
+    s = "   ";     trim_whitespace(s);  OIIO_CHECK_EQUAL (s, "");
+    s = "foo";     trim_whitespace(s);  OIIO_CHECK_EQUAL (s, "foo");
+    s = "\tfoo\t"; trim_whitespace(s);  OIIO_CHECK_EQUAL (s, "foo");
+    s = "  foo  "; trim_whitespace(s);  OIIO_CHECK_EQUAL (s, "foo");
+   
+    OIIO_CHECK_EQUAL(trimmed_whitespace(""),        "");
+    OIIO_CHECK_EQUAL(trimmed_whitespace("   "),     "");
+    OIIO_CHECK_EQUAL(trimmed_whitespace("foo"),     "foo");
+    OIIO_CHECK_EQUAL(trimmed_whitespace("\tfoo\t"), "foo");
+    OIIO_CHECK_EQUAL(trimmed_whitespace("  foo  "), "foo");
+   
     s = "abc"; OIIO_CHECK_ASSERT (! parse_char (s, 'd') && s == "abc");
 
     s = "abc"; OIIO_CHECK_ASSERT (parse_char (s, 'a', true, false) && s == "abc");
@@ -1347,8 +1453,16 @@ void test_parse ()
 
     s = "[a([b]c)]x]"; ss = parse_nested (s);
     OIIO_CHECK_EQUAL (ss, "[a([b]c)]"); OIIO_CHECK_EQUAL (s, "x]");
-    s = "[a([b]c)]x]"; ss = parse_nested (s, false);
+    s = "[a([b]c)]x]"; ss = parse_nested (s, false);  // no eating
     OIIO_CHECK_EQUAL (ss, "[a([b]c)]"); OIIO_CHECK_EQUAL (s, "[a([b]c)]x]");
+    s = "([a([b]c)])x]"; ss = parse_nested (s);
+    OIIO_CHECK_EQUAL (ss, "([a([b]c)])"); OIIO_CHECK_EQUAL (s, "x]");
+    s = "blah[a([b]c)]x]"; ss = parse_nested (s);
+    OIIO_CHECK_EQUAL (ss, ""); OIIO_CHECK_EQUAL (s, "blah[a([b]c)]x]");
+    s = ""; ss = parse_nested (s);
+    OIIO_CHECK_EQUAL (ss, ""); OIIO_CHECK_EQUAL (s, "");
+    s = "(blah"; ss = parse_nested (s);
+    OIIO_CHECK_EQUAL (ss, ""); OIIO_CHECK_EQUAL (s, "(blah");
 }
 
 
@@ -1410,7 +1524,7 @@ test_float_formatting()
         sstream.precision(9);
         sstream << *f;
         char buffer[64];
-        std::sprintf(buffer, "%.9g", *f);
+        std::snprintf(buffer, sizeof(buffer), "%.9g", *f);
         std::string tiny = Strutil::sprintf("%.9g", *f);
         if (sstream.str() != tiny || tiny != buffer)
             Strutil::printf(
@@ -1516,6 +1630,14 @@ test_edit_distance()
 
 
 
+void
+test_base64_encode()
+{
+    OIIO_CHECK_EQUAL(Strutil::base64_encode("foo123,()"), "Zm9vMTIzLCgp");
+}
+
+
+
 int
 main(int /*argc*/, char* /*argv*/[])
 {
@@ -1542,6 +1664,7 @@ main(int /*argc*/, char* /*argv*/[])
     test_extract();
     test_safe_strcpy();
     test_safe_strcat();
+    test_safe_strlen();
     test_string_view();
     test_parse();
     test_locale();
@@ -1549,6 +1672,9 @@ main(int /*argc*/, char* /*argv*/[])
     test_string_compare_function();
     test_datetime();
     test_edit_distance();
+    test_base64_encode();
+
+    Strutil::debug("debug message\n");
 
     return unit_test_failures;
 }
