@@ -10,6 +10,7 @@
 #include <limits>
 #include <sstream>
 
+#include <OpenImageIO/Imath.h>
 #include <OpenImageIO/argparse.h>
 #include <OpenImageIO/color.h>
 #include <OpenImageIO/dassert.h>
@@ -185,6 +186,7 @@ getargs(int argc, char* argv[], ImageSpec& configspec)
     std::string colorconfigname;
     std::string channelnames;
     std::string bumpformat = "auto";
+    std::string handed;
     std::vector<std::string> string_attrib_names, string_attrib_values;
     std::vector<std::string> any_attrib_names, any_attrib_values;
     filenames.clear();
@@ -192,8 +194,9 @@ getargs(int argc, char* argv[], ImageSpec& configspec)
     // clang-format off
     ArgParse ap;
     ap.intro("maketx -- convert images to tiled, MIP-mapped textures\n"
-             OIIO_INTRO_STRING);
-    ap.usage("maketx [options] file...");
+             OIIO_INTRO_STRING)
+      .usage("maketx [options] file...")
+      .add_version(OIIO_VERSION_STRING);
 
     ap.arg("filename")
       .hidden()
@@ -294,7 +297,7 @@ getargs(int argc, char* argv[], ImageSpec& configspec)
     ap.arg("--cdfsigma %f:N", &cdfsigma)
       .help("Specify the Gaussian sigma parameter when writing the forward and inverse Gaussian CDF data. The default vale is 1/6 (0.1667)");
     ap.arg("--cdfbits %d:N", &cdfbits)
-      .help("Specify the number of bits used to store the forward and inverse Guassian CDF. The default value is 8 bits");
+      .help("Specify the number of bits used to store the forward and inverse Gaussian CDF. The default value is 8 bits");
 
     ap.separator("Basic modes (default is plain texture):");
     ap.arg("--shadow", &shadowmode)
@@ -310,6 +313,8 @@ getargs(int argc, char* argv[], ImageSpec& configspec)
     ap.arg("--bumpformat %s:NAME", &bumpformat)
       .help("Specify the interpretation of a 3-channel input image for --bumpslopes: \"height\", \"normal\" or \"auto\" (default).");
 //                  "--envcube", &envcubemode, "Create cubic env map (file order: px, nx, py, ny, pz, nz) (UNIMP)",
+    ap.arg("--handed %s:STRING", &handed)
+      .help("Specify the handedness of a vector or normal map: \"left\", \"right\", or \"\" (default).");
 
     ap.separator(colortitle_help_string());
     ap.arg("--colorconfig %s:FILENAME", &colorconfigname)
@@ -436,6 +441,8 @@ getargs(int argc, char* argv[], ImageSpec& configspec)
         configspec.attribute("maketx:mipimages", Strutil::join(mipimages, ";"));
     if (bumpslopesmode)
         configspec.attribute("maketx:bumpformat", bumpformat);
+    if (handed.size())
+        configspec.attribute("handed", handed);
     configspec.attribute("maketx:cdf", cdf);
     configspec.attribute("maketx:cdfsigma", cdfsigma);
     configspec.attribute("maketx:cdfbits", cdfbits);
