@@ -18,7 +18,6 @@
 #include "oiiotool.h"
 
 #include <OpenImageIO/Imath.h>
-#include <OpenImageIO/argparse.h>
 #include <OpenImageIO/deepdata.h>
 #include <OpenImageIO/hash.h>
 #include <OpenImageIO/imagebuf.h>
@@ -26,55 +25,14 @@
 #include <OpenImageIO/imageio.h>
 #include <OpenImageIO/span.h>
 #include <OpenImageIO/strutil.h>
-#include <OpenImageIO/sysutil.h>
 
 using namespace OIIO;
+using namespace OIIO::pvt;
 using namespace OiioTool;
 using namespace ImageBufAlgo;
 
 
 
-static std::string
-compute_sha1(Oiiotool& ot, ImageInput* input, int subimage)
-{
-    SHA1 sha;
-    const ImageSpec& spec(input->spec());
-    if (spec.deep) {
-        // Special handling of deep data
-        DeepData dd;
-        if (!input->read_native_deep_image(subimage, 0, dd)) {
-            std::string err = input->geterror();
-            if (err.empty())
-                err = "could not read image";
-            ot.errorf("-info", "SHA-1: %s", err);
-            return std::string();
-        }
-        // Hash both the sample counts and the data block
-        sha.append(dd.all_samples());
-        sha.append(dd.all_data());
-    } else {
-        imagesize_t size = input->spec().image_bytes(true /*native*/);
-        if (size >= std::numeric_limits<size_t>::max()) {
-            ot.errorf("-info", "SHA-1: unable to compute, image is too big");
-            return std::string();
-        } else if (size != 0) {
-            std::unique_ptr<char[]> buf(new char[size]);
-            if (!input->read_image(TypeDesc::UNKNOWN /*native*/, &buf[0])) {
-                std::string err = input->geterror();
-                if (err.empty())
-                    err = "could not read image";
-                ot.errorf("-info", "SHA-1: %s", err);
-                return std::string();
-            }
-            sha.append(&buf[0], size);
-        }
-    }
-
-    return sha.digest().c_str();
-}
-
-
-
 template<typename T>
 static void
 print_nums(std::ostream& out, int n, const T* val, string_view sep = " ",
@@ -110,11 +68,12 @@ print_nums(std::ostream& out, int n, const T* val, string_view sep = " ",
 template<typename T>
 static bool
 dump_flat_data(std::ostream& out, ImageInput* input,
-               const print_info_options& opt)
+               const pvt::print_info_options& opt, int subimage)
 {
-    const ImageSpec& spec(input->spec());
+    ImageSpec spec = input->spec_dimensions(subimage);
     std::vector<T> buf(spec.image_pixels() * spec.nchannels);
-    if (!input->read_image(BaseTypeFromC<T>::value, &buf[0])) {
+    if (!input->read_image(subimage, 0, 0, spec.nchannels,
+                           BaseTypeFromC<T>::value, &buf[0])) {
         Strutil::print(out, "    dump data Error: could not read image: {}\n",
                        input->geterror());
         return false;
@@ -193,14 +152,14 @@ dump_flat_data(std::ostream& out, ImageInput* input,
 
 
 static void
-dump_data(std::ostream& out, ImageInput* input, const print_info_options& opt)
+dump_data(std::ostream& out, ImageInput* input,
+          const pvt::print_info_options& opt, int subimage)
 {
-    using Strutil::print;
-    const ImageSpec& spec(input->spec());
+    ImageSpec spec = input->spec(subimage);
     if (spec.deep) {
         // Special handling of deep data
         DeepData dd;
-        if (!input->read_native_deep_image(dd)) {
+        if (!input->read_native_deep_image(subimage, 0, dd)) {
             print(out, "    dump data: could not read image\n");
             return;
         }
@@ -223,7 +182,7 @@ dump_data(std::ostream& out, ImageInput* input, const print_info_options& opt)
                         if (s)
                             print(out, " / ");
                         for (int c = 0; c < nc; ++c) {
-                            print(out, " {}=", spec.channelnames[c]);
+                            print(out, " {}=", spec.channel_name(c));
                             if (dd.channeltype(c) == TypeDesc::UINT)
                                 print(out, "{}",
                                       dd.deep_value_uint(pixel, c, s));
@@ -239,7 +198,7 @@ dump_data(std::ostream& out, ImageInput* input, const print_info_options& opt)
     } else {
         OIIO_UNUSED_OK bool ok = true;
         OIIO_DISPATCH_TYPES(ok, "dump_flat_data", dump_flat_data, spec.format,
-                            out, input, opt);
+                            out, input, opt, subimage);
     }
 }
 
@@ -273,78 +232,6 @@ read_input(Oiiotool& ot, const std::string& filename, ImageBuf& img,
 
 
 
-static std::string
-stats_num(float val, int maxval, bool round)
-{
-    // Ensure uniform printing of NaN and Inf on all platforms
-    using Strutil::fmt::format;
-    std::string result;
-    if (isnan(val))
-        result = "nan";
-    else if (isinf(val))
-        result = "inf";
-    else if (maxval == 0) {
-        result = format("{:f}", val);
-    } else {
-        float fval = val * static_cast<float>(maxval);
-        if (round) {
-            int v  = static_cast<int>(roundf(fval));
-            result = format("{}", v);
-        } else {
-            result = format("{:0.2f}", fval);
-        }
-    }
-    return result;
-}
-
-
-// First check oiio:BitsPerSample int attribute.  If not set,
-// fall back on the TypeDesc. return 0 for float types
-// or those that exceed the int range (long long, etc)
-static unsigned long long
-get_intsample_maxval(const ImageSpec& spec)
-{
-    TypeDesc type = spec.format;
-    int bits      = spec.get_int_attribute("oiio:BitsPerSample");
-    if (bits > 0) {
-        if (type.basetype == TypeDesc::UINT8
-            || type.basetype == TypeDesc::UINT16
-            || type.basetype == TypeDesc::UINT32)
-            return ((1LL) << bits) - 1;
-        if (type.basetype == TypeDesc::INT8 || type.basetype == TypeDesc::INT16
-            || type.basetype == TypeDesc::INT32)
-            return ((1LL) << (bits - 1)) - 1;
-    }
-
-    // These correspond to all the int enums in typedesc.h <= int
-    if (type.basetype == TypeDesc::UCHAR)
-        return 0xff;
-    if (type.basetype == TypeDesc::CHAR)
-        return 0x7f;
-    if (type.basetype == TypeDesc::USHORT)
-        return 0xffff;
-    if (type.basetype == TypeDesc::SHORT)
-        return 0x7fff;
-    if (type.basetype == TypeDesc::UINT)
-        return 0xffffffff;
-    if (type.basetype == TypeDesc::INT)
-        return 0x7fffffff;
-
-    return 0;
-}
-
-
-static std::string
-stats_footer(unsigned int maxval)
-{
-    if (maxval == 0)
-        return "(float)";
-    else
-        return Strutil::fmt::format("(of {})", maxval);
-}
-
-
-
 void
 OiioTool::print_stats(std::ostream& out, Oiiotool& ot,
                       const std::string& filename, int subimage, int miplevel,
@@ -355,206 +242,9 @@ OiioTool::print_stats(std::ostream& out, Oiiotool& ot,
         ot.error("stats", input.geterror());
         return;
     }
-    print_stats(out, ot, input, /*input.nativespec(),*/ indent, roi);
-}
-
-
-
-void
-OiioTool::print_stats(std::ostream& out, Oiiotool& ot, const ImageBuf& input,
-                      string_view indent, ROI roi)
-{
-    using Strutil::print;
-
-    PixelStats stats = computePixelStats(input, roi);
-    if (!stats.min.size()) {
-        std::string err = input.geterror();
-        ot.errorfmt("stats", "unable to compute: {}",
-                    err.empty() ? "unspecified error" : err.c_str());
-        return;
-    }
-
-    // The original spec is used, otherwise the bit depth will
-    // be reported incorrectly (as FLOAT)
-    const ImageSpec& originalspec(input.nativespec());
-    unsigned int maxval = (unsigned int)get_intsample_maxval(originalspec);
-
-    print(out, "{}Stats Min: ", indent);
-    for (unsigned int i = 0; i < stats.min.size(); ++i) {
-        Strutil::print(out, "{} ", stats_num(stats.min[i], maxval, true));
-    }
-    Strutil::print(out, "{}\n", stats_footer(maxval));
-
-    print(out, "{}Stats Max: ", indent);
-    for (unsigned int i = 0; i < stats.max.size(); ++i) {
-        Strutil::print(out, "{} ", stats_num(stats.max[i], maxval, true));
-    }
-    print(out, "{}\n", stats_footer(maxval));
-
-    print(out, "{}Stats Avg: ", indent);
-    for (unsigned int i = 0; i < stats.avg.size(); ++i) {
-        print(out, "{} ", stats_num(stats.avg[i], maxval, false));
-    }
-    print(out, "{}\n", stats_footer(maxval));
-
-    print(out, "{}Stats StdDev: ", indent);
-    for (unsigned int i = 0; i < stats.stddev.size(); ++i) {
-        print(out, "{} ", stats_num(stats.stddev[i], maxval, false));
-    }
-    print(out, "{}\n", stats_footer(maxval));
-
-    print(out, "{}Stats NanCount: ", indent);
-    for (unsigned int i = 0; i < stats.nancount.size(); ++i) {
-        print(out, "{} ", (unsigned long long)stats.nancount[i]);
-    }
-    print(out, "\n");
-
-    print(out, "{}Stats InfCount: ", indent);
-    for (unsigned int i = 0; i < stats.infcount.size(); ++i) {
-        print(out, "{} ", (unsigned long long)stats.infcount[i]);
-    }
-    print(out, "\n");
-
-    print(out, "{}Stats FiniteCount: ", indent);
-    for (unsigned int i = 0; i < stats.finitecount.size(); ++i) {
-        print(out, "{} ", (unsigned long long)stats.finitecount[i]);
-    }
-    print(out, "\n");
-
-    if (input.deep()) {
-        const DeepData* dd(input.deepdata());
-        size_t npixels      = dd->pixels();
-        size_t totalsamples = 0, emptypixels = 0;
-        size_t maxsamples = 0, minsamples = std::numeric_limits<size_t>::max();
-        size_t maxsamples_npixels = 0;
-        float mindepth            = std::numeric_limits<float>::max();
-        float maxdepth            = -std::numeric_limits<float>::max();
-        Imath::V3i maxsamples_pixel(-1, -1, -1);
-        Imath::V3i mindepth_pixel(-1, -1, -1), maxdepth_pixel(-1, -1, -1);
-        Imath::V3i nonfinite_pixel(-1, -1, -1);
-        int nonfinite_pixel_samp(-1), nonfinite_pixel_chan(-1);
-        int nchannels        = dd->channels();
-        int depthchannel     = -1;
-        long long nonfinites = 0;
-        for (int c = 0; c < nchannels; ++c)
-            if (Strutil::iequals(originalspec.channelnames[c], "Z"))
-                depthchannel = c;
-        int xend = originalspec.x + originalspec.width;
-        int yend = originalspec.y + originalspec.height;
-        int zend = originalspec.z + originalspec.depth;
-        size_t p = 0;
-        std::vector<size_t> nsamples_histogram;
-        for (int z = originalspec.z; z < zend; ++z) {
-            for (int y = originalspec.y; y < yend; ++y) {
-                for (int x = originalspec.x; x < xend; ++x, ++p) {
-                    size_t samples = input.deep_samples(x, y, z);
-                    totalsamples += samples;
-                    if (samples == maxsamples)
-                        ++maxsamples_npixels;
-                    if (samples > maxsamples) {
-                        maxsamples = samples;
-                        maxsamples_pixel.setValue(x, y, z);
-                        maxsamples_npixels = 1;
-                    }
-                    if (samples < minsamples)
-                        minsamples = samples;
-                    if (samples == 0)
-                        ++emptypixels;
-                    if (samples >= nsamples_histogram.size())
-                        nsamples_histogram.resize(samples + 1, 0);
-                    nsamples_histogram[samples] += 1;
-                    for (unsigned int s = 0; s < samples; ++s) {
-                        for (int c = 0; c < nchannels; ++c) {
-                            float d = input.deep_value(x, y, z, c, s);
-                            if (!isfinite(d)) {
-                                if (nonfinites++ == 0) {
-                                    nonfinite_pixel.setValue(x, y, z);
-                                    nonfinite_pixel_samp = s;
-                                    nonfinite_pixel_chan = c;
-                                }
-                            }
-                            if (depthchannel == c) {
-                                if (d < mindepth) {
-                                    mindepth = d;
-                                    mindepth_pixel.setValue(x, y, z);
-                                }
-                                if (d > maxdepth) {
-                                    maxdepth = d;
-                                    maxdepth_pixel.setValue(x, y, z);
-                                }
-                            }
-                        }
-                    }
-                }
-            }
-        }
-        print(out, "{}Min deep samples in any pixel : {}\n", indent,
-              minsamples);
-        print(out, "{}Max deep samples in any pixel : {}\n", indent,
-              maxsamples);
-        print(out,
-              "{}{} pixel{} had the max of {} samples, including (x={}, y={})\n",
-              indent, maxsamples_npixels, maxsamples_npixels > 1 ? "s" : "",
-              maxsamples, maxsamples_pixel.x, maxsamples_pixel.y);
-        print(out, "{}Average deep samples per pixel: {:.2f}\n", indent,
-              double(totalsamples) / double(npixels));
-        print(out, "{}Total deep samples in all pixels: {}\n", indent,
-              totalsamples);
-        print(out, "{}Pixels with deep samples   : {}\n", indent,
-              (npixels - emptypixels));
-        print(out, "{}Pixels with no deep samples: {}\n", indent, emptypixels);
-        print(out, "{}Samples/pixel histogram:\n", indent);
-        size_t grandtotal = 0;
-        for (size_t i = 0, e = nsamples_histogram.size(); i < e; ++i)
-            grandtotal += nsamples_histogram[i];
-        size_t binstart = 0, bintotal = 0;
-        for (size_t i = 0, e = nsamples_histogram.size(); i < e; ++i) {
-            bintotal += nsamples_histogram[i];
-            if (i < 8 || i == (e - 1) || OIIO::ispow2(i + 1)) {
-                // batch by powers of 2, unless it's a small number
-                if (i == binstart)
-                    print(out, "{}  {:3}    ", indent, i);
-                else
-                    print(out, "{}  {:3}-{:3}", indent, binstart, i);
-                print(out, " : {:8} ({:4.1f}%)\n", bintotal,
-                      (100.0 * bintotal) / grandtotal);
-                binstart = i + 1;
-                bintotal = 0;
-            }
-        }
-        if (depthchannel >= 0) {
-            print(out, "{}Minimum depth was {} at ({}, {})\n", indent, mindepth,
-                  mindepth_pixel.x, mindepth_pixel.y);
-            print(out, "{}Maximum depth was {} at ({}, {})\n", indent, maxdepth,
-                  maxdepth_pixel.x, maxdepth_pixel.y);
-        }
-        if (nonfinites > 0) {
-            Strutil::print(
-                out,
-                "{}Nonfinite values: {}, including (x={}, y={}, chan={}, samp={})\n",
-                indent, nonfinites, nonfinite_pixel.x, nonfinite_pixel.y,
-                input.spec().channelnames[nonfinite_pixel_chan].c_str(),
-                nonfinite_pixel_samp);
-        }
-    } else {
-        std::vector<float> constantValues(input.spec().nchannels);
-        if (isConstantColor(input, &constantValues[0])) {
-            print(out, "{}Constant: Yes\n", indent);
-            print(out, "{}Constant Color: ", indent);
-            for (unsigned int i = 0; i < constantValues.size(); ++i) {
-                print(out, "{} ", stats_num(constantValues[i], maxval, false));
-            }
-            print(out, "{}\n", stats_footer(maxval));
-        } else {
-            print(out, "{}Constant: No\n", indent);
-        }
-
-        if (isMonochrome(input)) {
-            print(out, "{}Monochrome: Yes\n", indent);
-        } else {
-            print(out, "{}Monochrome: No\n", indent);
-        }
-    }
+    std::string err;
+    if (!pvt::print_stats(out, indent, input, input.nativespec(), roi, err))
+        ot.errorfmt("stats", "unable to compute: {}", err);
 }
 
 
@@ -584,8 +274,9 @@ static void
 print_info_subimage(std::ostream& out, Oiiotool& ot, int current_subimage,
                     int num_of_subimages, int nmip, const ImageSpec& spec,
                     ImageRec* img, ImageInput* input,
-                    const std::string& filename, const print_info_options& opt,
-                    std::regex& field_re, std::regex& field_exclude_re,
+                    const std::string& filename,
+                    const pvt::print_info_options& opt, std::regex& field_re,
+                    std::regex& field_exclude_re,
                     ImageSpec::SerialFormat serformat,
                     ImageSpec::SerialVerbose verbose)
 {
@@ -606,8 +297,11 @@ print_info_subimage(std::ostream& out, Oiiotool& ot, int current_subimage,
         && (opt.metamatch.empty() || std::regex_search("sha-1", field_re))) {
         // Before sha-1, be sure to point back to the highest-res MIP level
         ImageSpec tmpspec;
-        std::string sha = compute_sha1(ot, input, current_subimage);
-        if (serformat == ImageSpec::SerialText)
+        std::string err;
+        std::string sha = compute_sha1(input, current_subimage, 0, err);
+        if (!err.empty())
+            ot.errorfmt("-info", "SHA-1: {}", err);
+        else if (serformat == ImageSpec::SerialText)
             lines.insert(lines.begin() + 1, format("    SHA-1: {}", sha));
         else if (serformat == ImageSpec::SerialXML)
             lines.insert(lines.begin() + 1, format("<SHA1>{}</SHA1>", sha));
@@ -740,7 +434,7 @@ print_info_subimage(std::ostream& out, Oiiotool& ot, int current_subimage,
     if (input && opt.dumpdata) {
         ImageSpec tmp;
         input->seek_subimage(current_subimage, 0, tmp);
-        dump_data(out, input, opt);
+        dump_data(out, input, opt, current_subimage);
     }
 
     if (opt.compute_stats
@@ -762,9 +456,14 @@ print_info_subimage(std::ostream& out, Oiiotool& ot, int current_subimage,
             if (input)
                 print_stats(out, ot, filename, /*spec,*/ current_subimage, m,
                             nmip > 1 ? "      " : "    ", opt.roi);
-            else if (img)
-                print_stats(out, ot, (*img)(current_subimage, m),
-                            nmip > 1 ? "      " : "    ", opt.roi);
+            else if (img) {
+                std::string err;
+                if (!pvt::print_stats(out, nmip > 1 ? "      " : "    ",
+                                      (*img)(current_subimage, m),
+                                      (*img)(current_subimage, m).nativespec(),
+                                      opt.roi, err))
+                    ot.errorfmt("stats", "unable to compute: {}", err);
+            }
             if (!opt.subimages)
                 break;
         }
@@ -775,7 +474,7 @@ print_info_subimage(std::ostream& out, Oiiotool& ot, int current_subimage,
 
 bool
 OiioTool::print_info(std::ostream& out, Oiiotool& ot, ImageRec* img,
-                     const print_info_options& opt, std::string& error)
+                     const pvt::print_info_options& opt, std::string& error)
 {
     using Strutil::fmt::format;
     error.clear();
@@ -833,8 +532,8 @@ OiioTool::print_info(std::ostream& out, Oiiotool& ot, ImageRec* img,
 
 bool
 OiioTool::print_info(std::ostream& out, Oiiotool& ot,
-                     const std::string& filename, const print_info_options& opt,
-                     std::string& error)
+                     const std::string& filename,
+                     const pvt::print_info_options& opt, std::string& error)
 {
     using Strutil::fmt::format;
     error.clear();
