@@ -11,20 +11,11 @@
 #include <memory>
 #include <numeric>
 
+#include <OpenImageIO/Imath.h>
 #include <OpenImageIO/platform.h>
 
 #include <boost/version.hpp>
 
-#if OIIO_CPLUSPLUS_VERSION >= 17
-using std::gcd;
-#elif BOOST_VERSION >= 106900
-#    include <boost/integer/common_factor_rt.hpp>
-using boost::integer::gcd;
-#else
-#    include <boost/math/common_factor_rt.hpp>
-using boost::math::gcd;
-#endif
-
 #include "exr_pvt.h"
 
 #include <OpenEXR/openexr.h>
@@ -110,54 +101,52 @@ oiio_exr_read_func(exr_const_context_t ctxt, void* userdata, void* buffer,
 class OpenEXRCoreInput final : public ImageInput {
 public:
     OpenEXRCoreInput();
-    virtual ~OpenEXRCoreInput() { close(); }
-    virtual const char* format_name(void) const override { return "openexr"; }
-    virtual int supports(string_view feature) const override
+    ~OpenEXRCoreInput() override { close(); }
+    const char* format_name(void) const override { return "openexr"; }
+    int supports(string_view feature) const override
     {
         return (feature == "arbitrary_metadata"
                 || feature == "exif"  // Because of arbitrary_metadata
                 || feature == "iptc"  // Because of arbitrary_metadata
                 || feature == "ioproxy");
     }
-    virtual bool valid_file(const std::string& filename) const override;
-    virtual bool open(const std::string& name, ImageSpec& newspec,
-                      const ImageSpec& config) override;
-    virtual bool open(const std::string& name, ImageSpec& newspec) override
+    bool valid_file(const std::string& filename) const override;
+    bool open(const std::string& name, ImageSpec& newspec,
+              const ImageSpec& config) override;
+    bool open(const std::string& name, ImageSpec& newspec) override
     {
         return open(name, newspec, ImageSpec());
     }
-    virtual bool close() override;
-    virtual int current_subimage(void) const override { return m_subimage; }
-    virtual int current_miplevel(void) const override { return m_miplevel; }
-    virtual bool seek_subimage(int subimage, int miplevel) override;
-    virtual ImageSpec spec(int subimage, int miplevel) override;
-    virtual ImageSpec spec_dimensions(int subimage, int miplevel) override;
-    virtual bool read_native_scanline(int subimage, int miplevel, int y, int z,
-                                      void* data) override;
-    virtual bool read_native_scanlines(int subimage, int miplevel, int ybegin,
-                                       int yend, int z, void* data) override;
-    virtual bool read_native_scanlines(int subimage, int miplevel, int ybegin,
-                                       int yend, int z, int chbegin, int chend,
-                                       void* data) override;
-    virtual bool read_native_tile(int subimage, int miplevel, int x, int y,
-                                  int z, void* data) override;
-    virtual bool read_native_tiles(int subimage, int miplevel, int xbegin,
-                                   int xend, int ybegin, int yend, int zbegin,
-                                   int zend, void* data) override;
-    virtual bool read_native_tiles(int subimage, int miplevel, int xbegin,
-                                   int xend, int ybegin, int yend, int zbegin,
-                                   int zend, int chbegin, int chend,
-                                   void* data) override;
-    virtual bool read_native_deep_scanlines(int subimage, int miplevel,
-                                            int ybegin, int yend, int z,
-                                            int chbegin, int chend,
-                                            DeepData& deepdata) override;
-    virtual bool read_native_deep_tiles(int subimage, int miplevel, int xbegin,
-                                        int xend, int ybegin, int yend,
-                                        int zbegin, int zend, int chbegin,
-                                        int chend, DeepData& deepdata) override;
+    bool close() override;
+    int current_subimage(void) const override { return m_subimage; }
+    int current_miplevel(void) const override { return m_miplevel; }
+    bool seek_subimage(int subimage, int miplevel) override;
+    ImageSpec spec(int subimage, int miplevel) override;
+    ImageSpec spec_dimensions(int subimage, int miplevel) override;
+    bool read_native_scanline(int subimage, int miplevel, int y, int z,
+                              void* data) override;
+    bool read_native_scanlines(int subimage, int miplevel, int ybegin, int yend,
+                               int z, void* data) override;
+    bool read_native_scanlines(int subimage, int miplevel, int ybegin, int yend,
+                               int z, int chbegin, int chend,
+                               void* data) override;
+    bool read_native_tile(int subimage, int miplevel, int x, int y, int z,
+                          void* data) override;
+    bool read_native_tiles(int subimage, int miplevel, int xbegin, int xend,
+                           int ybegin, int yend, int zbegin, int zend,
+                           void* data) override;
+    bool read_native_tiles(int subimage, int miplevel, int xbegin, int xend,
+                           int ybegin, int yend, int zbegin, int zend,
+                           int chbegin, int chend, void* data) override;
+    bool read_native_deep_scanlines(int subimage, int miplevel, int ybegin,
+                                    int yend, int z, int chbegin, int chend,
+                                    DeepData& deepdata) override;
+    bool read_native_deep_tiles(int subimage, int miplevel, int xbegin,
+                                int xend, int ybegin, int yend, int zbegin,
+                                int zend, int chbegin, int chend,
+                                DeepData& deepdata) override;
 
-    virtual bool set_ioproxy(Filesystem::IOProxy* ioproxy) override
+    bool set_ioproxy(Filesystem::IOProxy* ioproxy) override
     {
         OIIO_ASSERT(!m_exr_context);
         m_userdata.m_io = ioproxy;
@@ -359,7 +348,7 @@ OpenEXRCoreInput::open(const std::string& name, ImageSpec& newspec,
     if (param)
         m_userdata.m_io = param->get<Filesystem::IOProxy*>();
 
-    // Quick check to immediately reject nonexistant or non-exr files.
+    // Quick check to immediately reject nonexistent or non-exr files.
     //KDTDISABLE quick checks are still file iOPs, let the file open handle this
     //KDTDISABLE if (!m_io && !Filesystem::is_regular(name)) {
     //KDTDISABLE     errorf("Could not open file \"%s\"", name);
@@ -688,7 +677,10 @@ OpenEXRCoreInput::PartInfo::parse_header(OpenEXRCoreInput* in,
             break;
         }
 
-        case EXR_ATTR_STRING: spec.attribute(oname, attr->string->str); break;
+        case EXR_ATTR_STRING:
+            if (attr->string && attr->string->str && attr->string->str[0])
+                spec.attribute(oname, attr->string->str);
+            break;
         case EXR_ATTR_STRING_VECTOR: {
             std::vector<ustring> ustrvec(attr->stringvector->n_strings);
             for (int32_t i = 0, e = attr->stringvector->n_strings; i < e; ++i)
@@ -964,8 +956,8 @@ OpenEXRCoreInput::PartInfo::query_channels(OpenEXRCoreInput* in,
             spec.z_channel = c;
         if (cnh[c].xSampling != 1 || cnh[c].ySampling != 1) {
             ok = false;
-            in->errorfmt(
-                "Subsampled channels are not supported (channel \"{}\" has sampling {},{}).",
+            in->errorf(
+                "Subsampled channels are not supported (channel \"%s\" has sampling %d,%d).",
                 cnh[c].fullname, cnh[c].xSampling, cnh[c].ySampling);
             // FIXME: Some day, we should handle channel subsampling.
         }
