@@ -65,23 +65,23 @@ class OpenVDBInput final : public ImageInput {
 
 public:
     OpenVDBInput() { init(); }
-    virtual ~OpenVDBInput() { close(); }
+    ~OpenVDBInput() override { close(); }
 
-    virtual const char* format_name(void) const override { return "openvdb"; }
-    virtual int supports(string_view feature) const override
+    const char* format_name(void) const override { return "openvdb"; }
+    int supports(string_view feature) const override
     {
         return (feature == "arbitrary_metadata");
     }
-    virtual bool valid_file(const std::string& filename) const override;
-    virtual bool open(const std::string& name, ImageSpec& newspec) override;
-    virtual bool close() override;
-    virtual int current_subimage(void) const override;
-    virtual bool seek_subimage(int subimage, int miplevel) override;
-    virtual bool seek_subimage_nolock(int subimage, int miplevel);
-    virtual bool read_native_scanline(int subimage, int miplevel, int y, int z,
-                                      void* data) override;
-    virtual bool read_native_tile(int subimage, int miplevel, int x, int y,
-                                  int z, void* data) override;
+    bool valid_file(const std::string& filename) const override;
+    bool open(const std::string& name, ImageSpec& newspec) override;
+    bool close() override;
+    int current_subimage(void) const override;
+    bool seek_subimage(int subimage, int miplevel) override;
+    bool seek_subimage_nolock(int subimage, int miplevel);
+    bool read_native_scanline(int subimage, int miplevel, int y, int z,
+                              void* data) override;
+    bool read_native_tile(int subimage, int miplevel, int x, int y, int z,
+                          void* data) override;
 
     ImageSpec spec(int subimage, int miplevel) override;
     ImageSpec spec_dimensions(int subimage, int miplevel) override;
@@ -295,7 +295,7 @@ openVDB(const std::string& filename, const ImageInput* errReport)
     if (!f)
         return nullptr;
 
-    // Endianess of OPENVDB_MAGIC isn't clear, so just leave as is
+    // Endianness of OPENVDB_MAGIC isn't clear, so just leave as is
     int32_t magic;
     static_assert(sizeof(magic) == sizeof(OPENVDB_MAGIC),
                   "Magic type not the same size");
@@ -556,6 +556,10 @@ bool
 OpenVDBInput::read_native_tile(int subimage, int miplevel, int x, int y, int z,
                                void* data)
 {
+    OIIO_PRAGMA_WARNING_PUSH
+#if OIIO_GNUC_VERSION >= 120100
+    OIIO_GCC_ONLY_PRAGMA(GCC diagnostic ignored "-Wstringop-overflow")
+#endif
     lock_guard lock(*this);
     if (!seek_subimage_nolock(subimage, miplevel))
         return false;
@@ -573,11 +577,12 @@ OpenVDBInput::read_native_tile(int subimage, int miplevel, int x, int y, int z,
     default: break;
     }
     return false;
+    OIIO_PRAGMA_WARNING_POP
 }
 
 
 
-// Obligatory material to make this a recognizeable imageio plugin:
+// Obligatory material to make this a recognizable imageio plugin:
 OIIO_PLUGIN_EXPORTS_BEGIN
 
 OIIO_EXPORT ImageInput*
