@@ -29,6 +29,7 @@
 #    undef copysign
 #endif
 
+#include <OpenImageIO/Imath.h>
 #include <OpenImageIO/deepdata.h>
 #include <OpenImageIO/imagebuf.h>
 #include <OpenImageIO/imagecache.h>
@@ -135,6 +136,7 @@ template<> struct PyTypeForCType<unsigned short> { typedef py::int_ type; };
 template<> struct PyTypeForCType<char> { typedef py::int_ type; };
 template<> struct PyTypeForCType<unsigned char> { typedef py::int_ type; };
 template<> struct PyTypeForCType<int64_t> { typedef py::int_ type; };
+template<> struct PyTypeForCType<uint64_t> { typedef py::int_ type; };
 template<> struct PyTypeForCType<float> { typedef py::float_ type; };
 template<> struct PyTypeForCType<half> { typedef py::float_ type; };
 template<> struct PyTypeForCType<double> { typedef py::float_ type; };
@@ -443,6 +445,7 @@ template<typename T>
 inline py::object
 C_to_val_or_tuple(const T* vals, TypeDesc type, int nvalues = 1)
 {
+    OIIO_DASSERT(vals && nvalues);
     size_t n = type.numelements() * type.aggregate * nvalues;
     if (n == 1 && !type.arraylen)
         return typename PyTypeForCType<T>::type(vals[0]);
@@ -505,6 +508,14 @@ attribute_typed(T& myobj, string_view name, TypeDesc type, const POBJ& dataobj)
 
 
 
+// `data` points to values of `type`. Make a python object that represents
+// them.
+py::object
+make_pyobject(const void* data, TypeDesc type, int nvalues = 1,
+              py::object defaultvalue = py::none());
+
+
+
 template<typename T>
 py::object
 getattribute_typed(const T& obj, const std::string& name,
@@ -516,25 +527,7 @@ getattribute_typed(const T& obj, const std::string& name,
     bool ok    = obj.getattribute(name, type, data);
     if (!ok)
         return py::none();  // None
-    if (type.basetype == TypeDesc::INT)
-        return C_to_val_or_tuple((const int*)data, type);
-    if (type.basetype == TypeDesc::UINT)
-        return C_to_val_or_tuple((const unsigned int*)data, type);
-    if (type.basetype == TypeDesc::INT16)
-        return C_to_val_or_tuple((const short*)data, type);
-    if (type.basetype == TypeDesc::UINT16)
-        return C_to_val_or_tuple((const unsigned short*)data, type);
-    if (type.basetype == TypeDesc::UINT8)
-        return C_to_val_or_tuple((const unsigned char*)data, type);
-    if (type.basetype == TypeDesc::FLOAT)
-        return C_to_val_or_tuple((const float*)data, type);
-    if (type.basetype == TypeDesc::DOUBLE)
-        return C_to_val_or_tuple((const double*)data, type);
-    if (type.basetype == TypeDesc::HALF)
-        return C_to_val_or_tuple((const half*)data, type);
-    if (type.basetype == TypeDesc::STRING)
-        return C_to_val_or_tuple((const char**)data, type);
-    return py::none();
+    return make_pyobject(data, type);
 }
 
 
@@ -613,40 +606,6 @@ make_numpy_array(TypeDesc format, void* data, int dims, size_t chans,
 
 
 
-inline py::object
-ParamValue_getitem(const ParamValue& self, bool allitems = false,
-                   py::object defaultvalue = py::none())
-{
-    TypeDesc t = self.type();
-    int nvals  = allitems ? self.nvalues() : 1;
-
-#define ParamValue_convert_dispatch(TYPE)                                  \
-case TypeDesc::TYPE:                                                       \
-    return C_to_val_or_tuple((CType<TypeDesc::TYPE>::type*)self.data(), t, \
-                             nvals)
-
-    switch (t.basetype) {
-        ParamValue_convert_dispatch(UCHAR);
-        // ParamValue_convert_dispatch(CHAR);
-        ParamValue_convert_dispatch(USHORT);
-        ParamValue_convert_dispatch(SHORT);
-        ParamValue_convert_dispatch(UINT);
-        ParamValue_convert_dispatch(INT);
-        // ParamValue_convert_dispatch(ULONGLONG);
-        // ParamValue_convert_dispatch(LONGLONG);
-        ParamValue_convert_dispatch(HALF);
-        ParamValue_convert_dispatch(FLOAT);
-        ParamValue_convert_dispatch(DOUBLE);
-    case TypeDesc::STRING:
-        return C_to_val_or_tuple((const char**)self.data(), t, nvals);
-    default: return defaultvalue;
-    }
-
-#undef ParamValue_convert_dispatch
-}
-
-
-
 template<typename C>
 inline void
 delegate_setitem(C& self, const std::string& key, py::object obj)
@@ -657,6 +616,8 @@ delegate_setitem(C& self, const std::string& key, py::object obj)
         self[key] = int(obj.template cast<py::int_>());
     else if (py::isinstance<py::str>(obj))
         self[key] = std::string(obj.template cast<py::str>());
+    else if (py::isinstance<py::bytes>(obj))
+        self[key] = std::string(obj.template cast<py::bytes>());
     else
         throw std::invalid_argument("Bad type for __setitem__");
 }
