@@ -7,6 +7,8 @@
 #include <iostream>
 #include <memory>
 
+#include <OpenImageIO/half.h>
+
 #include <OpenImageIO/fmath.h>
 #include <OpenImageIO/imageio.h>
 #include <OpenImageIO/platform.h>
@@ -55,19 +57,19 @@ OIIO_PLUGIN_NAMESPACE_BEGIN
 class RawInput final : public ImageInput {
 public:
     RawInput() {}
-    virtual ~RawInput() { close(); }
-    virtual const char* format_name(void) const override { return "raw"; }
-    virtual int supports(string_view feature) const override
+    ~RawInput() override { close(); }
+    const char* format_name(void) const override { return "raw"; }
+    int supports(string_view feature) const override
     {
         return (feature == "exif"
                 /* not yet? || feature == "iptc"*/);
     }
-    virtual bool open(const std::string& name, ImageSpec& newspec) override;
-    virtual bool open(const std::string& name, ImageSpec& newspec,
-                      const ImageSpec& config) override;
-    virtual bool close() override;
-    virtual bool read_native_scanline(int subimage, int miplevel, int y, int z,
-                                      void* data) override;
+    bool open(const std::string& name, ImageSpec& newspec) override;
+    bool open(const std::string& name, ImageSpec& newspec,
+              const ImageSpec& config) override;
+    bool close() override;
+    bool read_native_scanline(int subimage, int miplevel, int y, int z,
+                              void* data) override;
 
 private:
     bool process();
@@ -100,6 +102,7 @@ private:
     void get_makernotes_sony();
     void get_lensinfo();
     void get_shootinginfo();
+    void get_colorinfo();
 
     template<typename T> static bool allval(cspan<T> d, T v = T(0))
     {
@@ -223,7 +226,7 @@ OIIO_EXPORT int raw_imageio_version = OIIO_PLUGIN_VERSION;
 OIIO_EXPORT const char*
 raw_imageio_library_version()
 {
-    return ustring::sprintf("libraw %s", libraw_version()).c_str();
+    return ustring::fmtformat("libraw {}", libraw_version()).c_str();
 }
 
 OIIO_EXPORT ImageInput*
@@ -409,7 +412,8 @@ RawInput::open_raw(bool unpack, const std::string& name,
 
 #ifdef _WIN32
     // Convert to wide chars, just on Windows.
-    int ret = m_processor->open_file(Strutil::utf8_to_utf16(name).c_str());
+    int ret = m_processor->open_file(
+        Strutil::utf8_to_utf16wstring(name).c_str());
 #else
     int ret = m_processor->open_file(name.c_str());
 #endif
@@ -429,6 +433,8 @@ RawInput::open_raw(bool unpack, const std::string& name,
         m_unpacked = true;
     }
 
+    //Store flip before it is potentially overwritten
+    int original_flip = m_processor->imgdata.sizes.flip;
     m_processor->adjust_sizes_info_only();
 
     // Process image at half size if "raw:half_size" is not 0
@@ -544,7 +550,7 @@ RawInput::open_raw(bool unpack, const std::string& name,
         m_processor->imgdata.params.gamm[1]      = 12.92;
     } else if (Strutil::iequals(cs, "sRGB-linear")
                || Strutil::iequals(cs, "linear") /* DEPRECATED */) {
-        // Request "sRGB" primaries, linear reponse
+        // Request "sRGB" primaries, linear response
         m_processor->imgdata.params.output_color = 1;
         m_processor->imgdata.params.gamm[0]      = 1.0;
         m_processor->imgdata.params.gamm[1]      = 1.0;
@@ -938,10 +944,18 @@ RawInput::open_raw(bool unpack, const std::string& name,
         m_spec.attribute("raw:Orientation", ori);
     m_spec.attribute("Orientation", 1);
 
+    // If user flip is set to 1, it means we ignore the flip
+    // Let's set the orientation exif flags to the original flip
+    // value so that it is still displayed correctly
+    if (config.get_int_attribute("raw:user_flip", -1) == 1) {
+        m_spec.attribute("Orientation", original_flip);
+    }
+
     // FIXME -- thumbnail possibly in m_processor->imgdata.thumbnail
 
     get_lensinfo();
     get_shootinginfo();
+    get_colorinfo();
     get_makernotes();
 
     return true;
@@ -1402,6 +1416,31 @@ RawInput::get_shootinginfo()
 
 
 
+void
+RawInput::get_colorinfo()
+{
+#if LIBRAW_VERSION >= LIBRAW_MAKE_VERSION(0, 18, 0)
+    add("raw", "pre_mul",
+        cspan<float>(&(m_processor->imgdata.color.pre_mul[0]),
+                     &(m_processor->imgdata.color.pre_mul[4])),
+        false, 0.f);
+    add("raw", "cam_mul",
+        cspan<float>(&(m_processor->imgdata.color.cam_mul[0]),
+                     &(m_processor->imgdata.color.cam_mul[4])),
+        false, 0.f);
+    add("raw", "rgb_cam",
+        cspan<float>(&(m_processor->imgdata.color.rgb_cam[0][0]),
+                     &(m_processor->imgdata.color.rgb_cam[2][4])),
+        false, 0.f);
+    add("raw", "cam_xyz",
+        cspan<float>(&(m_processor->imgdata.color.cam_xyz[0][0]),
+                     &(m_processor->imgdata.color.cam_xyz[3][3])),
+        false, 0.f);
+#endif
+}
+
+
+
 bool
 RawInput::close()
 {
