@@ -22,18 +22,18 @@ using namespace RLA_pvt;
 class RLAInput final : public ImageInput {
 public:
     RLAInput() { init(); }
-    virtual ~RLAInput() { close(); }
-    virtual const char* format_name(void) const override { return "rla"; }
-    virtual bool open(const std::string& name, ImageSpec& newspec) override;
-    virtual int current_subimage(void) const override
+    ~RLAInput() override { close(); }
+    const char* format_name(void) const override { return "rla"; }
+    bool open(const std::string& name, ImageSpec& newspec) override;
+    int current_subimage(void) const override
     {
         lock_guard lock(*this);
         return m_subimage;
     }
-    virtual bool seek_subimage(int subimage, int miplevel) override;
-    virtual bool close() override;
-    virtual bool read_native_scanline(int subimage, int miplevel, int y, int z,
-                                      void* data) override;
+    bool seek_subimage(int subimage, int miplevel) override;
+    bool close() override;
+    bool read_native_scanline(int subimage, int miplevel, int y, int z,
+                              void* data) override;
 
 private:
     std::string m_filename;            ///< Stash the filename
@@ -110,7 +110,7 @@ private:
             uint16_t s[2];
             uint32_t i;
         } u;
-        read(&u.c, 4);  // because it's char, it didn't swap endian
+        read((char*)&u.c, 4);  // because it's char, it didn't swap endian
         uint16_t s[2] = { u.s[0], u.s[1] };
         uint32_t i    = u.i;
         if (littleendian()) {
@@ -127,7 +127,7 @@ private:
 
 
 
-// Obligatory material to make this a recognizeable imageio plugin:
+// Obligatory material to make this a recognizable imageio plugin:
 OIIO_PLUGIN_EXPORTS_BEGIN
 
 OIIO_EXPORT ImageInput*
@@ -221,7 +221,7 @@ RLAInput::seek_subimage(int subimage, int miplevel)
         return true;  // already on the right level
 
     // RLA images allow multiple subimages; they are simply concatenated
-    // together, wth image N's header field NextOffset giving the
+    // together, with image N's header field NextOffset giving the
     // absolute offset of the start of image N+1.
     int diff = subimage - current_subimage();
     if (subimage - current_subimage() < 0) {
@@ -325,8 +325,10 @@ RLAInput::seek_subimage(int subimage, int miplevel)
             && (aux_type == TypeDesc::FLOAT || aux_type == TypeDesc::INT32
                 || aux_type == TypeDesc::UINT32)) {
             z_channel = m_rla.NumOfColorChannels + m_rla.NumOfMatteChannels;
-            m_spec.z_channel               = z_channel;
-            m_spec.channelnames[z_channel] = "Z";
+            m_spec.z_channel = z_channel;
+            if (m_spec.channelnames.size() < size_t(z_channel + 1))
+                m_spec.channelnames.resize(z_channel + 1);
+            m_spec.channelnames[z_channel] = std::string("Z");
         }
     }
     m_stride += m_rla.NumOfAuxChannels * aux_type.size();
@@ -399,7 +401,7 @@ RLAInput::seek_subimage(int subimage, int miplevel)
             m_spec.attribute("oiio:ColorSpace", "Linear");
         else {
             m_spec.attribute("oiio:ColorSpace",
-                             Strutil::sprintf("GammaCorrected%.2g", gamma));
+                             Strutil::sprintf("Gamma%.2g", gamma));
             m_spec.attribute("oiio:Gamma", gamma);
         }
     }
@@ -487,7 +489,8 @@ RLAInput::decode_rle_span(unsigned char* buf, int n, int stride,
         signed char count = (signed char)encoded[e++];
         if (count >= 0) {
             // run count positive: value repeated count+1 times
-            for (int i = 0; i <= count && n; ++i, buf += stride, --n)
+            for (int i = 0; i <= count && n && e < elen;
+                 ++i, buf += stride, --n)
                 *buf = encoded[e];
             ++e;
         } else {
