@@ -27,17 +27,17 @@ using namespace RLA_pvt;
 class RLAOutput final : public ImageOutput {
 public:
     RLAOutput();
-    virtual ~RLAOutput();
-    virtual const char* format_name(void) const override { return "rla"; }
-    virtual int supports(string_view feature) const override;
-    virtual bool open(const std::string& name, const ImageSpec& spec,
-                      OpenMode mode = Create) override;
-    virtual bool close() override;
-    virtual bool write_scanline(int y, int z, TypeDesc format, const void* data,
-                                stride_t xstride) override;
-    virtual bool write_tile(int x, int y, int z, TypeDesc format,
-                            const void* data, stride_t xstride,
-                            stride_t ystride, stride_t zstride) override;
+    ~RLAOutput() override;
+    const char* format_name(void) const override { return "rla"; }
+    int supports(string_view feature) const override;
+    bool open(const std::string& name, const ImageSpec& spec,
+              OpenMode mode = Create) override;
+    bool close() override;
+    bool write_scanline(int y, int z, TypeDesc format, const void* data,
+                        stride_t xstride) override;
+    bool write_tile(int x, int y, int z, TypeDesc format, const void* data,
+                    stride_t xstride, stride_t ystride,
+                    stride_t zstride) override;
 
 private:
     std::string m_filename;  ///< Stash the filename
@@ -92,7 +92,7 @@ private:
 
 
 
-// Obligatory material to make this a recognizeable imageio plugin:
+// Obligatory material to make this a recognizable imageio plugin:
 OIIO_PLUGIN_EXPORTS_BEGIN
 
 OIIO_EXPORT ImageOutput*
@@ -175,7 +175,7 @@ RLAOutput::open(const std::string& name, const ImageSpec& userspec,
     }
     if (m_spec.width > 65535 || m_spec.height > 65535) {
         errorf(
-            "Image resolution %d x %d too large for RLA (maxiumum 65535x65535)",
+            "Image resolution %d x %d too large for RLA (maximum 65535x65535)",
             m_spec.width, m_spec.height);
         return false;
     }
@@ -210,7 +210,7 @@ RLAOutput::open(const std::string& name, const ImageSpec& userspec,
     int remaining = m_spec.nchannels;
     if (m_spec.channelformats.size()) {
         int streak;
-        // accomodate first 3 channels of the same type as colour ones
+        // accommodate first 3 channels of the same type as colour ones
         for (streak = 1; streak <= 3 && remaining > 0; ++streak, --remaining)
             if (m_spec.channelformats[streak] != m_spec.channelformats[0]
                 || m_spec.alpha_channel == streak || m_spec.z_channel == streak)
@@ -292,12 +292,12 @@ RLAOutput::open(const std::string& name, const ImageSpec& userspec,
     //           << m_rla.NumOfMatteChannels << " z " << m_rla.NumOfAuxChannels << "\n";
     m_rla.Revision = 0xFFFE;
 
-    std::string colorspace = m_spec.get_string_attribute("oiio:ColorSpace",
-                                                         "Unknown");
+    string_view colorspace = m_spec.get_string_attribute("oiio:ColorSpace");
     if (Strutil::iequals(colorspace, "Linear"))
         Strutil::safe_strcpy(m_rla.Gamma, "1.0", sizeof(m_rla.Gamma));
-    else if (Strutil::istarts_with(colorspace, "GammaCorrected")) {
-        float g = Strutil::from_string<float>(colorspace.c_str() + 14);
+    else if (Strutil::istarts_with(colorspace, "Gamma")) {
+        Strutil::parse_word(colorspace);
+        float g = Strutil::from_string<float>(colorspace);
         if (!(g >= 0.01f && g <= 10.0f /* sanity check */))
             g = m_spec.get_float_attribute("oiio:Gamma", 1.f);
         safe_snprintf(m_rla.Gamma, sizeof(m_rla.Gamma), "%.10f", g);
