@@ -35,32 +35,31 @@ static bool tgadebug = Strutil::stoi(Sysutil::getenv("OIIO_TARGA_DEBUG"));
 class TGAInput final : public ImageInput {
 public:
     TGAInput() { init(); }
-    virtual ~TGAInput() { close(); }
-    virtual const char* format_name(void) const override { return "targa"; }
-    virtual int supports(string_view feature) const override
+    ~TGAInput() override { close(); }
+    const char* format_name(void) const override { return "targa"; }
+    int supports(string_view feature) const override
     {
-        return (feature == "thumbnail");
+        return (feature == "thumbnail" || feature == "ioproxy");
     }
-    virtual bool open(const std::string& name, ImageSpec& newspec) override;
-    virtual bool open(const std::string& name, ImageSpec& newspec,
-                      const ImageSpec& config) override;
-    virtual bool close() override;
-    virtual bool read_native_scanline(int subimage, int miplevel, int y, int z,
-                                      void* data) override;
-    virtual bool get_thumbnail(ImageBuf& thumb, int subimage) override;
+    bool open(const std::string& name, ImageSpec& newspec) override;
+    bool open(const std::string& name, ImageSpec& newspec,
+              const ImageSpec& config) override;
+    bool close() override;
+    bool read_native_scanline(int subimage, int miplevel, int y, int z,
+                              void* data) override;
+    bool get_thumbnail(ImageBuf& thumb, int subimage) override;
 
 private:
-    std::string m_filename;            ///< Stash the filename
-    FILE* m_file;                      ///< Open image handle
-    tga_header m_tga;                  ///< Targa header
-    tga_footer m_foot;                 ///< Targa 2.0 footer
-    tga_alpha_type m_alpha_type;       ///< Alpha type
-    int64_t m_ofs_thumb       = 0;     ///< Offset of thumbnail info
-    int64_t m_ofs_palette     = 0;     ///< Offset of palette
-    int64_t m_ofs_colcorr_tbl = 0;     ///< Offset to colour correction table
-    bool m_keep_unassociated_alpha;    ///< Do not convert unassociated alpha
+    std::string m_filename;            // Stash the filename
+    tga_header m_tga;                  // Targa header
+    tga_footer m_foot;                 // Targa 2.0 footer
+    tga_alpha_type m_alpha_type;       // Alpha type
+    int64_t m_ofs_thumb       = 0;     // Offset of thumbnail info
+    int64_t m_ofs_palette     = 0;     // Offset of palette
+    int64_t m_ofs_colcorr_tbl = 0;     // Offset to colour correction table
+    bool m_keep_unassociated_alpha;    // Do not convert unassociated alpha
     short m_tga_version = 1;           // TGA version (1 or 2)
-    std::vector<unsigned char> m_buf;  ///< Buffer the image pixels
+    std::unique_ptr<uint8_t[]> m_buf;  // Buffer the image pixels
 
     // Is this a palette image, i.e. it has a color map?
     bool is_palette() const { return m_tga.cmap_type != 0; }
@@ -69,18 +68,20 @@ private:
     ///
     void init()
     {
-        m_file = NULL;
-        m_buf.clear();
+        m_buf.reset();
         m_ofs_thumb               = 0;
         m_ofs_palette             = 0;
         m_ofs_colcorr_tbl         = 0;
         m_alpha_type              = TGA_ALPHA_NONE;
         m_keep_unassociated_alpha = false;
         m_tga_version             = 1;
+        ioproxy_clear();
     }
 
-    /// Helper function: read the image.
-    ///
+    // Helper: read the tga 2.0 specific parts of the header
+    bool read_tga2_header();
+
+    // Helper function: read the image.
     bool readimg();
 
     /// Helper function: decode a pixel.
@@ -88,30 +89,16 @@ private:
                              unsigned char* palette, int bytespp,
                              int palbytespp);
 
-    // Helper: read items, with error detection. This does not do any
-    // byte swapping! Upon error, issue error messag and return false.
-    bool fread(void* buf, size_t itemsize, size_t nitems)
-    {
-        size_t n = ::fread(buf, itemsize, nitems, m_file);
-        if (n != nitems) {
-            if (feof(m_file))
-                errorfmt("Read error on \"{}\": hit end of file", m_filename);
-            else
-                errorfmt("Read error on \"{}\": could not read", m_filename);
-        }
-        return n == nitems;
-    }
+    // Read one byte
+    bool read(uint8_t& buf) { return ioread(&buf, sizeof(buf), 1); }
 
     // Read one byte
-    bool read(uint8_t& buf) { return fread(&buf, sizeof(buf), 1); }
-
-    // Read one byte
-    bool read(char& buf) { return fread(&buf, sizeof(buf), 1); }
+    bool read(char& buf) { return ioread(&buf, sizeof(buf), 1); }
 
     // Read a short, byte swap as necessary.
     bool read(uint16_t& buf)
     {
-        bool ok = fread(&buf, sizeof(buf), 1);
+        bool ok = ioread(&buf, sizeof(buf), 1);
         if (bigendian())  // TGAs are always little-endian
             swap_endian(&buf);
         return ok;
@@ -120,27 +107,16 @@ private:
     // Read an int, byte swap as necessary.
     bool read(uint32_t& buf)
     {
-        bool ok = fread(&buf, sizeof(buf), 1);
+        bool ok = ioread(&buf, sizeof(buf), 1);
         if (bigendian())  // TGAs are always little-endian
             swap_endian(&buf);
         return ok;
     }
-
-    bool fseek(int64_t addr)
-    {
-        // Strutil::print("Seeking to {}\n", addr);
-        int r = Filesystem::fseek(m_file, addr, SEEK_SET);
-        if (r) {
-            errorfmt("Could not seek file \"{}\" to offset {}", m_filename,
-                     addr);
-        }
-        return (r == 0);
-    }
 };
 
 
 
-// Obligatory material to make this a recognizeable imageio plugin:
+// Obligatory material to make this a recognizable imageio plugin:
 OIIO_PLUGIN_EXPORTS_BEGIN
 
 OIIO_EXPORT ImageInput*
@@ -169,12 +145,9 @@ TGAInput::open(const std::string& name, ImageSpec& newspec)
     m_filename = name;
 
     DBG("TGA opening {}\n", name);
-    uint64_t filesize = Filesystem::file_size(name);
-    m_file            = Filesystem::fopen(name, "rb");
-    if (!m_file) {
-        errorf("Could not open file \"%s\"", name);
+    if (!ioproxy_use_or_open(name))
         return false;
-    }
+    ioseek(0);
 
     // Due to struct packing, we may get a corrupt header if we just load the
     // struct from file; to address that, read every member individually save
@@ -271,214 +244,39 @@ TGAInput::open(const std::string& name, ImageSpec& newspec)
         // in case the comment lacks null termination
         char id[256];
         memset(id, 0, sizeof(id));
-        if (!fread(id, m_tga.idlen, 1))
+        if (!ioread(id, m_tga.idlen, 1))
             return false;
         m_spec.attribute("targa:ImageID", id);
     }
 
-    int64_t ofs   = Filesystem::ftell(m_file);
+    int64_t ofs   = iotell();
     m_ofs_palette = ofs;
 
     // now try and see if it's a TGA 2.0 image
     // TGA 2.0 files are identified by a nifty "TRUEVISION-XFILE.\0" signature
-    bool check_for_tga2 = (filesize > 26 + 18);
-    if (check_for_tga2 && Filesystem::fseek(m_file, -26, SEEK_END) != 0) {
+    bool check_for_tga2 = (ioproxy()->size() > 26 + 18);
+    if (check_for_tga2 && !ioseek(-26, SEEK_END)) {
         errorfmt("Could not seek to find the TGA 2.0 signature.");
         return false;
     }
     if (check_for_tga2 && read(m_foot.ofs_ext) && read(m_foot.ofs_dev)
-        && fread(&m_foot.signature, sizeof(m_foot.signature), 1)
+        && ioread(&m_foot.signature, sizeof(m_foot.signature), 1)
         && !strncmp(m_foot.signature, "TRUEVISION-XFILE.", 17)) {
         //std::cerr << "[tga] this is a TGA 2.0 file\n";
-        m_spec.attribute("targa:version", 2);
         m_tga_version = 2;
-
-        // read the extension area
-        if (!fseek(m_foot.ofs_ext)) {
+        if (!read_tga2_header())
             return false;
-        }
-        // check if this is a TGA 2.0 extension area
-        // according to the 2.0 spec, the size for valid 2.0 files is exactly
-        // 495 bytes, and the reader should only read as much as it understands
-        // for < 495, we ignore this section of the file altogether
-        // for > 495, we only read what we know
-        uint16_t s;
-        if (!read(s))
-            return false;
-        //std::cerr << "[tga] extension area size: " << s << "\n";
-        if (s >= 495) {
-            union {
-                unsigned char c[324];  // so as to accommodate the comments
-                uint16_t s[6];
-                uint32_t l;
-            } buf;
-
-            // load image author
-            if (!fread(buf.c, 41, 1))
-                return false;
-            if (buf.c[0])
-                m_spec.attribute("Artist", (char*)buf.c);
-
-            // load image comments
-            if (!fread(buf.c, 324, 1))
-                return false;
-
-            // concatenate the lines into a single string
-            std::string tmpstr((const char*)buf.c);
-            if (buf.c[81]) {
-                tmpstr += "\n";
-                tmpstr += (const char*)&buf.c[81];
-            }
-            if (buf.c[162]) {
-                tmpstr += "\n";
-                tmpstr += (const char*)&buf.c[162];
-            }
-            if (buf.c[243]) {
-                tmpstr += "\n";
-                tmpstr += (const char*)&buf.c[243];
-            }
-            if (tmpstr.length() > 0)
-                m_spec.attribute("ImageDescription", tmpstr);
-
-            // timestamp
-            if (!fread(buf.s, 2, 6))
-                return false;
-            if (buf.s[0] || buf.s[1] || buf.s[2] || buf.s[3] || buf.s[4]
-                || buf.s[5]) {
-                if (bigendian())
-                    swap_endian(&buf.s[0], 6);
-                sprintf((char*)&buf.c[12], "%04u:%02u:%02u %02u:%02u:%02u",
-                        buf.s[2], buf.s[0], buf.s[1], buf.s[3], buf.s[4],
-                        buf.s[5]);
-                m_spec.attribute("DateTime", (char*)&buf.c[12]);
-            }
-
-            // job name/ID
-            if (!fread(buf.c, 41, 1))
-                return false;
-            if (buf.c[0])
-                m_spec.attribute("DocumentName", (char*)buf.c);
-
-            // job time
-            if (!fread(buf.s, 2, 3))
-                return false;
-            if (buf.s[0] || buf.s[1] || buf.s[2]) {
-                if (bigendian())
-                    swap_endian(&buf.s[0], 3);
-                sprintf((char*)&buf.c[6], "%u:%02u:%02u", buf.s[0], buf.s[1],
-                        buf.s[2]);
-                m_spec.attribute("targa:JobTime", (char*)&buf.c[6]);
-            }
-
-            // software
-            if (!fread(buf.c, 41, 1))
-                return false;
-            uint16_t n;
-            char l;
-            if (!read(n) || !read(l))
-                return false;
-            if (buf.c[0]) {
-                // tack on the version number and letter
-                sprintf((char*)&buf.c[strlen((char*)buf.c)], " %u.%u%c",
-                        n / 100, n % 100, l != ' ' ? l : 0);
-                m_spec.attribute("Software", (char*)buf.c);
-            }
-
-            // background (key) colour
-            if (!fread(buf.c, 4, 1))
-                return false;
-            // FIXME: what do we do with it?
-
-            // aspect ratio
-            if (!fread(buf.s, 2, 2))
-                return false;
-            // if the denominator is zero, it's unused
-            if (buf.s[1]) {
-                if (bigendian())
-                    swap_endian(&buf.s[0], 2);
-                m_spec.attribute("PixelAspectRatio",
-                                 (float)buf.s[0] / (float)buf.s[1]);
-            }
-
-            // gamma
-            if (!fread(buf.s, 2, 2))
-                return false;
-            // if the denominator is zero, it's unused
-            if (buf.s[1]) {
-                if (bigendian())
-                    swap_endian(&buf.s[0], 2);
-                float gamma = (float)buf.s[0] / (float)buf.s[1];
-                // Round gamma to the nearest hundredth to prevent stupid
-                // precision choices and make it easier for apps to make
-                // decisions based on known gamma values. For example, you want
-                // 2.2, not 2.19998.
-                gamma = roundf(100.0 * gamma) / 100.0f;
-                if (gamma == 1.f) {
-                    m_spec.attribute("oiio:ColorSpace", "linear");
-                } else {
-                    m_spec.attribute("oiio:ColorSpace",
-                                     Strutil::fmt::format("Gamma{:.2g}", gamma));
-                    m_spec.attribute("oiio:Gamma", gamma);
-                }
-            }
-
-            // offset to colour correction table
-            if (!read(buf.l))
-                return false;
-            m_ofs_colcorr_tbl = buf.l;
-            /*std::cerr << "[tga] colour correction table offset: "
-                      << (int)m_ofs_colcorr_tbl << "\n";*/
-
-            // offset to thumbnail
-            if (!read(buf.l))
-                return false;
-            m_ofs_thumb = buf.l;
-
-            // offset to scan-line table
-            if (!read(buf.l))
-                return false;
-            // TODO: can we find any use for this? we can't advertise random
-            // access anyway, because not all RLE-compressed files will have
-            // this table
-
-            // alpha type
-            if (!fread(buf.c, 1, 1))
-                return false;
-            m_alpha_type = (tga_alpha_type)buf.c[0];
-            if (m_alpha_type)
-                m_spec.attribute("targa:alpha_type", m_alpha_type);
-
-            // Check for presence of a thumbnail and set the metadata that
-            // says its dimensions, but don't read and decode it unless
-            // thumbnail() is called.
-            if (m_ofs_thumb > 0) {
-                if (!fseek(m_ofs_thumb)) {
-                    return false;
-                }
-                // Read the thumbnail dimensions -- sometimes it's 0x0 to
-                // indicate no thumbnail.
-                unsigned char res[2];
-                if (fread(&res, 2, 1) && res[0] > 0 && res[1] > 0) {
-                    m_spec.attribute("thumbnail_width", (int)res[0]);
-                    m_spec.attribute("thumbnail_height", (int)res[1]);
-                    m_spec.attribute("thumbnail_nchannels", m_spec.nchannels);
-                }
-            }
-        }
-        // FIXME: provide access to the developer area; according to Larry,
-        // it's probably safe to ignore it altogether until someone complains
-        // that it's missing :)
     } else {
         m_tga_version = 1;
-        m_spec.attribute("targa:version", 1);
     }
+    m_spec.attribute("targa:version", int(m_tga_version));
 
     if (m_spec.alpha_channel != -1 && m_alpha_type == TGA_ALPHA_USEFUL
         && m_keep_unassociated_alpha)
         m_spec.attribute("oiio:UnassociatedAlpha", 1);
 
     // Reposition back to where the palette starts
-    if (!fseek(ofs)) {
+    if (!ioseek(ofs)) {
         return false;
     }
 
@@ -496,11 +294,201 @@ TGAInput::open(const std::string& name, ImageSpec& newspec,
     // Check 'config' for any special requests
     if (config.get_int_attribute("oiio:UnassociatedAlpha", 0) == 1)
         m_keep_unassociated_alpha = true;
+    ioproxy_retrieve_from_config(config);
     return open(name, newspec);
 }
 
 
 
+bool
+TGAInput::read_tga2_header()
+{
+    // read the extension area
+    if (!ioseek(m_foot.ofs_ext)) {
+        return false;
+    }
+    // check if this is a TGA 2.0 extension area
+    // according to the 2.0 spec, the size for valid 2.0 files is exactly
+    // 495 bytes, and the reader should only read as much as it understands
+    // for < 495, we ignore this section of the file altogether
+    // for > 495, we only read what we know
+    uint16_t s;
+    if (!read(s))
+        return false;
+    //std::cerr << "[tga] extension area size: " << s << "\n";
+    if (s >= 495) {
+        union {
+            unsigned char c[324];  // so as to accommodate the comments
+            uint16_t s[6];
+            uint32_t l;
+        } buf;
+
+        // load image author
+        if (!ioread(buf.c, 41, 1))
+            return false;
+        if (buf.c[0])
+            m_spec.attribute("Artist", (char*)buf.c);
+
+        // load image comments
+        if (!ioread(buf.c, 324, 1))
+            return false;
+
+        // concatenate the lines into a single string
+        std::string tmpstr = Strutil::safe_string((const char*)buf.c, 81);
+        if (buf.c[81]) {
+            tmpstr += "\n";
+            tmpstr += Strutil::safe_string((const char*)&buf.c[81], 81);
+        }
+        if (buf.c[162]) {
+            tmpstr += "\n";
+            tmpstr += Strutil::safe_string((const char*)&buf.c[162], 81);
+        }
+        if (buf.c[243]) {
+            tmpstr += "\n";
+            tmpstr += Strutil::safe_string((const char*)&buf.c[243], 81);
+        }
+        if (tmpstr.length() > 0)
+            m_spec.attribute("ImageDescription", tmpstr);
+
+        // timestamp
+        if (!ioread(buf.s, 2, 6))
+            return false;
+        if (buf.s[0] || buf.s[1] || buf.s[2] || buf.s[3] || buf.s[4]
+            || buf.s[5]) {
+            if (bigendian())
+                swap_endian(&buf.s[0], 6);
+            m_spec.attribute(
+                "DateTime",
+                Strutil::fmt::format("{:04}:{:02}:{:02} {:02}:{:02}:{:02}",
+                                     buf.s[2], buf.s[0], buf.s[1], buf.s[3],
+                                     buf.s[4], buf.s[5]));
+        }
+
+        // job name/ID
+        if (!ioread(buf.c, 41, 1))
+            return false;
+        if (buf.c[0])
+            m_spec.attribute("DocumentName", (char*)buf.c);
+
+        // job time
+        if (!ioread(buf.s, 2, 3))
+            return false;
+        if (buf.s[0] || buf.s[1] || buf.s[2]) {
+            if (bigendian())
+                swap_endian(&buf.s[0], 3);
+            m_spec.attribute("targa:JobTime",
+                             Strutil::fmt::format("{}:{:02}:{:02}", buf.s[0],
+                                                  buf.s[1], buf.s[2]));
+        }
+
+        // software
+        if (!ioread(buf.c, 41, 1))
+            return false;
+        uint16_t n;
+        char l;
+        if (!read(n) || !read(l))
+            return false;
+        if (buf.c[0]) {
+            // tack on the version number and letter
+            std::string soft((const char*)buf.c);
+            soft += Strutil::fmt::format(" {}.{}", n / 100, n % 100);
+            if (l != ' ')
+                soft += l;
+            m_spec.attribute("Software", soft);
+        }
+
+        // background (key) colour
+        if (!ioread(buf.c, 4, 1))
+            return false;
+        // FIXME: what do we do with it?
+
+        // aspect ratio
+        if (!ioread(buf.s, 2, 2))
+            return false;
+        // if the denominator is zero, it's unused
+        if (buf.s[1]) {
+            if (bigendian())
+                swap_endian(&buf.s[0], 2);
+            m_spec.attribute("PixelAspectRatio",
+                             (float)buf.s[0] / (float)buf.s[1]);
+        }
+
+        // gamma
+        if (!ioread(buf.s, 2, 2))
+            return false;
+        // if the denominator is zero, it's unused
+        if (buf.s[1]) {
+            if (bigendian())
+                swap_endian(&buf.s[0], 2);
+            float gamma = (float)buf.s[0] / (float)buf.s[1];
+            // Round gamma to the nearest hundredth to prevent stupid
+            // precision choices and make it easier for apps to make
+            // decisions based on known gamma values. For example, you want
+            // 2.2, not 2.19998.
+            gamma = roundf(100.0 * gamma) / 100.0f;
+            if (gamma == 1.f) {
+                m_spec.attribute("oiio:ColorSpace", "linear");
+            } else {
+                m_spec.attribute("oiio:ColorSpace",
+                                 Strutil::fmt::format("Gamma{:.2g}", gamma));
+                m_spec.attribute("oiio:Gamma", gamma);
+            }
+        }
+
+        // offset to colour correction table
+        if (!read(buf.l))
+            return false;
+        m_ofs_colcorr_tbl = buf.l;
+        /*std::cerr << "[tga] colour correction table offset: "
+                      << (int)m_ofs_colcorr_tbl << "\n";*/
+
+        // offset to thumbnail
+        if (!read(buf.l))
+            return false;
+        m_ofs_thumb = buf.l;
+
+        // offset to scan-line table
+        if (!read(buf.l))
+            return false;
+        // TODO: can we find any use for this? we can't advertise random
+        // access anyway, because not all RLE-compressed files will have
+        // this table
+
+        // alpha type
+        if (!ioread(buf.c, 1, 1))
+            return false;
+        m_alpha_type = (tga_alpha_type)buf.c[0];
+        if (m_alpha_type)
+            m_spec.attribute("targa:alpha_type", m_alpha_type);
+
+        // Check for presence of a thumbnail and set the metadata that
+        // says its dimensions, but don't read and decode it unless
+        // thumbnail() is called.
+        if (m_ofs_thumb > 0) {
+            if (!ioseek(m_ofs_thumb)) {
+                return false;
+            }
+            // Read the thumbnail dimensions -- sometimes it's 0x0 to
+            // indicate no thumbnail.
+            unsigned char res[2];
+            if (!ioread(&res, 2, 1))
+                return false;
+            if (res[0] > 0 && res[1] > 0) {
+                m_spec.attribute("thumbnail_width", (int)res[0]);
+                m_spec.attribute("thumbnail_height", (int)res[1]);
+                m_spec.attribute("thumbnail_nchannels", m_spec.nchannels);
+            }
+        }
+    }
+    // FIXME: provide access to the developer area; according to Larry,
+    // it's probably safe to ignore it altogether until someone complains
+    // that it's missing :)
+
+    return true;
+}
+
+
+
 bool
 TGAInput::get_thumbnail(ImageBuf& thumb, int subimage)
 {
@@ -509,16 +497,17 @@ TGAInput::get_thumbnail(ImageBuf& thumb, int subimage)
 
     lock_guard lock(*this);
     bool result         = false;
-    int64_t save_offset = Filesystem::ftell(m_file);
+    int64_t save_offset = iotell();
 
-    if (!fseek(m_ofs_thumb)) {
+    if (!ioseek(m_ofs_thumb))
         return false;
-    }
 
     // Read the thumbnail dimensions -- sometimes it's 0x0 to indicate no
     // thumbnail.
     unsigned char res[2];
-    if (fread(&res, 2, 1) && res[0] > 0 && res[1] > 0) {
+    if (!ioread(&res, 2, 1))
+        return false;
+    if (res[0] > 0 && res[1] > 0) {
         // Most of this code is a dupe of readimg(); according to the spec,
         // the thumbnail is in the same format as the main image but
         // uncompressed.
@@ -533,13 +522,13 @@ TGAInput::get_thumbnail(ImageBuf& thumb, int subimage)
         // read palette, if there is any
         std::unique_ptr<unsigned char[]> palette;
         if (is_palette()) {
-            if (!fseek(m_ofs_palette)) {
+            if (!ioseek(m_ofs_palette)) {
                 return false;
             }
             palette.reset(new unsigned char[palbytespp * m_tga.cmap_length]);
-            if (!fread(palette.get(), palbytespp, m_tga.cmap_length))
+            if (!ioread(palette.get(), palbytespp, m_tga.cmap_length))
                 return false;
-            if (!fseek(m_ofs_thumb + 2)) {
+            if (!ioseek(m_ofs_thumb + 2)) {
                 return false;
             }
         }
@@ -550,7 +539,7 @@ TGAInput::get_thumbnail(ImageBuf& thumb, int subimage)
             char* img = (char*)thumb.pixeladdr(0, y);
             for (int64_t x = 0; x < thumbspec.width;
                  x++, img += m_spec.nchannels) {
-                if (!fread(in, bytespp, 1))
+                if (!ioread(in, bytespp, 1))
                     return false;
                 decode_pixel(in, pixel, palette.get(), bytespp, palbytespp);
                 memcpy(img, pixel, m_spec.nchannels);
@@ -559,7 +548,7 @@ TGAInput::get_thumbnail(ImageBuf& thumb, int subimage)
         result = true;
     }
 
-    if (!fseek(save_offset)) {
+    if (!ioseek(save_offset)) {
         return false;
     }
     return result;
@@ -711,7 +700,7 @@ TGAInput::readimg()
         DBG("TGA {} allocating for {}x{} {}-chan image = {}\n", m_filename,
             m_spec.width, m_spec.height, m_spec.nchannels,
             m_spec.image_bytes());
-        m_buf.resize(m_spec.image_bytes());
+        m_buf.reset(new uint8_t[m_spec.image_bytes()]);
     } catch (const std::exception& e) {
         errorfmt("Cannot allocate enough memory for {}x{} {}-chan image {}",
                  m_spec.width, m_spec.height, m_spec.nchannels, m_filename);
@@ -719,10 +708,10 @@ TGAInput::readimg()
     }
 
     // read palette, if there is any
-    std::unique_ptr<unsigned char> palette;
+    std::unique_ptr<unsigned char[]> palette;
     if (is_palette()) {
         palette.reset(new unsigned char[palbytespp * m_tga.cmap_length]);
-        if (!fread(palette.get(), palbytespp, m_tga.cmap_length))
+        if (!ioread(palette.get(), palbytespp, m_tga.cmap_length))
             return false;
     }
 
@@ -733,11 +722,11 @@ TGAInput::readimg()
         unsigned char in[4];
         for (int64_t y = m_spec.height - 1; y >= 0; y--) {
             for (int64_t x = 0; x < m_spec.width; x++) {
-                if (!fread(in, bytespp, 1))
+                if (!ioread(in, bytespp, 1))
                     return false;
                 decode_pixel(in, pixel, palette.get(), bytespp, palbytespp);
-                memcpy(&m_buf[y * m_spec.width * m_spec.nchannels
-                              + x * m_spec.nchannels],
+                memcpy(m_buf.get() + y * m_spec.width * m_spec.nchannels
+                           + x * m_spec.nchannels,
                        pixel, m_spec.nchannels);
             }
         }
@@ -748,7 +737,7 @@ TGAInput::readimg()
         int packet_size;
         for (int64_t y = m_spec.height - 1; y >= 0; y--) {
             for (int64_t x = 0; x < m_spec.width; x++) {
-                if (!fread(in, 1 + bytespp, 1)) {
+                if (!ioread(in, 1 + bytespp, 1)) {
                     DBG("Failed on scanline {}\n", y);
                     return false;
                 }
@@ -758,8 +747,8 @@ TGAInput::readimg()
                     // DBG("[tga] run length packet size {} @ ({},{})\n",
                     //     packet_size, x, y);
                     for (int i = 0; i < packet_size; i++) {
-                        memcpy(&m_buf[y * m_spec.width * m_spec.nchannels
-                                      + x * m_spec.nchannels],
+                        memcpy(m_buf.get() + y * m_spec.width * m_spec.nchannels
+                                   + x * m_spec.nchannels,
                                pixel, m_spec.nchannels);
                         if (i < packet_size - 1) {
                             x++;
@@ -778,8 +767,8 @@ TGAInput::readimg()
                     //     packet_size, x, y, pixel[0], pixel[1], pixel[2],
                     //     pixel[3]);
                     for (int i = 0; i < packet_size; i++) {
-                        memcpy(&m_buf[y * m_spec.width * m_spec.nchannels
-                                      + x * m_spec.nchannels],
+                        memcpy(m_buf.get() + y * m_spec.width * m_spec.nchannels
+                                   + x * m_spec.nchannels,
                                pixel, m_spec.nchannels);
                         if (i < packet_size - 1) {
                             x++;
@@ -792,7 +781,7 @@ TGAInput::readimg()
                                     goto loop_break;
                             }
                             // skip the packet header byte
-                            if (!fread(&in[1], bytespp, 1)) {
+                            if (!ioread(&in[1], bytespp, 1)) {
                                 DBG("Failed on scanline {}\n", y);
                                 return false;
                             }
@@ -818,8 +807,8 @@ TGAInput::readimg()
         std::vector<unsigned char> flip (m_spec.width * bytespp);
         unsigned char *src, *dst, *tmp = &flip[0];
         for (int y = 0; y < m_spec.height / 2; y++) {
-            src = &m_buf[(m_spec.height - y - 1) * m_spec.width * bytespp];
-            dst = &m_buf[y * m_spec.width * bytespp];
+            src = m_buf.get() + (m_spec.height - y - 1) * m_spec.width * bytespp;
+            dst = m_buf.get() + y * m_spec.width * bytespp;
 
             memcpy(tmp, src, m_spec.width * bytespp);
             memcpy(src, dst, m_spec.width * bytespp);
@@ -832,7 +821,7 @@ TGAInput::readimg()
         int nc             = m_spec.nchannels;
         int scanline_bytes = m_spec.width * m_spec.nchannels;
         for (int64_t y = 0; y < m_spec.height; y++) {
-            unsigned char* line = &m_buf[y * scanline_bytes];
+            unsigned char* line = m_buf.get() + y * scanline_bytes;
             for (int64_t x = 0; x < m_spec.width / 2; x++) {
                 unsigned char* src = line + x * m_spec.nchannels;
                 unsigned char* dst
@@ -860,7 +849,7 @@ TGAInput::readimg()
         }
         if (!alpha0_everywhere) {
             float gamma = m_spec.get_float_attribute("oiio:Gamma", 1.0f);
-            associateAlpha((unsigned char*)m_buf.data(), size, m_spec.nchannels,
+            associateAlpha((unsigned char*)m_buf.get(), size, m_spec.nchannels,
                            m_spec.alpha_channel, gamma);
         }
     }
@@ -874,11 +863,6 @@ TGAInput::readimg()
 bool
 TGAInput::close()
 {
-    if (m_file) {
-        fclose(m_file);
-        m_file = NULL;
-    }
-
     init();  // Reset to initial state
     return true;
 }
@@ -893,7 +877,7 @@ TGAInput::read_native_scanline(int subimage, int miplevel, int y, int /*z*/,
     if (!seek_subimage(subimage, miplevel))
         return false;
 
-    if (m_buf.empty()) {
+    if (!m_buf) {
         if (!readimg()) {
             return false;
         }
@@ -902,7 +886,7 @@ TGAInput::read_native_scanline(int subimage, int miplevel, int y, int /*z*/,
     if (m_tga.attr & FLAG_Y_FLIP)
         y = m_spec.height - y - 1;
     size_t size = spec().scanline_bytes();
-    memcpy(data, m_buf.data() + y * size, size);
+    memcpy(data, m_buf.get() + y * size, size);
     return true;
 }
 
