@@ -11,6 +11,7 @@
 #include <iostream>
 #include <iterator>
 
+#include <OpenImageIO/Imath.h>
 #include <OpenImageIO/argparse.h>
 #include <OpenImageIO/benchmark.h>
 #include <OpenImageIO/filesystem.h>
@@ -35,6 +36,7 @@ static std::vector<ustring> filenames;
 static std::string output_filename = "out.exr";
 static bool verbose                = false;
 static int nthreads                = 0;
+static int minthreads              = 1;
 static int threadtimes             = 0;
 static int output_xres = 512, output_yres = 512;
 static int nchannels_override     = 0;
@@ -56,17 +58,19 @@ static bool test_getimagespec = false;
 static bool filtertest        = false;
 static TextureSystem* texsys  = NULL;
 static std::string searchpath;
-static bool batch        = false;
-static bool nowarp       = false;
-static bool tube         = false;
-static bool use_handle   = false;
-static float cachesize   = -1;
-static int maxfiles      = -1;
-static int mipmode       = TextureOpt::MipModeDefault;
-static int interpmode    = TextureOpt::InterpSmartBicubic;
-static float missing[4]  = { -1, 0, 0, 1 };
-static float fill        = -1;  // -1 signifies unset
-static float scalefactor = 1.0f;
+static bool batch         = false;
+static bool nowarp        = false;
+static bool tube          = false;
+static bool use_handle    = false;
+static bool use_bluenoise = false;
+static float cachesize    = -1;
+static int maxfiles       = -1;
+static int mipmode        = TextureOpt::MipModeDefault;
+static int interpmode     = TextureOpt::InterpSmartBicubic;
+static int stochastic     = 0;
+static float missing[4]   = { -1, 0, 0, 1 };
+static float fill         = -1;  // -1 signifies unset
+static float scalefactor  = 1.0f;
 static Imath::V3f texoffset(0, 0, 0);
 static bool nountiled              = false;
 static bool nounmipped             = false;
@@ -76,6 +80,7 @@ static bool resetstats             = false;
 static bool testhash               = false;
 static bool wedge                  = false;
 static int ntrials                 = 1;
+static int lowtrials               = 10;
 static int testicwrite             = 0;
 static bool test_derivs            = false;
 static bool test_statquery         = false;
@@ -92,6 +97,7 @@ static std::vector<std::string> filenames_to_delete;
 const int pieces_per_udim = 20;
 static std::vector<TextureSystem::TextureHandle*> texture_handles;
 void* dummyptr;
+static const ImageBuf& bluenoiseimg(ImageBufAlgo::bluenoise_image());
 
 typedef void (*Mapping2D)(const int&, const int&, float&, float&, float&,
                           float&, float&, float&);
@@ -154,11 +160,13 @@ getargs(int argc, const char* argv[])
     ap.arg("--anisoaspect %f:ASPECT", &anisoaspect)
       .help("Set anisotropic ellipse aspect ratio for threadtimes tests (default: 2.0)");
     ap.arg("--anisomax %d:MAX", &anisomax)
-      .help(Strutil::sprintf("Set max anisotropy (default: %d)", anisomax));
+      .help(Strutil::fmt::format("Set max anisotropy (default: {})", anisomax));
     ap.arg("--mipmode %d:MODE", &mipmode)
       .help("Set mip mode (default: 0 = aniso)");
     ap.arg("--interpmode %d:MODE", &interpmode)
       .help("Set interp mode (default: 3 = smart bicubic)");
+    ap.arg("--stochastic %d:MODE", &stochastic)
+      .help("Set stochastic sampling mode (default: 0 = none)");
     ap.arg("--missing %f:R %f:G %f:B", &missing[0], &missing[1], &missing[2])
       .help("Specify missing texture color");
     ap.arg("--autotile %d:TILESIZE", &autotile)
@@ -166,7 +174,7 @@ getargs(int argc, const char* argv[])
     ap.arg("--automip", &automip)
       .help("Set auto-MIPmap for the image cache");
     ap.arg("--batch", &batch)
-      .help(Strutil::sprintf("Use batched shading, batch size = %d", Tex::BatchWidth));
+      .help(Strutil::fmt::format("Use batched shading, batch size = {}", Tex::BatchWidth));
     ap.arg("--handle", &use_handle)
       .help("Use texture handle rather than name lookup");
     ap.arg("--searchpath %s:PATHLIST", &searchpath)
@@ -185,7 +193,7 @@ getargs(int argc, const char* argv[])
       .help("Test TextureSystem::get_imagespec");
     ap.arg("--gettextureinfo %s:NAME", &gtiname)
       .help("Test gettextureinfo, retrieving attrib 'NAME'");
-    ap.arg("--offset %f:SOFF %f:TOFF %f:ROFF", &texoffset[0], &texoffset[1], &texoffset[2])
+    ap.arg("--offset %f:SOFF %f:TOFF %f:ROFF", &texoffset.x, &texoffset.y, &texoffset.z)
       .help("Offset texture coordinates");
     ap.arg("--scalest %f:SSCALE %f:TSCALE", &sscale, &tscale)
       .help("Scale texture lookups (s, t)");
@@ -215,8 +223,12 @@ getargs(int argc, const char* argv[])
       .help("Do thread timings (arg = workload profile)");
     ap.arg("--trials %d:N", &ntrials)
       .help("Number of trials for timings");
+    ap.arg("--lowtrials %d:N", &lowtrials)
+      .help("Optional lower number of trials for <= 2 thread timings");
     ap.arg("--wedge", &wedge)
       .help("Wedge test");
+    ap.arg("--minthreads %d:N", &minthreads)
+      .help("Minimum number of threads for wedges (default: 1)");
     ap.arg("--noinvalidate %!", &invalidate_before_iter)
       .help("Don't invalidate the cache before each --threadtimes trial");
     ap.arg("--closebeforeiter", &close_before_iter)
@@ -231,6 +243,8 @@ getargs(int argc, const char* argv[])
       .help("Make tests from a template (e.g., \"tmp/test{:04}.exr\")");
     ap.arg("--udim", &udim_tests)
       .help("Do udim-oriented tests");
+    ap.arg("--bluenoise", &use_bluenoise)
+      .help("Use blue noise for stochastic choices");
 
     // clang-format on
     ap.parse(argc, argv);
@@ -299,61 +313,58 @@ test_gettextureinfo(ustring filename)
     int res[2] = { 0 };
     ok         = texsys->get_texture_info(filename, 0, ustring("resolution"),
                                   TypeDesc(TypeDesc::INT, 2), res);
-    std::cout << "Result of get_texture_info resolution = " << ok << ' '
-              << res[0] << 'x' << res[1] << "\n";
+    Strutil::print("Result of get_texture_info resolution = {} {}x{}\n", ok,
+                   res[0], res[1]);
 
     int chan = 0;
     ok       = texsys->get_texture_info(filename, 0, ustring("channels"),
                                   TypeDesc::INT, &chan);
-    std::cout << "Result of get_texture_info channels = " << ok << ' ' << chan
-              << "\n";
+    Strutil::print("Result of get_texture_info channels = {} {}\n", ok, chan);
 
     float fchan = 0;
     ok          = texsys->get_texture_info(filename, 0, ustring("channels"),
                                   TypeDesc::FLOAT, &fchan);
-    std::cout << "Result of get_texture_info channels = " << ok << ' ' << fchan
-              << "\n";
+    Strutil::print("Result of get_texture_info channels = {} {:g}\n", ok,
+                   fchan);
 
     int dataformat = 0;
     ok = texsys->get_texture_info(filename, 0, ustring("format"), TypeDesc::INT,
                                   &dataformat);
-    std::cout << "Result of get_texture_info data format = " << ok << ' '
-              << TypeDesc((TypeDesc::BASETYPE)dataformat).c_str() << "\n";
+    Strutil::print("Result of get_texture_info data format = {} {}\n", ok,
+                   TypeDesc((TypeDesc::BASETYPE)dataformat).c_str());
 
     const char* datetime = NULL;
     ok = texsys->get_texture_info(filename, 0, ustring("DateTime"),
                                   TypeDesc::STRING, &datetime);
-    std::cout << "Result of get_texture_info datetime = " << ok << ' '
-              << (datetime ? datetime : "") << "\n";
+    Strutil::print("Result of get_texture_info datetime = {} {}\n", ok,
+                   (datetime ? datetime : ""));
 
     float avg[4];
     ok = texsys->get_texture_info(filename, 0, ustring("averagecolor"),
                                   TypeDesc(TypeDesc::FLOAT, 4), avg);
-    std::cout << "Result of get_texture_info averagecolor = "
-              << (ok ? "yes" : "no\n");
+    Strutil::print("Result of get_texture_info averagecolor = {}",
+                   ok ? "yes" : "no\n");
     if (ok)
-        std::cout << " " << avg[0] << ' ' << avg[1] << ' ' << avg[2] << ' '
-                  << avg[3] << "\n";
+        Strutil::print(" {} {} {} {}\n", avg[0], avg[1], avg[2], avg[3]);
     ok = texsys->get_texture_info(filename, 0, ustring("averagealpha"),
                                   TypeFloat, avg);
-    std::cout << "Result of get_texture_info averagealpha = "
-              << (ok ? "yes" : "no\n");
+    Strutil::print("Result of get_texture_info averagealpha = {}",
+                   ok ? "yes" : "no\n");
     if (ok)
-        std::cout << " " << avg[0] << "\n";
+        Strutil::print(" {}\n", avg[0]);
     ok = texsys->get_texture_info(filename, 0, ustring("constantcolor"),
                                   TypeDesc(TypeDesc::FLOAT, 4), avg);
-    std::cout << "Result of get_texture_info constantcolor = "
-              << (ok ? "yes" : "no\n");
+    Strutil::print("Result of get_texture_info constantcolor = {}",
+                   ok ? "yes" : "no\n");
     if (ok)
-        std::cout << " " << avg[0] << ' ' << avg[1] << ' ' << avg[2] << ' '
-                  << avg[3] << "\n";
+        Strutil::print(" {} {} {} {}\n", avg[0], avg[1], avg[2], avg[3]);
 
     const char* texturetype = NULL;
     ok = texsys->get_texture_info(filename, 0, ustring("textureformat"),
                                   TypeDesc::STRING, &texturetype);
-    std::cout << "Texture type is " << ok << ' '
-              << (texturetype ? texturetype : "") << "\n";
-    std::cout << "\n";
+    Strutil::print("Texture type is {} {}\n", ok,
+                   texturetype ? texturetype : "");
+    Strutil::print("\n");
 }
 
 
@@ -397,8 +408,8 @@ void
 map_default(const Int& x, const Int& y, Float& s, Float& t, Float& dsdx,
             Float& dtdx, Float& dsdy, Float& dtdy)
 {
-    s    = (Float(x) + 0.5f) / output_xres * sscale + texoffset[0];
-    t    = (Float(y) + 0.5f) / output_yres * tscale + texoffset[1];
+    s    = (Float(x) + 0.5f) / output_xres * sscale + texoffset.x;
+    t    = (Float(y) + 0.5f) / output_yres * tscale + texoffset.y;
     dsdx = 1.0f / output_xres * sscale;
     dtdx = 0.0f;
     dsdy = 0.0f;
@@ -412,19 +423,18 @@ static void
 map_warp(const Int& x, const Int& y, Float& s, Float& t, Float& dsdx,
          Float& dtdx, Float& dsdy, Float& dtdy)
 {
-    const Imath::Vec2<Float> coord  = warp_coord(Float(x) + 0.5f,
-                                                Float(y) + 0.5f);
-    const Imath::Vec2<Float> coordx = warp_coord(Float(x) + 1.5f,
-                                                 Float(y) + 0.5f);
-    const Imath::Vec2<Float> coordy = warp_coord(Float(x) + 0.5f,
-                                                 Float(y) + 1.5f);
+    Imath::Vec2<Float> coord  = warp_coord(Float(x) + 0.5f, Float(y) + 0.5f);
+    Imath::Vec2<Float> coordx = warp_coord(Float(x) + 1.5f, Float(y) + 0.5f)
+                                - coord;
+    Imath::Vec2<Float> coordy = warp_coord(Float(x) + 0.5f, Float(y) + 1.5f)
+                                - coord;
 
-    s    = coord[0];
-    t    = coord[1];
-    dsdx = coordx[0] - coord[0];
-    dtdx = coordx[1] - coord[1];
-    dsdy = coordy[0] - coord[0];
-    dtdy = coordy[1] - coord[1];
+    s    = coord.x;
+    t    = coord.y;
+    dsdx = coordx.x;
+    dtdx = coordx.y;
+    dsdy = coordy.x;
+    dtdy = coordy.y;
 }
 
 
@@ -525,16 +535,16 @@ map_default_3D(const Int& x, const Int& y, Imath::Vec3<Float>& P,
                Imath::Vec3<Float>& dPdx, Imath::Vec3<Float>& dPdy,
                Imath::Vec3<Float>& dPdz)
 {
-    P[0] = (Float(x) + 0.5f) / output_xres * sscale;
-    P[1] = (Float(y) + 0.5f) / output_yres * tscale;
-    P[2] = 0.5f * sscale;
+    P.x = (Float(x) + 0.5f) / output_xres * sscale;
+    P.y = (Float(y) + 0.5f) / output_yres * tscale;
+    P.z = 0.5f * sscale;
     P += texoffset;
-    dPdx[0] = 1.0f / output_xres * sscale;
-    dPdx[1] = 0;
-    dPdx[2] = 0;
-    dPdy[0] = 0;
-    dPdy[1] = 1.0f / output_yres * tscale;
-    dPdy[2] = 0;
+    dPdx.x = 1.0f / output_xres * sscale;
+    dPdx.y = 0;
+    dPdx.z = 0;
+    dPdy.x = 0;
+    dPdy.y = 1.0f / output_yres * tscale;
+    dPdy.z = 0;
     dPdz.setValue(0, 0, 0);
 }
 
@@ -597,12 +607,16 @@ plain_tex_region(ImageBuf& image, ustring filename, Mapping2D mapping,
             opt.swidth = OIIO::lerp(width, widthramp, s);
             opt.twidth = opt.swidth;
         }
-        if (mipmode == TextureOpt::MipModeStochasticTrilinear
-            || mipmode == TextureOpt::MipModeStochasticAniso) {
+        if (stochastic) {
             // Hash the pixel coords to get a pseudo-random variant
-            constexpr float inv = 1.0f
-                                  / float(std::numeric_limits<uint32_t>::max());
-            opt.rnd = bjhash::bjfinal(p.x(), p.y()) * inv;
+            if (use_bluenoise) {
+                opt.rnd = bluenoiseimg.getchannel(p.x(), p.y(), 0, 0,
+                                                  ImageBuf::WrapPeriodic);
+            } else {
+                constexpr float inv
+                    = 1.0f / float(std::numeric_limits<uint32_t>::max());
+                opt.rnd = bjhash::bjfinal(p.x(), p.y()) * inv;
+            }
         }
 
         // Call the texture system to do the filtering.
@@ -617,7 +631,7 @@ plain_tex_region(ImageBuf& image, ustring filename, Mapping2D mapping,
         if (!ok) {
             std::string e = texsys->geterror();
             if (!e.empty())
-                Strutil::fprintf(std::cerr, "ERROR: %s\n", e);
+                Strutil::print(std::cerr, "ERROR: {}\n", e);
         }
 
         // Save filtered pixels back to the image.
@@ -636,8 +650,8 @@ plain_tex_region(ImageBuf& image, ustring filename, Mapping2D mapping,
 void
 test_plain_texture(Mapping2D mapping)
 {
-    std::cout << "Testing 2d texture " << filenames[0]
-              << ", output = " << output_filename << "\n";
+    Strutil::print("Testing 2d texture {}, output = {}\n", filenames[0],
+                   output_filename);
     const int nchannels = 4;
     ImageSpec outspec(output_xres, output_yres, nchannels, TypeDesc::FLOAT);
     ImageBuf image(outspec);
@@ -661,7 +675,7 @@ test_plain_texture(Mapping2D mapping)
             // Use a different filename for each iteration
             int texid = iter % (int)filenames.size();
             filename  = (filenames[texid]);
-            std::cout << "iter " << iter << " file " << filename << "\n";
+            Strutil::print("iter {} file {}\n", iter, filename);
         }
         if (close_before_iter)
             texsys->close_all();
@@ -672,23 +686,21 @@ test_plain_texture(Mapping2D mapping)
                       test_derivs ? &image_ds : NULL,
                       test_derivs ? &image_dt : NULL, _1));
         if (resetstats) {
-            std::cout << texsys->getstats(2) << "\n";
+            Strutil::print("{}\n", texsys->getstats(2));
             texsys->reset_stats();
         }
     }
 
     if (!image.write(output_filename))
-        Strutil::fprintf(std::cerr, "Error writing %s : %s\n", output_filename,
-                         image.geterror());
+        Strutil::print(std::cerr, "Error writing {} : {}\n", output_filename,
+                       image.geterror());
     if (test_derivs) {
         if (!image_ds.write(output_filename + "-ds.exr"))
-            Strutil::fprintf(std::cerr, "Error writing %s : %s\n",
-                             (output_filename + "-ds.exr"),
-                             image_ds.geterror());
+            Strutil::print(std::cerr, "Error writing {} : {}\n",
+                           (output_filename + "-ds.exr"), image_ds.geterror());
         if (!image_dt.write(output_filename + "-dt.exr"))
-            Strutil::fprintf(std::cerr, "Error writing %s : %s\n",
-                             (output_filename + "-dt.exr"),
-                             image_dt.geterror());
+            Strutil::print(std::cerr, "Error writing {} : {}\n",
+                           (output_filename + "-dt.exr"), image_dt.geterror());
     }
 }
 
@@ -730,14 +742,19 @@ plain_tex_region_batch(ImageBuf& image, ustring filename, Mapping2DWide mapping,
                     opt.twidth[i] = opt.swidth[i];
                 }
             }
-            if (mipmode == TextureOpt::MipModeStochasticTrilinear
-                || mipmode == TextureOpt::MipModeStochasticAniso) {
+            if (stochastic) {
                 // Hash the pixel coords to get a pseudo-random variant
 #if OIIO_VERSION_GREATER_EQUAL(2, 4, 0)
                 constexpr float inv
                     = 1.0f / float(std::numeric_limits<uint32_t>::max());
-                for (int i = 0; i < BatchWidth; ++i)
-                    opt.rnd[i] = bjhash::bjfinal(x + i, y) * inv;
+                for (int i = 0; i < BatchWidth; ++i) {
+                    if (use_bluenoise)
+                        opt.rnd[i]
+                            = bluenoiseimg.getchannel(x + i, y, 0, 0,
+                                                      ImageBuf::WrapPeriodic);
+                    else
+                        opt.rnd[i] = bjhash::bjfinal(x + i, y) * inv;
+                }
 #endif
             }
 
@@ -759,7 +776,7 @@ plain_tex_region_batch(ImageBuf& image, ustring filename, Mapping2DWide mapping,
             if (!ok) {
                 std::string e = texsys->geterror();
                 if (!e.empty())
-                    Strutil::fprintf(std::cerr, "ERROR: %s\n", e);
+                    Strutil::print(std::cerr, "ERROR: {}\n", e);
             }
             // Save filtered pixels back to the image.
             for (int c = 0; c < nchannels; ++c)
@@ -788,8 +805,8 @@ plain_tex_region_batch(ImageBuf& image, ustring filename, Mapping2DWide mapping,
 void
 test_plain_texture_batch(Mapping2DWide mapping)
 {
-    std::cout << "Testing BATCHED 2d texture " << filenames[0]
-              << ", output = " << output_filename << "\n";
+    Strutil::print("Testing BATCHED 2d texture {}, output = {}\n", filenames[0],
+                   output_filename);
     const int nchannels = 4;
     ImageSpec outspec(output_xres, output_yres, nchannels, TypeDesc::FLOAT);
     TypeDesc fmt(dataformatname);
@@ -813,7 +830,7 @@ test_plain_texture_batch(Mapping2DWide mapping)
             // Use a different filename for each iteration
             int texid = iter % (int)filenames.size();
             filename  = (filenames[texid]);
-            std::cout << "iter " << iter << " file " << filename << "\n";
+            Strutil::print("iter {} file {}\n", iter, filename);
         }
         if (close_before_iter)
             texsys->close_all();
@@ -824,23 +841,21 @@ test_plain_texture_batch(Mapping2DWide mapping)
                                        image_dt.get(), roi);
             });
         if (resetstats) {
-            std::cout << texsys->getstats(2) << "\n";
+            Strutil::print("{}\n", texsys->getstats(2));
             texsys->reset_stats();
         }
     }
 
     if (!image.write(output_filename))
-        Strutil::fprintf(std::cerr, "Error writing %s : %s\n", output_filename,
-                         image.geterror());
+        Strutil::print(std::cerr, "Error writing {} : {}\n", output_filename,
+                       image.geterror());
     if (test_derivs) {
         if (!image_ds->write(output_filename + "-ds.exr"))
-            Strutil::fprintf(std::cerr, "Error writing %s : %s\n",
-                             (output_filename + "-ds.exr"),
-                             image_ds->geterror());
+            Strutil::print(std::cerr, "Error writing {} : {}\n",
+                           (output_filename + "-ds.exr"), image_ds->geterror());
         if (!image_dt->write(output_filename + "-dt.exr"))
-            Strutil::fprintf(std::cerr, "Error writing %s : %s\n",
-                             (output_filename + "-dt.exr"),
-                             image_dt->geterror());
+            Strutil::print(std::cerr, "Error writing {} : {}\n",
+                           (output_filename + "-dt.exr"), image_dt->geterror());
     }
 }
 
@@ -874,7 +889,7 @@ tex3d_region(ImageBuf& image, ustring filename, Mapping3D mapping, ROI roi)
         if (!ok) {
             std::string e = texsys->geterror();
             if (!e.empty())
-                Strutil::fprintf(std::cerr, "ERROR: %s\n", e);
+                Strutil::print(std::cerr, "ERROR: {}\n", e);
         }
 
         // Save filtered pixels back to the image.
@@ -918,7 +933,7 @@ tex3d_region_batch(ImageBuf& image, ustring filename, Mapping3DWide mapping,
 
             // Call the texture system to do the filtering.
             if (y == 0 && x == 0)
-                std::cout << "P = " << P << "\n";
+                Strutil::print("P = {}\n", P);
             bool ok = texsys->texture3d(texture_handle, perthread_info, opt,
                                         mask, (float*)&P, (float*)&dPdx,
                                         (float*)&dPdy, (float*)&dPdz, nchannels,
@@ -927,7 +942,7 @@ tex3d_region_batch(ImageBuf& image, ustring filename, Mapping3DWide mapping,
             if (!ok) {
                 std::string e = texsys->geterror();
                 if (!e.empty())
-                    Strutil::fprintf(std::cerr, "ERROR: %s\n", e);
+                    Strutil::print(std::cerr, "ERROR: {}\n", e);
             }
 
             // Save filtered pixels back to the image.
@@ -947,8 +962,8 @@ tex3d_region_batch(ImageBuf& image, ustring filename, Mapping3DWide mapping,
 void
 test_texture3d(ustring filename, Mapping3D mapping)
 {
-    std::cout << "Testing 3d texture " << filename
-              << ", output = " << output_filename << "\n";
+    Strutil::print("Testing 3d texture {}, output = {}\n", filename,
+                   output_filename);
     int nchannels = nchannels_override ? nchannels_override : 4;
     ImageSpec outspec(output_xres, output_yres, nchannels, TypeDesc::FLOAT);
     ImageBuf image(outspec);
@@ -968,8 +983,8 @@ test_texture3d(ustring filename, Mapping3D mapping)
     }
 
     if (!image.write(output_filename))
-        Strutil::fprintf(std::cerr, "Error writing %s : %s\n", output_filename,
-                         image.geterror());
+        Strutil::print(std::cerr, "Error writing {} : {}\n", output_filename,
+                       image.geterror());
 }
 
 
@@ -977,8 +992,8 @@ test_texture3d(ustring filename, Mapping3D mapping)
 void
 test_texture3d_batch(ustring filename, Mapping3DWide mapping)
 {
-    std::cout << "Testing 3d texture " << filename
-              << ", output = " << output_filename << "\n";
+    Strutil::print("Testing 3d texture {}, output = {}\n", filename,
+                   output_filename);
     int nchannels = nchannels_override ? nchannels_override : 4;
     ImageSpec outspec(output_xres, output_yres, nchannels, TypeDesc::FLOAT);
     ImageBuf image(outspec);
@@ -1000,8 +1015,8 @@ test_texture3d_batch(ustring filename, Mapping3DWide mapping)
     }
 
     if (!image.write(output_filename))
-        Strutil::fprintf(std::cerr, "Error writing %s : %s\n", output_filename,
-                         image.geterror());
+        Strutil::print(std::cerr, "Error writing {} : {}\n", output_filename,
+                       image.geterror());
 }
 
 
@@ -1043,9 +1058,9 @@ test_getimagespec_gettexels(ustring filename)
     for (int i = 0; i < iters; ++i) {
         bool ok = texsys->get_texels(filename, opt, miplevel, x, x + w, y,
                                      y + h, 0, 1, 0, nchannels,
-                                     postagespec.format, &tmp[0]);
+                                     postagespec.format, tmp.data());
         if (!ok)
-            Strutil::fprintf(std::cerr, "ERROR: %s\n", texsys->geterror());
+            Strutil::print(std::cerr, "ERROR: {}\n", texsys->geterror());
     }
     for (int y = 0; y < h; ++y)
         for (int x = 0; x < w; ++x) {
@@ -1072,8 +1087,8 @@ test_hash()
     const int res      = 4 * 1024;  // Simulate tiles from a 4k image
     const int tilesize = 64;
     const int nfiles   = iters / ((res / tilesize) * (res / tilesize));
-    std::cout << "Testing hashing with " << nfiles << " files of " << res << 'x'
-              << res << " with " << tilesize << 'x' << tilesize << " tiles:\n";
+    Strutil::print("Testing hashing with {} files of {}x{} with {}x{} tiles:",
+                   nfiles, res, res, tilesize, tilesize);
 
     ImageCache* imagecache = ImageCache::create();
 
@@ -1083,7 +1098,7 @@ test_hash()
     using OIIO::pvt::ImageCacheImpl;
     std::vector<ImageCacheFileRef> icf;
     for (int f = 0; f < nfiles; ++f) {
-        ustring filename = ustring::sprintf("%06d.tif", f);
+        ustring filename = ustring::fmtformat("{:06}.tif", f);
         icf.push_back(
             new ImageCacheFile(*(ImageCacheImpl*)imagecache, NULL, filename));
     }
@@ -1100,11 +1115,10 @@ test_hash()
             }
         }
     }
-    std::cout << "hh = " << hh << "\n";
+    Strutil::print("hh = {}\n", hh);
     double time = timer();
     double rate = (i / 1.0e6) / time;
-    std::cout << "Hashing rate: " << Strutil::sprintf("%3.2f", rate)
-              << " Mhashes/sec\n";
+    Strutil::print("Hashing rate:` {:3.2f} Mhashes/sec\n", rate);
 
     // Now, check the quality of the hash by looking at the low 4, 8, and
     // 16 bits and making sure that they divide into hash buckets fairly
@@ -1119,7 +1133,7 @@ test_hash()
                 ++eightbits[h & 0xff];
                 ++highereightbits[(h >> 24) & 0xff];
                 ++sixteenbits[h & 0xffff];
-                // if (i < 16) std::cout << Strutil::sprintf("%llx\n", h);
+                // if (i < 16) Strutil::print({:x}\n", h);
             }
         }
     }
@@ -1133,8 +1147,7 @@ test_hash()
         if (fourbits[i] > max)
             max = fourbits[i];
     }
-    std::cout << "4-bit hash buckets range from " << min << " to " << max
-              << "\n";
+    Strutil::print("4-bit hash buckets range from {} to {}\n", min, max);
 
     min = std::numeric_limits<size_t>::max();
     max = 0;
@@ -1144,8 +1157,7 @@ test_hash()
         if (eightbits[i] > max)
             max = eightbits[i];
     }
-    std::cout << "8-bit hash buckets range from " << min << " to " << max
-              << "\n";
+    Strutil::print("8-bit hash buckets range from {} to {}\n", min, max);
 
     min = std::numeric_limits<size_t>::max();
     max = 0;
@@ -1155,8 +1167,7 @@ test_hash()
         if (highereightbits[i] > max)
             max = highereightbits[i];
     }
-    std::cout << "higher 8-bit hash buckets range from " << min << " to " << max
-              << "\n";
+    Strutil::print("higher 8-bit hash buckets range from {} to {}\n", min, max);
 
     min = std::numeric_limits<size_t>::max();
     max = 0;
@@ -1166,10 +1177,9 @@ test_hash()
         if (sixteenbits[i] > max)
             max = sixteenbits[i];
     }
-    std::cout << "16-bit hash buckets range from " << min << " to " << max
-              << "\n";
+    Strutil::print("16-bit hash buckets range from {} to {}\n", min, max);
 
-    std::cout << "\n";
+    Strutil::print("\n");
 
     ImageCache::destroy(imagecache);
 }
@@ -1291,7 +1301,7 @@ do_tex_thread_workout(int iterations, int mythread)
             }
             break;
         default:
-            OIIO_ASSERT_MSG(0, "Unkonwn thread work pattern %d", threadtimes);
+            OIIO_ASSERT_MSG(0, "Unknown thread work pattern %d", threadtimes);
         }
         if (!ok && spec0.width && spec0.height) {
             s = (((2 * pixel) % spec0.width) + 0.5f) / spec0.width;
@@ -1311,8 +1321,8 @@ do_tex_thread_workout(int iterations, int mythread)
                                      dresultds, dresultdt);
         }
         if (!ok) {
-            Strutil::fprintf(std::cerr, "Unexpected error: %s\n",
-                             texsys->geterror());
+            Strutil::print(std::cerr, "Unexpected error: {}\n",
+                           texsys->geterror());
             return;
         }
         // Do some pointless work, to simulate that in a real app, there
@@ -1353,21 +1363,21 @@ public:
         : m_miplevel(-1)
     {
     }
-    virtual ~GridImageInput() { close(); }
-    virtual const char* format_name(void) const final { return "grid"; }
-    virtual bool valid_file(const std::string& /*filename*/) const final
+    ~GridImageInput() override { close(); }
+    const char* format_name(void) const final { return "grid"; }
+    bool valid_file(const std::string& /*filename*/) const final
     {
         return true;
     }
-    virtual bool open(const std::string& /*name*/, ImageSpec& newspec) final
+    bool open(const std::string& /*name*/, ImageSpec& newspec) final
     {
         bool ok = seek_subimage(0, 0);
         newspec = spec();
         return ok;
     }
-    virtual bool close() { return true; }
-    virtual int current_miplevel(void) const final { return m_miplevel; }
-    virtual bool seek_subimage(int subimage, int miplevel) final
+    bool close() override { return true; }
+    int current_miplevel(void) const final { return m_miplevel; }
+    bool seek_subimage(int subimage, int miplevel) final
     {
         if (subimage > 0)
             return false;
@@ -1386,14 +1396,13 @@ public:
         m_miplevel         = miplevel;
         return true;
     }
-    virtual bool read_native_scanline(int /*subimage*/, int /*miplevel*/,
-                                      int /*y*/, int /*z*/,
-                                      void* /*data*/) final
+    bool read_native_scanline(int /*subimage*/, int /*miplevel*/, int /*y*/,
+                              int /*z*/, void* /*data*/) final
     {
         return false;
     }
-    virtual bool read_native_tile(int subimage, int miplevel, int xbegin,
-                                  int ybegin, int zbegin, void* data) final
+    bool read_native_tile(int subimage, int miplevel, int xbegin, int ybegin,
+                          int zbegin, void* data) final
     {
         lock_guard lock(*this);
         if (!seek_subimage(subimage, miplevel))
@@ -1430,13 +1439,13 @@ make_grid_input()
 void
 test_icwrite(int testicwrite)
 {
-    std::cout << "Testing IC write, mode " << testicwrite << "\n";
+    Strutil::print("Testing IC write, mode {}\n", testicwrite);
 
     // The global "shared" ImageCache will be the same one the
     // TextureSystem uses.
     ImageCache* ic = ImageCache::create();
 
-    // Set up the fake file ane add it
+    // Set up the fake file and add it
     int tw = 64, th = 64;  // tile width and height
     int nc = nchannels_override ? nchannels_override : 3;  // channels
     ImageSpec spec(512, 512, nc, TypeDesc::FLOAT);
@@ -1447,7 +1456,7 @@ test_icwrite(int testicwrite)
     ustring filename(filenames[0]);
     bool ok = ic->add_file(filename, make_grid_input);
     if (!ok) {
-        std::cout << "ic->add_file error: " << ic->geterror() << "\n";
+        Strutil::print("ic->add_file error: {}\n", ic->geterror());
         OIIO_ASSERT(ok);
     }
 
@@ -1473,8 +1482,8 @@ test_icwrite(int testicwrite)
                 bool ok = ic->add_tile(filename, 0, 0, tx, ty, 0, 0, -1,
                                        TypeDesc::FLOAT, &tile[0]);
                 if (!ok) {
-                    Strutil::fprintf(std::cerr, "ic->add_tile error: %s\n",
-                                     ic->geterror());
+                    Strutil::print(std::cerr, "ic->add_tile error: {}\n",
+                                   ic->geterror());
                     return;
                 }
             }
@@ -1584,7 +1593,7 @@ main(int argc, const char* argv[])
     OIIO::attribute("threads", nthreads);
 
     texsys = TextureSystem::create();
-    std::cout << "Created texture system\n";
+    Strutil::print("Created texture system\n");
     if (texoptions.size())
         texsys->attribute("options", texoptions);
     texsys->attribute("autotile", autotile);
@@ -1604,6 +1613,7 @@ main(int argc, const char* argv[])
         texsys->attribute("accept_unmipped", 0);
     texsys->attribute("gray_to_rgb", gray_to_rgb);
     texsys->attribute("flip_t", flip_t);
+    texsys->attribute("stochastic", stochastic);
 
     if (test_construction) {
         Timer t;
@@ -1611,7 +1621,7 @@ main(int argc, const char* argv[])
             TextureOpt opt;
             dummyptr = &opt;  // This forces the optimizer to keep the loop
         }
-        std::cout << "TextureOpt construction: " << t() << " ns\n";
+        Strutil::print("TextureOpt construction: {} ns\n", t());
         TextureOpt canonical, copy;
         t.reset();
         t.start();
@@ -1619,7 +1629,7 @@ main(int argc, const char* argv[])
             copy     = canonical;
             dummyptr = &copy;  // This forces the optimizer to keep the loop
         }
-        std::cout << "TextureOpt copy: " << t() << " ns\n";
+        Strutil::print("TextureOpt copy: {} ns\n", t());
     }
 
     if (maketest_template.size()
@@ -1684,12 +1694,12 @@ main(int argc, const char* argv[])
         // (divided among the threads). If not supplied (iters will be 1),
         // then use a large constant *per thread*.
         const int iterations = iters > 1 ? iters : 2000000;
-        std::cout << "Workload: " << workload_names[threadtimes] << "\n";
-        std::cout << "texture cache size = " << cachesize << " MB\n";
-        std::cout << "hw threads = " << Sysutil::hardware_concurrency() << "\n";
-        std::cout << "times are best of " << ntrials << " trials\n\n";
-        std::cout << "threads  time (s)   speedup efficiency\n";
-        std::cout << "-------- -------- --------- ----------\n";
+        Strutil::print("Workload: {}\n", workload_names[threadtimes]);
+        Strutil::print("texture cache size = {} MB\n", cachesize);
+        Strutil::print("hw threads = {}\n", Sysutil::hardware_concurrency());
+        Strutil::print("times are best of {} trials\n\n", ntrials);
+        Strutil::print("threads  time (s)   speedup efficiency\n");
+        Strutil::print("-------- -------- --------- ----------\n");
 
         if (nthreads == 0)
             nthreads = Sysutil::hardware_concurrency();
@@ -1697,25 +1707,26 @@ main(int argc, const char* argv[])
                                       24, 32, 64, 128, 1024, 1 << 30 };
         float single_thread_time  = 0.0f;
         for (int i = 0; threadcounts[i] <= nthreads; ++i) {
-            int nt  = wedge ? threadcounts[i] : nthreads;
-            int its = iters > 1 ? (std::max(1, iters / nt))
-                                : iterations;  // / nt;
+            if (threadcounts[i] < minthreads)
+                continue;
+            int nt    = wedge ? threadcounts[i] : nthreads;
+            int its   = iters > 1 ? (std::max(1, iters / nt)) : iterations;
+            int tries = nt <= 2 ? std::min(lowtrials, ntrials) : ntrials;
             double range;
-            double t = time_trial(std::bind(launch_tex_threads, nt, its),
-                                  ntrials, &range);
-            if (nt == 1)
-                single_thread_time = (float)t;
-            float speedup    = (single_thread_time /*/nt*/) / (float)t;
-            float efficiency = (single_thread_time / nt) / float(t);
-            std::cout << Strutil::sprintf(
-                "%3d     %8.2f   %6.1fx  %6.1f%%    range %.2f\t(%d iters/thread)\n",
+            float t = (float)time_trial(std::bind(launch_tex_threads, nt, its),
+                                        tries, &range);
+            if (single_thread_time == 0.0f)
+                single_thread_time = t * nt;
+            float speedup    = single_thread_time / t;
+            float efficiency = speedup / nt;
+            Strutil::print(
+                "{:3}     {:8.2f}   {:6.1f}x  {:6.1f}%    range {:.2f}\t({} iters/thread)\n",
                 nt, t, speedup, efficiency * 100.0f, range, its);
-            std::cout.flush();
+            fflush(stdout);
             if (!wedge)
                 break;  // don't loop if we're not wedging
         }
-        std::cout << "\n";
-
+        Strutil::print("\n");
     } else if (iters > 0 && filenames.size()) {
         ustring filename(filenames[0]);
         test_gettextureinfo(filenames[0]);
@@ -1811,15 +1822,10 @@ main(int argc, const char* argv[])
     if (runstats || verbose) {
         Strutil::print("Memory use: {}\n",
                        Strutil::memformat(Sysutil::memory_used(true)));
-        Strutil::print("{}\n", texsys->getstats(verbose ? 2 : 0));
+        Strutil::print("{}\n", texsys->getstats(verbose ? 2 : 1));
     }
     TextureSystem::destroy(texsys);
 
-    // Force all files to close, ugh, it's the only way I can find to solve
-    // an occasional problem with static destructor order fiasco with
-    // field3d when building with EMBEDPLUGINS=0 on MacOS.
-    ImageCache::create()->close_all();
-
     if (verbose)
         Strutil::print("\nustrings: {}\n\n", ustring::getstats(false));
 
