@@ -11,14 +11,11 @@ files = [ "g01bg.bmp", "g01bw.bmp", "g01p1.bmp", "g01wb.bmp",
           "g08res11.bmp", "g08res21.bmp", "g08res22.bmp",
           "g08s0.bmp", "g08w124.bmp", "g08w125.bmp", "g08w126.bmp",
           "g08rle.bmp", "g08offs.bmp",
-          "g24.bmp", "g32bf.bmp", "g32def.bmp", "g32bf.bmp"
-           ]
+          "g24.bmp", "g32bf.bmp", "g32def.bmp",
+          "g16bf555.bmp", "g16bf565.bmp", "g16def555.bmp" ]
 for f in files :
     command += rw_command (OIIO_TESTSUITE_IMAGEDIR, f)
 
-# TODO: seems broken: g16bf555.bmp,
-#                     g16bf565.bmp, g16def555.bmp
-
 # Test BMR version 5
 command += rw_command ("src", "g01bg2-v5.bmp")
 
@@ -29,5 +26,6 @@ command += rw_command ("src", "PRINTER.BMP")
 # Test BMP of the 56-byte DIB header variety
 command += rw_command ("../oiio-images/bmp", "gracehopper.bmp")
 
-# See if we handle this corrupt file with a useful error message
+# See if we handle these corrupt files with useful error messages
 command += info_command ("src/decodecolormap-corrupt.bmp")
+command += info_command ("src/bad-y.bmp")
