@@ -44,6 +44,7 @@ OIIO_BUILD_ROOT = path
 tmpdir = "."
 tmpdir = os.path.abspath (tmpdir)
 redirect = " >> out.txt "
+wrapper_cmd = ""
 
 def make_relpath (path, start=os.curdir):
     "Wrapper around os.path.relpath which always uses '/' as the separator."
@@ -78,9 +79,34 @@ if str(mytest).endswith('.batch') :
     mytest = mytest.split('.')[0]
 test_source_dir = os.getenv('OIIO_TESTSUITE_SRC',
                             os.path.join(OIIO_TESTSUITE_ROOT, mytest))
-colorconfig_file = os.getenv('OIIO_TESTSUITE_OCIOCONFIG',
-                             '../common/OpenColorIO/nuke-default/config.ocio')
-colorconfig_file = make_relpath(colorconfig_file)
+
+def oiio_app (app):
+    if (platform.system () != 'Windows' or options.devenv_config == ""):
+        cmd = os.path.join(OIIO_BUILD_ROOT, "bin", app) + " "
+    else:
+        cmd = os.path.join(OIIO_BUILD_ROOT, "bin", options.devenv_config, app) + " "
+    if wrapper_cmd != "":
+        cmd = wrapper_cmd + " " + cmd
+    return cmd
+
+
+# Ask oiiotool what version of OpenColorIO it has emedded
+ociover = subprocess.check_output([oiio_app('oiiotool').strip(),
+                                   '--echo', '{getattribute(opencolorio_version)}'])
+ociover = ociover.strip().decode('utf-8')[0:3]
+ociover = os.getenv('OCIO_VERSION_OVERRIDE', ociover)
+#print(f"OpenColorIO version = '{ociover}'")
+
+OCIO_env = os.getenv('OCIO')
+if OCIO_env is None:
+    os.environ['OCIO'] = 'ocio:://default'
+    if ociover == '2.2' or ociover == '2.3' :
+        colorconfig_file = os.getenv('OIIO_TESTSUITE_OCIOCONFIG', 'ocio://default')
+    else :
+        colorconfig_file = os.getenv('OIIO_TESTSUITE_OCIOCONFIG',
+                                     '../common/OpenColorIO/nuke-default/config.ocio')
+    if not colorconfig_file.startswith('ocio://') :
+        colorconfig_file = make_relpath(colorconfig_file)
 
 
 command = ""
@@ -92,6 +118,7 @@ outputs = [ "out.txt" ]    # default
 failthresh = 0.004         # "Failure" threshold for any pixel value
 failpercent = 0.02         # Ok fo this percentage of pixels to "fail"
 hardfail = 0.012           # Even one pixel this wrong => hard failure
+allowfailures = 0          # Freebie failures
 
 # Some tests are designed for the app running to "fail" (in the sense of
 # terminating with an error return code), for example, a test that is designed
@@ -199,27 +226,31 @@ def run_app(app, silent=False, concat=True):
         command += " ;\n"
     return command
 
-def oiio_app (app):
-    if (platform.system () != 'Windows' or options.devenv_config == ""):
-        return os.path.join(OIIO_BUILD_ROOT, "bin", app) + " "
-    else:
-        return os.path.join(OIIO_BUILD_ROOT, "bin", options.devenv_config, app) + " "
-
 
 # Construct a command that will print info for an image, appending output to
 # the file "out.txt".  If 'safematch' is nonzero, it will exclude printing
 # of fields that tend to change from run to run or release to release.
 def info_command (file, extraargs="", safematch=False, hash=True,
-                  verbose=True) :
-    args = "--info"
+                  verbose=True, silent=False, concat=True, failureok=False,
+                  info_program="oiiotool") :
+    args = ""
+    if info_program == "oiiotool" :
+        args += " --info"
     if verbose :
         args += " -v -a"
     if safematch :
         args += " --no-metamatch \"DateTime|Software|OriginatingProgram|ImageHistory\""
     if hash :
         args += " --hash"
-    return (oiio_app("oiiotool") + args + " " + extraargs
-            + " " + make_relpath(file,tmpdir) + redirect + ";\n")
+    cmd = (oiio_app(info_program) + args + " " + extraargs
+            + " " + make_relpath(file,tmpdir))
+    if not silent :
+        cmd += redirect
+    if failureok :
+        cmd += " || true "
+    if concat:
+        cmd += " ;\n"
+    return cmd
 
 
 # Construct a command that will compare two images, appending output to
@@ -231,6 +262,7 @@ def diff_command (fileA, fileB, extraargs="", silent=False, concat=True) :
                + " -fail " + str(failthresh)
                + " -failpercent " + str(failpercent)
                + " -hardfail " + str(hardfail)
+               + " -allowfailures " + str(allowfailures)
                + " -warn " + str(2*failthresh)
                + " -warnpercent " + str(failpercent)
                + " " + extraargs + " " + make_relpath(fileA,tmpdir)
@@ -288,6 +320,7 @@ def rw_command (dir, filename, testwrite=True, use_oiiotool=False, extraargs="",
                + " -fail " + str(failthresh)
                + " -failpercent " + str(failpercent)
                + " -hardfail " + str(hardfail)
+               + " -allowfailures " + str(allowfailures)
                + " -warn " + str(2*failthresh)
                + " " + idiffextraargs + " " + output_filename + redirect + ";\n")
     return cmd
@@ -318,8 +351,8 @@ def iconvert (args, silent=False, concat=True, failureok=False) :
 # Construct a command that will run oiiotool and append its output to out.txt
 def oiiotool (args, silent=False, concat=True, failureok=False) :
     cmd = (oiio_app("oiiotool") + " "
-           + "-colorconfig " + colorconfig_file + " "
            + args)
+    #       + "-colorconfig " + colorconfig_file + " "
     if not silent :
         cmd += redirect
     if failureok :
